@@ -1,19 +1,31 @@
 /*******************************************************************************
- * Copyright (c) 2012, 2013 SAP SE and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2012, 2022 SAP SE and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Tobias Oberlies (SAP SE) - initial API and implementation
+ *    Christoph Läubrich    - Issue #658 - Tycho strips p2 artifact properties (eg PGP, maven info...)
+ *                          - Issue #692 - Check Hashsums for local cached artifacts 
  *******************************************************************************/
 package org.eclipse.tycho.repository.local;
 
 import static org.eclipse.tycho.repository.util.internal.BundleConstants.BUNDLE_ID;
 
 import java.io.File;
+import java.io.FileInputStream;
+import java.io.IOException;
+import java.util.LinkedHashMap;
+import java.util.Map;
+import java.util.Objects;
+import java.util.TreeSet;
+import java.util.concurrent.locks.Lock;
 
+import org.apache.commons.codec.digest.DigestUtils;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.NullProgressMonitor;
@@ -25,6 +37,9 @@ import org.eclipse.equinox.p2.query.IQuery;
 import org.eclipse.equinox.p2.query.IQueryResult;
 import org.eclipse.equinox.p2.query.IQueryable;
 import org.eclipse.equinox.p2.repository.artifact.IArtifactDescriptor;
+import org.eclipse.tycho.PackagingType;
+import org.eclipse.tycho.TychoConstants;
+import org.eclipse.tycho.core.shared.MavenContext;
 import org.eclipse.tycho.core.shared.MavenLogger;
 import org.eclipse.tycho.core.shared.MultiLineLogger;
 import org.eclipse.tycho.repository.p2base.artifact.provider.IRawArtifactFileProvider;
@@ -63,6 +78,7 @@ public class MirroringArtifactProvider implements IRawArtifactFileProvider {
     protected final LocalArtifactRepository localArtifactRepository;
 
     protected final IProgressMonitor monitor;
+    private MavenContext mavenContext;
 
     /**
      * Creates a new {@link MirroringArtifactProvider} instance.
@@ -80,21 +96,22 @@ public class MirroringArtifactProvider implements IRawArtifactFileProvider {
      *            a logger for progress output
      */
     public static MirroringArtifactProvider createInstance(LocalArtifactRepository localArtifactRepository,
-            IRawArtifactProvider remoteProviders, boolean mirrorPacked, MavenLogger logger) {
+            IRawArtifactProvider remoteProviders, boolean mirrorPacked, MavenContext mavenContext) {
         if (!mirrorPacked) {
-            return new MirroringArtifactProvider(localArtifactRepository, remoteProviders, logger);
+            return new MirroringArtifactProvider(localArtifactRepository, remoteProviders, mavenContext);
         } else {
-            return new PackedFormatMirroringArtifactProvider(localArtifactRepository, remoteProviders, logger);
+            return new PackedFormatMirroringArtifactProvider(localArtifactRepository, remoteProviders, mavenContext);
         }
     }
 
     MirroringArtifactProvider(LocalArtifactRepository localArtifactRepository, IRawArtifactProvider remoteProviders,
-            MavenLogger logger) {
+            MavenContext mavenContext) {
         this.remoteProviders = remoteProviders;
         this.localArtifactRepository = localArtifactRepository;
-        this.logger = logger;
-        this.splittingLogger = new MultiLineLogger(logger);
-        this.monitor = new LoggingProgressMonitor(logger);
+        this.mavenContext = mavenContext;
+        this.logger = mavenContext.getLogger();
+        this.splittingLogger = new MultiLineLogger(this.logger);
+        this.monitor = new LoggingProgressMonitor(this.logger);
     }
 
     // pass through methods
@@ -133,8 +150,8 @@ public class MirroringArtifactProvider implements IRawArtifactFileProvider {
     }
 
     @Override
-    public final IStatus getArtifact(IArtifactSink sink, IProgressMonitor monitor) throws ArtifactSinkException,
-            MirroringFailedException {
+    public final IStatus getArtifact(IArtifactSink sink, IProgressMonitor monitor)
+            throws ArtifactSinkException, MirroringFailedException {
         IArtifactKey requestedKey = sink.getArtifactToBeWritten();
         if (makeLocallyAvailable(requestedKey)) {
             return localArtifactRepository.getArtifact(sink, monitor);
@@ -143,8 +160,8 @@ public class MirroringArtifactProvider implements IRawArtifactFileProvider {
     }
 
     @Override
-    public final IStatus getRawArtifact(IRawArtifactSink sink, IProgressMonitor monitor) throws ArtifactSinkException,
-            MirroringFailedException {
+    public final IStatus getRawArtifact(IRawArtifactSink sink, IProgressMonitor monitor)
+            throws ArtifactSinkException, MirroringFailedException {
         IArtifactKey requestedKey = sink.getArtifactToBeWritten();
         if (makeLocallyAvailable(requestedKey)) {
             return localArtifactRepository.getRawArtifact(sink, monitor);
@@ -194,29 +211,82 @@ public class MirroringArtifactProvider implements IRawArtifactFileProvider {
             return isAvailable;
 
         } catch (ProvisionException e) {
-            throw new MirroringFailedException("Error while mirroring artifact " + key
-                    + " to the local Maven repository" + e.getMessage(), e);
+            throw new MirroringFailedException(
+                    "Error while mirroring artifact " + key + " to the local Maven repository" + e.getMessage(), e);
         } catch (ArtifactSinkException e) {
-            throw new MirroringFailedException("Error while mirroring artifact " + key
-                    + " to the local Maven repository" + e.getMessage(), e);
+            throw new MirroringFailedException(
+                    "Error while mirroring artifact " + key + " to the local Maven repository" + e.getMessage(), e);
         }
     }
 
-    protected boolean makeOneFormatLocallyAvailable(IArtifactKey key) throws MirroringFailedException,
-            ProvisionException, ArtifactSinkException {
+    protected boolean makeOneFormatLocallyAvailable(IArtifactKey key)
+            throws MirroringFailedException, ProvisionException, ArtifactSinkException {
 
-        if (localArtifactRepository.contains(key)) {
+        if (isFileAlreadyAvailable(key)) {
             return true;
         } else if (remoteProviders.contains(key)) {
-            downloadArtifact(key);
+            Lock downloadLock = localArtifactRepository.getLockForDownload(key);
+            downloadLock.lock();
+            try {
+                if (!isFileAlreadyAvailable(key)) { // check again within lock
+                    File artifactFile;
+                    if (localArtifactRepository.contains(key)) {
+                        artifactFile = localArtifactRepository.getArtifactFile(key);
+                        localArtifactRepository.removeDescriptor(key);
+                    } else {
+                        artifactFile = localArtifactRepository.internalGetArtifactStorageLocation(
+                                localArtifactRepository.createArtifactDescriptor(key));
+                    }
+                    if (artifactFile != null && artifactFile.isFile()) {
+                        //check if only properties has changed...
+                        GAVArtifactDescriptor descriptor = newLocalDescriptor(key);
+                        if (fileMatchesProperties(artifactFile, descriptor.getProperties())) {
+                            localArtifactRepository.internalAddDescriptor(descriptor);
+                            localArtifactRepository.save();
+                            return true;
+                        }
+                    }
+                    downloadArtifact(key);
+                    localArtifactRepository.save();
+                }
+            } finally {
+                downloadLock.unlock();
+            }
             return true;
         } else {
             return false;
         }
     }
 
-    protected final void downloadArtifact(IArtifactKey key) throws MirroringFailedException, ProvisionException,
-            ArtifactSinkException {
+    private boolean fileMatchesProperties(File file, Map<String, String> properties) {
+        String downloadSize = properties.get("download.size");
+        if (downloadSize != null && !downloadSize.equals(String.valueOf(file.length()))) {
+            //early break out...
+            return false;
+        }
+        String sha256 = properties.get("download.checksum.sha-256");
+        if (sha256 != null) {
+            try (FileInputStream stream = new FileInputStream(file)) {
+                String fileSha256 = DigestUtils.sha256Hex(stream);
+                return fileSha256.equalsIgnoreCase(sha256);
+            } catch (IOException e) {
+                return false;
+            }
+        }
+        String md5 = properties.get("download.checksum.md5");
+        if (md5 != null) {
+            try (FileInputStream stream = new FileInputStream(file)) {
+                String fileMd5 = DigestUtils.md5Hex(stream);
+                return fileMd5.equalsIgnoreCase(md5);
+            } catch (IOException e) {
+                return false;
+            }
+        }
+        return false;
+    }
+
+    protected final void downloadArtifact(IArtifactKey key)
+            throws MirroringFailedException, ProvisionException, ArtifactSinkException {
 
 //        logger.info("Downloading " + key.getId() + "_" + key.getVersion() + "..."); // p2 output is enough
         IStatus transferStatus = downloadMostSpecificNeededFormatOfArtifact(key);
@@ -231,37 +301,52 @@ public class MirroringArtifactProvider implements IRawArtifactFileProvider {
 
     }
 
-    protected IStatus downloadMostSpecificNeededFormatOfArtifact(IArtifactKey key) throws ProvisionException,
-            ArtifactSinkException {
+    protected IStatus downloadMostSpecificNeededFormatOfArtifact(IArtifactKey key)
+            throws ProvisionException, ArtifactSinkException {
         // only need canonical format
         return downloadCanonicalArtifact(key);
     }
 
-    protected final IStatus downloadCanonicalArtifact(IArtifactKey key) throws ProvisionException,
-            ArtifactSinkException {
-        // TODO 397355 ignore ProvisionException.ARTIFACT_EXISTS - artifact may have been added by other thread in the meantime
-        IArtifactSink localSink = localArtifactRepository.newAddingArtifactSink(key);
+    protected final IStatus downloadCanonicalArtifact(IArtifactKey key)
+            throws ProvisionException, ArtifactSinkException {
+        GAVArtifactDescriptor localDescriptor = newLocalDescriptor(key);
+        IArtifactSink localSink = localArtifactRepository.newAddingArtifactSink(localDescriptor);
         return remoteProviders.getArtifact(localSink, monitorForDownload());
     }
 
-    private void ensureArtifactIsPresentInCanonicalFormat(IArtifactKey key) throws ProvisionException,
-            ArtifactSinkException {
+    protected GAVArtifactDescriptor newLocalDescriptor(IArtifactKey key) {
+        GAVArtifactDescriptor localDescriptor = localArtifactRepository.createArtifactDescriptor(key);
+        IArtifactDescriptor remoteDescriptor = findCanonicalDescriptor(remoteProviders.getArtifactDescriptors(key));
+        Map<String, String> properties = getProperties(remoteDescriptor);
+        properties.forEach(localDescriptor::setProperty);
+        return localDescriptor;
+    }
 
+    private void ensureArtifactIsPresentInCanonicalFormat(IArtifactKey key)
+            throws ProvisionException, ArtifactSinkException {
         if (findCanonicalDescriptor(localArtifactRepository.getArtifactDescriptors(key)) == null) {
-            // there was at least format available, but not the canonical format -> create it from the packed format 
-            createCanonicalArtifactFromLocalPackedArtifact(key);
+            boolean isPack200able = Runtime.version().feature() <= 13;
+            if (isPack200able) {
+                // there was at least format available, but not the canonical format -> create it from the packed format 
+                createCanonicalArtifactFromLocalPackedArtifact(key);
+            } else {
+                downloadCanonicalArtifact(key);
+            }
         }
     }
 
-    private void createCanonicalArtifactFromLocalPackedArtifact(IArtifactKey key) throws ProvisionException,
-            ArtifactSinkException {
+    private void createCanonicalArtifactFromLocalPackedArtifact(IArtifactKey key)
+            throws ProvisionException, ArtifactSinkException {
         logger.info("Unpacking " + key.getId() + "_" + key.getVersion() + "...");
 
         // TODO 397355 ignore ProvisionException.ARTIFACT_EXISTS
-        IArtifactSink sink = localArtifactRepository.newAddingArtifactSink(key);
+
+        IArtifactSink sink = localArtifactRepository
+                .newAddingArtifactSink(localArtifactRepository.createArtifactDescriptor(key));
         localArtifactRepository.getArtifact(sink, monitor);
     }
 
+    @Deprecated(forRemoval = true)
     static IArtifactDescriptor findPackedDescriptor(IArtifactDescriptor[] descriptors) {
         for (IArtifactDescriptor descriptor : descriptors) {
             if (ArtifactTransferPolicy.isPack200Format(descriptor)) {
@@ -282,7 +367,8 @@ public class MirroringArtifactProvider implements IRawArtifactFileProvider {
 
     private static IStatus artifactNotFoundStatus(IArtifactKey key) {
         return new Status(IStatus.ERROR, BUNDLE_ID, ProvisionException.ARTIFACT_NOT_FOUND, "Artifact " + key
-                + " is neither available in the local Maven repository nor in the configured remote repositories", null);
+                + " is neither available in the local Maven repository nor in the configured remote repositories",
+                null);
     }
 
     /**
@@ -309,4 +395,62 @@ public class MirroringArtifactProvider implements IRawArtifactFileProvider {
         }
 
     }
+
+    @Override
+    public boolean isFileAlreadyAvailable(IArtifactKey artifactKey) {
+        if (localArtifactRepository.contains(artifactKey)) {
+            if (mavenContext.isOffline()) {
+                return true;
+            }
+            if (remoteProviders.contains(artifactKey)) {
+                //we must compare remote versus local!
+                IArtifactDescriptor[] remoteDescriptors = remoteProviders.getArtifactDescriptors(artifactKey);
+                IArtifactDescriptor remoteDescriptor = findCanonicalDescriptor(remoteDescriptors);
+                IArtifactDescriptor[] localDescriptors = localArtifactRepository.getArtifactDescriptors(artifactKey);
+                GAVArtifactDescriptor localDescriptor = (GAVArtifactDescriptor) findCanonicalDescriptor(
+                        localDescriptors);
+                if (remoteDescriptor != null && localDescriptor != null && localDescriptor.getMavenCoordinates()
+                        .getGroupId().startsWith(TychoConstants.P2_GROUPID_PREFIX)) {
+                    Map<String, String> remoteProperties = getProperties(remoteDescriptor);
+                    Map<String, String> localProperties = getProperties(localDescriptor);
+                    if (!Objects.equals(remoteProperties, localProperties)) {
+                        if (logger.isExtendedDebugEnabled()) {
+                            logger.info("ArtifactKey " + artifactKey + " differs between local and remote:");
+                            TreeSet<String> allKeys = new TreeSet<>();
+                            allKeys.addAll(remoteProperties.keySet());
+                            allKeys.addAll(localProperties.keySet());
+                            for (String key : allKeys) {
+                                String remoteValue = remoteProperties.get(key);
+                                String localValue = localProperties.get(key);
+                                if (Objects.equals(remoteValue, localValue)) {
+                                    continue;
+                                }
+                                logger.info(
+                                        "\t" + key + " diverged, remote = " + remoteValue + ", local = " + localValue);
+                            }
+                        }
+                        return false;
+                    }
+                }
+            }
+            return true;
+        }
+        return false;
+    }
+
+    @SuppressWarnings("deprecation")
+    private Map<String, String> getProperties(IArtifactDescriptor descriptor) {
+        if (descriptor == null) {
+            return Map.of();
+        }
+        Map<String, String> map = new LinkedHashMap<>(descriptor.getProperties());
+        //fix bad metadata in p2...
+        if (ArtifactTransferPolicy.isCanonicalFormat(descriptor)
+                && TychoConstants.PACK200_CLASSIFIER.equals(map.get(TychoConstants.PROP_CLASSIFIER))) {
+            map.remove(TychoConstants.PROP_CLASSIFIER);
+            map.put(TychoConstants.PROP_EXTENSION, "jar");
+            map.put(TychoConstants.PROP_TYPE, PackagingType.TYPE_ECLIPSE_PLUGIN);
+        }
+        return map;
+    }
 }
