@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2012, 2013 SAP SE and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Tobias Oberlies (SAP SE) - initial API and implementation
@@ -11,20 +13,22 @@
 package org.eclipse.tycho.repository.p2base.artifact.provider.formats;
 
 import java.util.LinkedList;
+import java.util.List;
 
 import org.eclipse.equinox.p2.repository.artifact.IArtifactDescriptor;
 
 final class RemoteArtifactTransferPolicy extends ArtifactTransferPolicyBase {
 
     @Override
-    protected void insertCanonicalAndPacked(IArtifactDescriptor canonical, IArtifactDescriptor packed,
+    protected void insertCanonicalAndPacked(List<IArtifactDescriptor> canonical, List<IArtifactDescriptor> packed,
             LinkedList<IArtifactDescriptor> list) {
+        boolean isPack200able = Runtime.version().feature() < 14;
         if (canonical != null) {
-            list.add(0, canonical);
+            list.addAll(0, canonical);
         }
         if (packed != null) {
-            // packed is most preferred -> add at head of the list
-            list.add(0, packed);
+            // still consider for transtive inclusion in features on Java 14+
+            list.addAll(canonical == null || isPack200able ? 0 : canonical.size(), packed);
         }
     }
 
