@@ -1,12 +1,16 @@
 /*******************************************************************************
- * Copyright (c) 2011, 2015 SAP SE and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2011, 2020 SAP SE and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    SAP SE - initial API and implementation
+ *    Christoph Läubrich -  Bug 567098 - pomDependencies=consider should wrap non-osgi jars
+ *                          Bug 567639 - wrapAsBundle fails when dealing with esoteric versions
  *******************************************************************************/
 package org.eclipse.tycho.p2.target;
 
@@ -15,20 +19,28 @@ import static org.eclipse.tycho.p2.testutil.InstallableUnitMatchers.unit;
 import static org.eclipse.tycho.repository.testutil.ArtifactPropertiesMatchers.containsGAV;
 import static org.eclipse.tycho.repository.testutil.ArtifactPropertiesMatchers.hasProperty;
 import static org.eclipse.tycho.repository.testutil.ArtifactRepositoryMatchers.contains;
+import static org.hamcrest.CoreMatchers.containsString;
 import static org.hamcrest.CoreMatchers.is;
+import static org.hamcrest.MatcherAssert.assertThat;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
-import static org.junit.Assert.assertThat;
 
 import java.io.File;
+import java.io.IOException;
+import java.util.Properties;
 
 import org.apache.commons.io.FileUtils;
 import org.eclipse.equinox.p2.metadata.IArtifactKey;
 import org.eclipse.equinox.p2.metadata.IInstallableUnit;
 import org.eclipse.equinox.p2.repository.artifact.IArtifactDescriptor;
 import org.eclipse.equinox.p2.repository.artifact.spi.ArtifactDescriptor;
+import org.eclipse.tycho.core.shared.MockMavenContext;
 import org.eclipse.tycho.p2.impl.test.ArtifactMock;
+import org.eclipse.tycho.p2.impl.test.ReactorProjectStub;
 import org.eclipse.tycho.p2.metadata.IArtifactFacade;
 import org.eclipse.tycho.p2.repository.RepositoryLayoutHelper;
+import org.eclipse.tycho.p2.resolver.WrappedArtifact;
 import org.eclipse.tycho.repository.p2base.artifact.provider.IRawArtifactProvider;
 import org.eclipse.tycho.repository.streaming.testutil.ProbeRawArtifactSink;
 import org.eclipse.tycho.test.util.LogVerifier;
@@ -37,6 +49,8 @@ import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.TemporaryFolder;
 
+import aQute.bnd.version.Version;
+
 public class TargetPlatformBundlePublisherTest {
 
     private static final String GROUP_ID = "example.group";
@@ -53,15 +67,18 @@ public class TargetPlatformBundlePublisherTest {
     private TargetPlatformBundlePublisher subject;
 
     @Before
-    public void initSubject() {
-        logVerifier.expectNoWarnings();
+    public void initSubject() throws IOException {
 
         localRepositoryRoot = tempFolder.getRoot();
-        subject = new TargetPlatformBundlePublisher(localRepositoryRoot, logVerifier.getLogger());
+        MockMavenContext mavenContext = new MockMavenContext(localRepositoryRoot, false, logVerifier.getLogger(),
+                new Properties());
+        subject = new TargetPlatformBundlePublisher(new ReactorProjectStub(tempFolder.newFolder(), "test"),
+                mavenContext);
     }
 
     @Test
     public void testPomDependencyOnBundle() throws Exception {
+        logVerifier.expectNoWarnings();
         String bundleId = "org.eclipse.osgi";
         String bundleVersion = "3.5.2.R35x_v20100126";
 
@@ -70,7 +87,7 @@ public class TargetPlatformBundlePublisherTest {
                 RepositoryLayoutHelper.getRelativePath(GROUP_ID, ARTIFACT_ID, VERSION, null, "jar"));
         IArtifactFacade bundleArtifact = new ArtifactMock(bundleFile, GROUP_ID, ARTIFACT_ID, VERSION, "jar");
 
-        IInstallableUnit publishedUnit = subject.attemptToPublishBundle(bundleArtifact);
+        IInstallableUnit publishedUnit = subject.attemptToPublishBundle(bundleArtifact, false).getUnit();
 
         assertThat(publishedUnit, is(unit(bundleId, bundleVersion)));
         assertThat(publishedUnit.getProperties(), containsGAV(GROUP_ID, ARTIFACT_ID, VERSION));
@@ -92,22 +109,41 @@ public class TargetPlatformBundlePublisherTest {
 
     @Test
     public void testPomDependencyOnPlainJar() throws Exception {
+        logVerifier.expectNoWarnings();
+        logVerifier.expectInfo(containsString(
+                "is not a bundle and will be ignored, automatic wrapping of such artifacts can be enabled with <pomDependencies>wrapAsBundle</pomDependencies>"));
         File jarFile = resourceFile("platformbuilder/pom-dependencies/non-bundle.jar");
         IArtifactFacade jarArtifact = new ArtifactMock(jarFile, GROUP_ID, ARTIFACT_ID, VERSION, "jar");
+        assertNull(subject.attemptToPublishBundle(jarArtifact, false));
+    }
 
-        IInstallableUnit unit = subject.attemptToPublishBundle(jarArtifact);
-
-        assertNull(unit);
+    @Test
+    public void testPomDependencyWrapPlainJar() throws Exception {
+        logVerifier.expectWarning(
+                containsString("is not a bundle and was automatically wrapped with bundle-symbolic name"));
+        File jarFile = resourceFile("platformbuilder/pom-dependencies/non-bundle.jar");
+        IArtifactFacade jarArtifact = new ArtifactMock(jarFile, GROUP_ID, ARTIFACT_ID, VERSION, "jar");
+        MavenBundleInfo publishBundle = subject.attemptToPublishBundle(jarArtifact, true);
+        assertNotNull(publishBundle);
     }
 
     @Test
     public void testPomDependencyOnOtherType() throws Exception {
+        logVerifier.expectNoWarnings();
         File otherFile = resourceFile("platformbuilder/pom-dependencies/other-type.xml");
         IArtifactFacade otherArtifact = new ArtifactMock(otherFile, GROUP_ID, ARTIFACT_ID, VERSION, "pom");
+        assertNull(subject.attemptToPublishBundle(otherArtifact, false));
+    }
 
-        IInstallableUnit unit = subject.attemptToPublishBundle(otherArtifact);
-
-        assertNull(unit);
+    @Test
+    public void testVersions() {
+        assertEquals(new Version(0, 0, 1, "RELEASE121"),
+                WrappedArtifact.createOSGiVersionFromArtifact(new ArtifactMock(localRepositoryRoot, "org.netbeans.api",
+                        "org-netbeans-api-annotations-common", "RELEASE121", "jar")));
+        assertEquals(new Version(1, 0, 0, "SNAPSHOT"), WrappedArtifact.createOSGiVersionFromArtifact(
+                new ArtifactMock(localRepositoryRoot, "test", "me", "1.0.0-SNAPSHOT", "jar")));
+        assertEquals(new Version(2, 2, 0, "Final"), WrappedArtifact.createOSGiVersionFromArtifact(
+                new ArtifactMock(localRepositoryRoot, "io.undertow", "undertow-core", "2.2.0.Final", "jar")));
     }
 
     private static String artifactMD5Of(IArtifactKey key, IRawArtifactProvider artifactProvider) throws Exception {
