@@ -1,9 +1,11 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2015 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2008, 2018 Sonatype Inc. and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -82,7 +84,7 @@ public abstract class AbstractSiteDependenciesAction extends AbstractDependencie
         String range = iu.getRange();
         String type = iu.getQueryType();
         String expression = iu.getQueryExpression();
-        String[] params = iu.getQueryParams();
+        Object[] params = iu.getQueryParams();
         if (id != null) {
             VersionRange vRange = new VersionRange(range);
             return MetadataFactory.createRequirement(IInstallableUnit.NAMESPACE_IU_ID, id, vRange, null, false, false);
