@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2008, 2013 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -41,8 +43,11 @@ import org.eclipse.equinox.p2.publisher.eclipse.FeaturesAction;
 import org.eclipse.equinox.p2.publisher.eclipse.ProductAction;
 import org.eclipse.equinox.p2.repository.artifact.IArtifactDescriptor;
 import org.eclipse.equinox.p2.repository.artifact.spi.ArtifactDescriptor;
+import org.eclipse.tycho.Interpolator;
 import org.eclipse.tycho.PackagingType;
+import org.eclipse.tycho.TychoConstants;
 import org.eclipse.tycho.core.resolver.shared.OptionalResolutionAction;
+import org.eclipse.tycho.core.shared.MavenContext;
 import org.eclipse.tycho.core.shared.TargetEnvironment;
 import org.eclipse.tycho.p2.impl.publisher.model.ProductFile2;
 import org.eclipse.tycho.p2.impl.publisher.repo.FeatureRootfileArtifactRepository;
@@ -54,7 +59,7 @@ import org.eclipse.tycho.p2.metadata.IArtifactFacade;
 import org.eclipse.tycho.p2.metadata.IP2Artifact;
 import org.eclipse.tycho.p2.metadata.P2Generator;
 import org.eclipse.tycho.p2.metadata.PublisherOptions;
-import org.eclipse.tycho.p2.repository.RepositoryLayoutHelper;
+import org.eclipse.tycho.p2.metadata.ReactorProjectFacade;
 
 @SuppressWarnings("restriction")
 public class P2GeneratorImpl extends AbstractMetadataGenerator implements P2Generator {
@@ -68,6 +73,8 @@ public class P2GeneratorImpl extends AbstractMetadataGenerator implements P2Gene
      */
     private boolean dependenciesOnly;
 
+    private MavenContext mavenContext;
+
     public P2GeneratorImpl(boolean dependenciesOnly) {
         this.dependenciesOnly = dependenciesOnly;
     }
@@ -83,6 +90,7 @@ public class P2GeneratorImpl extends AbstractMetadataGenerator implements P2Gene
         Map<String, IP2Artifact> result = new LinkedHashMap<>();
 
         for (IArtifactFacade artifact : artifacts) {
+
             PublisherInfo publisherInfo = new PublisherInfo();
 
             DependencyMetadata metadata;
@@ -144,9 +152,10 @@ public class P2GeneratorImpl extends AbstractMetadataGenerator implements P2Gene
             // secondary metadata is meant to represent installable units that are provided by this project
             // but do not affect dependencies of the project itself. generateMetadata is called at the end
             // of project build lifecycle, and primary/secondary metadata separation is irrelevant at this point
-            P2Artifact p2artifact = new P2Artifact(artifact.getLocation(), metadata.getInstallableUnits(),
-                    getCanonicalArtifact(artifact.getClassifier(), metadata.getArtifactDescriptors()));
-            result.put(artifact.getClassifier(), p2artifact);
+            getCanonicalArtifact(artifact.getClassifier(), metadata.getArtifactDescriptors()).ifPresent(desc -> {
+                P2Artifact p2artifact = new P2Artifact(artifact.getLocation(), metadata.getInstallableUnits(), desc);
+                result.put(artifact.getClassifier(), p2artifact);
+            });
 
             IArtifactDescriptor packed = getPackedArtifactDescriptor(metadata.getArtifactDescriptors());
             if (packed != null) {
@@ -155,18 +164,16 @@ public class P2GeneratorImpl extends AbstractMetadataGenerator implements P2Gene
                     throw new IllegalArgumentException(
                             "Could not find packed artifact " + packed + " at " + packedLocation);
                 }
-                if (result.containsKey(RepositoryLayoutHelper.PACK200_CLASSIFIER)) {
+                if (result.containsKey(TychoConstants.PACK200_CLASSIFIER)) {
                     throw new IllegalArgumentException();
                 }
                 // workaround for bug 412497
                 Map<String, String> additionalProperties = new HashMap<>(5);
-                additionalProperties.put(RepositoryLayoutHelper.PROP_GROUP_ID, artifact.getGroupId());
-                additionalProperties.put(RepositoryLayoutHelper.PROP_ARTIFACT_ID, artifact.getArtifactId());
-                additionalProperties.put(RepositoryLayoutHelper.PROP_VERSION, artifact.getVersion());
-                additionalProperties.put(RepositoryLayoutHelper.PROP_CLASSIFIER,
-                        RepositoryLayoutHelper.PACK200_CLASSIFIER);
-                additionalProperties.put(RepositoryLayoutHelper.PROP_EXTENSION,
-                        RepositoryLayoutHelper.PACK200_EXTENSION);
+                additionalProperties.put(TychoConstants.PROP_GROUP_ID, artifact.getGroupId());
+                additionalProperties.put(TychoConstants.PROP_ARTIFACT_ID, artifact.getArtifactId());
+                additionalProperties.put(TychoConstants.PROP_VERSION, artifact.getVersion());
+                additionalProperties.put(TychoConstants.PROP_CLASSIFIER, TychoConstants.PACK200_CLASSIFIER);
+                additionalProperties.put(TychoConstants.PROP_EXTENSION, TychoConstants.PACK200_EXTENSION);
                 // workaround bug 539696
                 if (options.generateDownloadStatsProperty) {
                     Optional<IArtifactDescriptor> canonicalDescriptor = metadata.getArtifactDescriptors().stream()
@@ -179,7 +186,7 @@ public class P2GeneratorImpl extends AbstractMetadataGenerator implements P2Gene
                 }
 
                 ((ArtifactDescriptor) packed).addProperties(additionalProperties);
-                result.put(RepositoryLayoutHelper.PACK200_CLASSIFIER,
+                result.put(TychoConstants.PACK200_CLASSIFIER,
                         new P2Artifact(packedLocation, Collections.<IInstallableUnit> emptySet(), packed));
             }
         }
@@ -196,14 +203,15 @@ public class P2GeneratorImpl extends AbstractMetadataGenerator implements P2Gene
         return null;
     }
 
-    private IArtifactDescriptor getCanonicalArtifact(String classifier, Set<IArtifactDescriptor> artifactDescriptors) {
+    private Optional<IArtifactDescriptor> getCanonicalArtifact(String classifier,
+            Set<IArtifactDescriptor> artifactDescriptors) {
         for (IArtifactDescriptor descriptor : artifactDescriptors) {
-            String _classifier = descriptor.getProperty(RepositoryLayoutHelper.PROP_CLASSIFIER);
+            String _classifier = descriptor.getProperty(TychoConstants.PROP_CLASSIFIER);
             if (eq(classifier, _classifier) && descriptor.getProperty(IArtifactDescriptor.FORMAT) == null) {
-                return descriptor;
+                return Optional.of(descriptor);
             }
         }
-        throw new IllegalArgumentException();
+        return Optional.empty();
     }
 
     private static <T> boolean eq(T a, T b) {
@@ -335,7 +343,7 @@ public class P2GeneratorImpl extends AbstractMetadataGenerator implements P2Gene
     private List<File> getProductFiles(File projectLocation) {
         List<File> res = new ArrayList<>();
         for (File f : projectLocation.listFiles()) {
-            if (f.isFile() && f.getName().endsWith(".product")) {
+            if (f.isFile() && f.getName().endsWith(".product") && !f.getName().startsWith(".polyglot")) {
                 res.add(f);
             }
         }
@@ -353,10 +361,15 @@ public class P2GeneratorImpl extends AbstractMetadataGenerator implements P2Gene
 
     @Override
     protected List<IPublisherAdvice> getPublisherAdvice(IArtifactFacade artifact, PublisherOptions options) {
+        Interpolator interpolator;
+        if (artifact instanceof ReactorProjectFacade) {
+            interpolator = ((ReactorProjectFacade) artifact).getReactorProject().getInterpolator();
+        } else {
+            interpolator = null;
+        }
         ArrayList<IPublisherAdvice> advice = new ArrayList<>();
-        advice.add(new MavenPropertiesAdvice(artifact.getGroupId(), artifact.getArtifactId(), artifact.getVersion(),
-                artifact.getClassifier()));
-        advice.add(getExtraEntriesAdvice(artifact));
+        advice.add(new MavenPropertiesAdvice(artifact, mavenContext));
+        advice.add(getExtraEntriesAdvice(artifact, interpolator));
 
         if (options.generateDownloadStatsProperty) {
             advice.add(new DownloadStatsAdvice());
@@ -369,4 +382,8 @@ public class P2GeneratorImpl extends AbstractMetadataGenerator implements P2Gene
         }
         return advice;
     }
+
+    public void setMavenContext(MavenContext mavenContext) {
+        this.mavenContext = mavenContext;
+    }
 }
