@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2010, 2017 SAP AG and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    SAP AG - initial API and implementation
@@ -22,14 +24,16 @@ import org.eclipse.equinox.internal.p2.core.helpers.FileUtils.IPathComputer;
 import org.eclipse.equinox.internal.p2.publisher.FileSetDescriptor;
 import org.eclipse.equinox.p2.metadata.Version;
 import org.eclipse.equinox.p2.publisher.actions.IFeatureRootAdvice;
+import org.eclipse.tycho.BuildProperties;
+import org.eclipse.tycho.BuildPropertiesParser;
+import org.eclipse.tycho.Interpolator;
 import org.eclipse.tycho.PackagingType;
-import org.eclipse.tycho.core.shared.BuildProperties;
-import org.eclipse.tycho.core.shared.BuildPropertiesParser;
 import org.eclipse.tycho.p2.metadata.IArtifactFacade;
+import org.eclipse.tycho.p2.metadata.ReactorProjectFacade;
 
 /**
  * This class handles definitions of root files in build.properties according to
- * http://help.eclipse.org/helios/index.jsp?topic=/org.eclipse.pde.doc.user/tasks/pde_rootfiles.htm
+ * https://help.eclipse.org/helios/index.jsp?topic=/org.eclipse.pde.doc.user/tasks/pde_rootfiles.htm
  * <br>
  */
 @SuppressWarnings("restriction")
@@ -53,11 +57,17 @@ public class FeatureRootAdvice implements IFeatureRootAdvice {
      */
     public static IFeatureRootAdvice createRootFileAdvice(IArtifactFacade featureArtifact,
             BuildPropertiesParser buildPropertiesParser) {
+        Interpolator interpolator;
+        if (featureArtifact instanceof ReactorProjectFacade) {
+            interpolator = ((ReactorProjectFacade) featureArtifact).getReactorProject().getInterpolator();
+        } else {
+            interpolator = null;
+        }
         File projectDir = getProjectBaseDir(featureArtifact);
 
         if (projectDir != null) {
-            FeatureRootAdvice result = new FeatureRootAdvice(buildPropertiesParser.parse(projectDir), projectDir,
-                    featureArtifact.getArtifactId());
+            FeatureRootAdvice result = new FeatureRootAdvice(buildPropertiesParser.parse(projectDir, interpolator),
+                    projectDir, featureArtifact.getArtifactId());
             if (result.hasRootFiles()) {
                 return result;
             }
@@ -190,7 +200,7 @@ public class FeatureRootAdvice implements IFeatureRootAdvice {
     }
 
     private static void ensureRootFilesConfigured(RootFilesProperties rootProperties, ConfigSpec configuration) {
-        if (rootProperties.getFileMap().keySet().size() == 0) {
+        if (rootProperties.getFileMap().keySet().isEmpty()) {
             String message;
             if (configuration.equals(ConfigSpec.GLOBAL)) {
                 message = "Cannot set permissions or symbolic links if there are no root files";
