@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2008, 2011 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -12,6 +14,7 @@ package org.eclipse.tycho.p2.metadata;
 
 import java.util.List;
 
+import org.eclipse.tycho.IDependencyMetadata;
 import org.eclipse.tycho.core.resolver.shared.OptionalResolutionAction;
 import org.eclipse.tycho.core.shared.TargetEnvironment;
 
@@ -21,5 +24,5 @@ public interface DependencyMetadataGenerator {
      * Generates dependency-only artifact metadata
      */
     public IDependencyMetadata generateMetadata(IArtifactFacade artifact, List<TargetEnvironment> environments,
-            OptionalResolutionAction optionalAction);
+            OptionalResolutionAction optionalAction, PublisherOptions options);
 }
