@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2008, 2012 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -13,6 +15,8 @@ package org.eclipse.tycho.p2.repository;
 
 import java.io.File;
 
+import org.eclipse.tycho.core.shared.MavenContext;
+
 /**
  * Interface to obtain artifacts from GAV-indexed repositories.
  */
@@ -24,6 +28,8 @@ public interface RepositoryReader {
      * 
      * @return the local location of the artifact; never <code>null</code>
      */
-    File getLocalArtifactLocation(GAV gav, String classifier, String extension);
+    File getLocalArtifactLocation(GAV gav, String classifier, String type);
+
+    MavenContext getMavenContext();
 
 }
