@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2012 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -73,12 +75,11 @@ public class BaselineServiceImpl implements BaselineService {
 
         try {
             IProvisioningAgent remoteAgent = remoteAgentManager.getProvisioningAgent();
-            IRepositoryIdManager remoteRepositoryIdManager = (IRepositoryIdManager) remoteAgent
-                    .getService(IRepositoryIdManager.SERVICE_NAME);
-            IMetadataRepositoryManager remoteMetadataRepositoryManager = (IMetadataRepositoryManager) remoteAgent
-                    .getService(IMetadataRepositoryManager.SERVICE_NAME);
-            IArtifactRepositoryManager remoteArtifactRepositoryManager = (IArtifactRepositoryManager) remoteAgent
-                    .getService(IArtifactRepositoryManager.SERVICE_NAME);
+            IRepositoryIdManager remoteRepositoryIdManager = remoteAgent.getService(IRepositoryIdManager.class);
+            IMetadataRepositoryManager remoteMetadataRepositoryManager = remoteAgent
+                    .getService(IMetadataRepositoryManager.class);
+            IArtifactRepositoryManager remoteArtifactRepositoryManager = remoteAgent
+                    .getService(IArtifactRepositoryManager.class);
 
             baselineUnits = CompositeMetadataRepository.createMemoryComposite(remoteAgent);
             baselineArtifacts = CompositeArtifactRepository.createMemoryComposite(remoteAgent);
