@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2010, 2015 SAP SE and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     SAP SE - initial API and implementation
@@ -22,6 +24,7 @@ import org.eclipse.equinox.internal.p2.updatesite.CategoryXMLAction;
 import org.eclipse.equinox.p2.metadata.IInstallableUnit;
 import org.eclipse.equinox.p2.publisher.IPublisherAction;
 import org.eclipse.equinox.p2.publisher.actions.JREAction;
+import org.eclipse.tycho.core.ee.shared.ExecutionEnvironment;
 import org.eclipse.tycho.core.resolver.shared.DependencySeed;
 import org.eclipse.tycho.p2.target.ee.CustomEEResolutionHints;
 import org.eclipse.tycho.p2.tools.FacadeException;
@@ -43,8 +46,8 @@ class PublisherServiceImpl implements PublisherService {
     }
 
     @Override
-    public Collection<DependencySeed> publishCategories(File categoryDefinition) throws FacadeException,
-            IllegalStateException {
+    public Collection<DependencySeed> publishCategories(File categoryDefinition)
+            throws FacadeException, IllegalStateException {
 
         /*
          * At this point, we expect that the category.xml file does no longer contain any
@@ -82,15 +85,20 @@ class PublisherServiceImpl implements PublisherService {
         return toSeeds(null, allIUs);
     }
 
+    @Override
+    public Collection<DependencySeed> publishEEProfile(ExecutionEnvironment ee) throws FacadeException {
+        IPublisherAction jreAction = new ExecutionEnvironmentAction(ee);
+        Collection<IInstallableUnit> allIUs = publisherRunner.executeAction(jreAction,
+                publishingRepository.getMetadataRepository(), publishingRepository.getArtifactRepository());
+        return toSeeds(null, allIUs);
+    }
+
     void validateProfile(File profileFile) throws FacadeException {
         Properties profileProperties = new Properties();
         try {
-            FileInputStream stream = new FileInputStream(profileFile);
-            try {
+            try (FileInputStream stream = new FileInputStream(profileFile)) {
                 profileProperties.load(stream);
                 validateProfile(profileProperties, profileFile);
-            } finally {
-                stream.close();
             }
         } catch (IOException e) {
             throw new FacadeException(e);
@@ -107,8 +115,8 @@ class PublisherServiceImpl implements PublisherService {
         String profileNameKey = "osgi.java.profile.name";
         String profileName = props.getProperty(profileNameKey);
         if (profileName == null) {
-            throw new FacadeException("Mandatory property '" + profileNameKey + "' is missing in profile file "
-                    + profileFile);
+            throw new FacadeException(
+                    "Mandatory property '" + profileNameKey + "' is missing in profile file " + profileFile);
         }
 
         // make sure the profile name ends in a version
@@ -119,8 +127,8 @@ class PublisherServiceImpl implements PublisherService {
          * file name instead of the value specified as osgi.java.profile.name in the profile file),
          * require that these are the same.
          */
-        String fileNamePrefix = simpleFileName.substring(0, simpleFileName.length() - ".profile".length()).toLowerCase(
-                Locale.ENGLISH);
+        String fileNamePrefix = simpleFileName.substring(0, simpleFileName.length() - ".profile".length())
+                .toLowerCase(Locale.ENGLISH);
         if (!fileNamePrefix.equals(profileName.toLowerCase(Locale.ENGLISH))) {
             throw new FacadeException("Profile file with 'osgi.java.profile.name=" + profileName + "' must be named '"
                     + profileName + ".profile', but found file name: '" + simpleFileName + "'");
