@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2012, 2016 SAP SE and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    SAP SE - initial API and implementation
@@ -12,6 +14,7 @@ package org.eclipse.tycho.p2.tools.director.shared;
 
 import java.io.File;
 import java.net.URI;
+import java.util.Map;
 
 import org.eclipse.tycho.core.resolver.shared.DependencySeed;
 import org.eclipse.tycho.core.shared.TargetEnvironment;
@@ -44,7 +47,11 @@ public interface DirectorRuntime {
 
         void setVerifyOnly(boolean verifyOnly);
 
+        void setBundlePool(File path);
+
         void execute() throws DirectorCommandException;
+
+        void setProfileProperties(Map<String, String> profileProperties);
     }
 
     /**
