@@ -1,9 +1,11 @@
 /*******************************************************************************
- * Copyright (c) 2011, 2017 SAP SE and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2011, 2020 SAP SE and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    SAP SE - initial API and implementation
@@ -20,9 +22,10 @@ import java.io.File;
 import java.io.FileFilter;
 import java.util.Collections;
 
+import org.eclipse.tycho.BuildDirectory;
 import org.eclipse.tycho.BuildOutputDirectory;
 import org.eclipse.tycho.core.shared.MavenContext;
-import org.eclipse.tycho.core.shared.MavenContextImpl;
+import org.eclipse.tycho.core.shared.MockMavenContext;
 import org.eclipse.tycho.p2.tools.DestinationRepositoryDescriptor;
 import org.eclipse.tycho.p2.tools.FacadeException;
 import org.eclipse.tycho.p2.tools.RepositoryReferences;
@@ -46,13 +49,13 @@ public class MirrorStandaloneTest {
     @Rule
     public final LogVerifier logVerifier = new LogVerifier();
 
-    private BuildOutputDirectory targetFolder;
+    private BuildDirectory targetFolder;
 
     @Before
     public void initTestContext() throws Exception {
         destinationRepo = new DestinationRepositoryDescriptor(tempFolder.newFolder("dest"), DEFAULT_NAME);
         subject = new MirrorApplicationServiceImpl();
-        MavenContext mavenContext = new MavenContextImpl(null, logVerifier.getLogger());
+        MavenContext mavenContext = new MockMavenContext(null, logVerifier.getLogger());
         subject.setMavenContext(mavenContext);
         targetFolder = new BuildOutputDirectory(tempFolder.getRoot());
     }
@@ -96,11 +99,10 @@ public class MirrorStandaloneTest {
 
     @Test
     public void testMirrorMatchExpression() throws Exception {
-        subject.mirrorStandalone(
-                e342PlusFragmentsRepo(),
-                destinationRepo,
-                Collections.singletonList(new IUDescription(null, null, "id == $0 && version == $1", new String[] {
-                        "org.eclipse.core.runtime", "3.4.0.v20080512" })), new MirrorOptions(), targetFolder);
+        subject.mirrorStandalone(e342PlusFragmentsRepo(), destinationRepo,
+                Collections.singletonList(new IUDescription(null, null, "id == $0 && version == $1",
+                        new String[] { "org.eclipse.core.runtime", "3.4.0.v20080512" })),
+                new MirrorOptions(), targetFolder);
         assertEquals(1, getMirroredBundleFiles().length);
         assertTrue(repoFile(destinationRepo, "plugins/org.eclipse.core.runtime_3.4.0.v20080512.jar").exists());
     }
@@ -110,21 +112,16 @@ public class MirrorStandaloneTest {
         MirrorOptions mirrorOptions = new MirrorOptions();
         mirrorOptions.setLatestVersionOnly(true);
         subject.mirrorStandalone(sourceRepos("e342", "e352"), destinationRepo, null, mirrorOptions, targetFolder);
-        File[] runtimeBundles = new File(destinationRepo.getLocation(), "plugins").listFiles(new FileFilter() {
-
-            @Override
-            public boolean accept(File file) {
-                return file.getName().startsWith("org.eclipse.core.runtime");
-            }
-        });
+        File[] runtimeBundles = new File(destinationRepo.getLocation(), "plugins")
+                .listFiles((FileFilter) file -> file.getName().startsWith("org.eclipse.core.runtime"));
         assertEquals(1, runtimeBundles.length);
     }
 
     @Test(expected = FacadeException.class)
     public void testMirrorNotExisting() throws Exception {
         subject.mirrorStandalone(e342PlusFragmentsRepo(), destinationRepo,
-                Collections.singletonList(new IUDescription("org.eclipse.core.runtime", "10.0.0")),
-                new MirrorOptions(), targetFolder);
+                Collections.singletonList(new IUDescription("org.eclipse.core.runtime", "10.0.0")), new MirrorOptions(),
+                targetFolder);
     }
 
     @Test
