@@ -1,10 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2018 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2019 IBM Corporation and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
- * 
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
+ *
  * Contributors:
  *    IBM Corporation - initial API and implementation
  *    Sonatype Inc. - plexus.compiler implementation
@@ -20,11 +22,15 @@ import java.io.StringReader;
 import java.io.StringWriter;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.StringJoiner;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
+import java.util.stream.Collectors;
+import java.util.stream.Stream;
 
 import org.codehaus.plexus.compiler.AbstractCompiler;
 import org.codehaus.plexus.compiler.CompilerConfiguration;
@@ -41,13 +47,14 @@ import org.codehaus.plexus.util.cli.CommandLineUtils;
 import org.codehaus.plexus.util.cli.Commandline;
 import org.eclipse.jdt.core.compiler.CharOperation;
 import org.eclipse.jdt.internal.compiler.batch.Main;
+import org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
 import org.eclipse.jdt.internal.compiler.util.SuffixConstants;
 import org.eclipse.jdt.internal.compiler.util.Util;
 import org.eclipse.tycho.compiler.jdt.copied.LibraryInfo;
 
 /**
- * See http://help.eclipse.org/ganymede/topic/org.eclipse.jdt.doc.isv/guide/jdt_api_options.htm
+ * See https://help.eclipse.org/ganymede/topic/org.eclipse.jdt.doc.isv/guide/jdt_api_options.htm
  */
 
 @Component(role = org.codehaus.plexus.compiler.Compiler.class, hint = "jdt")
@@ -96,8 +103,9 @@ public class JDTCompiler extends AbstractCompiler {
         getLogger().info("Compiling " + sourceFiles.length + " " + "source file" + (sourceFiles.length == 1 ? "" : "s")
                 + " to " + destinationDir.getAbsolutePath());
 
-        Map<String, String> customCompilerArguments = config.getCustomCompilerArgumentsAsMap();
-        checkCompilerArgs(customCompilerArguments, custom);
+        Collection<Map.Entry<String, String>> customCompilerArgumentEntries = config
+                .getCustomCompilerArgumentsEntries();
+        checkCompilerArgs(customCompilerArgumentEntries, custom);
 
         String[] args = buildCompilerArguments(config, custom, sourceFiles);
 
@@ -112,7 +120,7 @@ public class JDTCompiler extends AbstractCompiler {
 
             messages = compileOutOfProcess(config.getWorkingDirectory(), executable, args);
         } else {
-            messages = compileInProcess(args, custom);
+            messages = compileInProcess(args, config, custom);
         }
 
         return messages;
@@ -127,6 +135,37 @@ public class JDTCompiler extends AbstractCompiler {
             String[] sourceFiles) {
         List<String> args = new ArrayList<>();
 
+        Collection<Map.Entry<String, String>> customCompilerArgumentsEntries = config
+                .getCustomCompilerArgumentsEntries();
+        for (Map.Entry<String, String> entry : customCompilerArgumentsEntries) {
+
+            String key = entry.getKey();
+
+            if (StringUtils.isEmpty(key) || key.startsWith("@")) {
+                continue;
+            }
+
+            if ("use.java.home".equals(key)) {
+                custom.javaHome = entry.getValue();
+                continue;
+            }
+
+            if ("org.osgi.framework.system.packages".equals(key)) {
+                custom.bootclasspathAccessRules = entry.getValue();
+                continue;
+            }
+
+            args.add(key);
+
+            String value = entry.getValue();
+
+            if (StringUtils.isEmpty(value)) {
+                continue;
+            }
+
+            args.add(value);
+        }
+
         // ----------------------------------------------------------------------
         // Set output
         // ----------------------------------------------------------------------
@@ -206,6 +245,10 @@ public class JDTCompiler extends AbstractCompiler {
             args.add("-nowarn");
         }
 
+        if (config.isFailOnWarning()) {
+            args.add("-failOnWarning");
+        }
+
         // TODO: this could be much improved
         if (StringUtils.isEmpty(config.getTargetVersion())) {
             // Required, or it defaults to the target of your JDK (eg 1.5)
@@ -225,42 +268,17 @@ public class JDTCompiler extends AbstractCompiler {
             args.add(config.getSourceVersion());
         }
 
+        if (!StringUtils.isEmpty(config.getReleaseVersion())) {
+            args.add("--release");
+            args.add(config.getReleaseVersion());
+        }
+
         if (!suppressEncoding(config) && !StringUtils.isEmpty(config.getSourceEncoding())) {
             args.add("-encoding");
             args.add(config.getSourceEncoding());
         }
 
-        Map<String, String> customCompilerArguments = config.getCustomCompilerArgumentsAsMap();
-        for (Map.Entry<String, String> entry : customCompilerArguments.entrySet()) {
-
-            String key = (String) entry.getKey();
-
-            if (StringUtils.isEmpty(key) || key.startsWith("@")) {
-                continue;
-            }
-
-            if ("use.java.home".equals(key)) {
-                custom.javaHome = (String) entry.getValue();
-                continue;
-            }
-
-            if ("org.osgi.framework.system.packages".equals(key)) {
-                custom.bootclasspathAccessRules = entry.getValue();
-                continue;
-            }
-
-            args.add(key);
-
-            String value = (String) entry.getValue();
-
-            if (StringUtils.isEmpty(value)) {
-                continue;
-            }
-
-            args.add(value);
-        }
-
-        return (String[]) args.toArray(new String[args.size()]);
+        return args.toArray(new String[args.size()]);
     }
 
     private static boolean suppressSource(CompilerConfiguration config) {
@@ -273,7 +291,7 @@ public class JDTCompiler extends AbstractCompiler {
 
     /**
      * Compile the java sources in a external process, calling an external executable, like javac.
-     * 
+     *
      * @param workingDirectory
      *            base directory where the process will be launched
      * @param executable
@@ -309,9 +327,7 @@ public class JDTCompiler extends AbstractCompiler {
             returnCode = CommandLineUtils.executeCommandLine(cli, out, err);
 
             messages = parseModernStream(new BufferedReader(new StringReader(err.getOutput())));
-        } catch (CommandLineException e) {
-            throw new CompilerException("Error while executing the external compiler.", e);
-        } catch (IOException e) {
+        } catch (CommandLineException | IOException e) {
             throw new CompilerException("Error while executing the external compiler.", e);
         }
 
@@ -327,24 +343,28 @@ public class JDTCompiler extends AbstractCompiler {
     /**
      * Compile the java sources in the current JVM, without calling an external executable, using
      * <code>com.sun.tools.javac.Main</code> class
-     * 
+     *
      * @param args
      *            arguments for the compiler as they would be used in the command line javac
      * @return CompilerResult with the errors and warnings encountered.
      * @throws CompilerException
      */
-    CompilerResult compileInProcess(String[] args, CustomCompilerConfiguration custom) throws CompilerException {
+    CompilerResult compileInProcess(String[] args, CompilerConfiguration config, CustomCompilerConfiguration custom)
+            throws CompilerException {
 
         List<CompilerMessage> messages;
 
         StringWriter out = new StringWriter();
         StringWriter err = new StringWriter();
 
-        Main compiler = new Main(new PrintWriter(out), new PrintWriter(err), false);
+        Main compiler = new Main(new PrintWriter(out), new PrintWriter(err), false, null, null);
         compiler.options.put(CompilerOptions.OPTION_ReportForbiddenReference, CompilerOptions.ERROR);
         List<String> jdtCompilerArgs = new ArrayList<>(Arrays.asList(args));
         if (custom.javaHome != null) {
-            addExternalJavaHomeArgs(jdtCompilerArgs, custom.javaHome);
+            String sourceLevel = config.getSourceVersion();
+            if (sourceLevel == null || CompilerOptions.versionToJdkLevel(sourceLevel) <= ClassFileConstants.JDK1_8) {
+                addExternalJavaHomeArgs(jdtCompilerArgs, custom.javaHome);
+            }
         }
         getLogger().debug("JDT compiler args: " + jdtCompilerArgs);
         boolean success = compiler.compile(jdtCompilerArgs.toArray(new String[0]));
@@ -390,7 +410,7 @@ public class JDTCompiler extends AbstractCompiler {
 
     /**
      * Parse the output from the compiler into a list of CompilerMessage objects
-     * 
+     *
      * @param input
      *            The output of the compiler
      * @return List of CompilerMessage objects
@@ -440,14 +460,14 @@ public class JDTCompiler extends AbstractCompiler {
 
     /**
      * Copy the classpath to the command line with access rules included.
-     * 
+     *
      * @param cmd
      *            the given command line
      * @param classpath
      *            the given classpath entry
      */
     private String createClasspathArgument(List<String> classpath, CustomCompilerConfiguration custom) {
-        final String[] pathElements = (String[]) classpath.toArray(new String[classpath.size()]);
+        final String[] pathElements = classpath.toArray(new String[classpath.size()]);
 
         // empty path return empty string
         if (pathElements.length == 0) {
@@ -456,35 +476,25 @@ public class JDTCompiler extends AbstractCompiler {
 
         // no access rules, can set the path directly
         if (custom.accessRules == null) {
-            StringBuilder result = new StringBuilder();
-            for (int i = 0; i < pathElements.length; i++) {
-                result.append(pathElements[i]);
-                if (i < pathElements.length - 1) {
-                    result.append(File.pathSeparatorChar);
-                }
-            }
-            return result.toString();
+            return Stream.of(pathElements).collect(Collectors.joining(File.pathSeparator));
         }
 
         int rulesLength = custom.accessRules.size();
-        String[] rules = (String[]) custom.accessRules.toArray(new String[rulesLength]);
+        String[] rules = custom.accessRules.toArray(new String[rulesLength]);
         int nextRule = 0;
-        final StringBuffer result = new StringBuffer();
+        final StringJoiner result = new StringJoiner(File.pathSeparator);
 
         // access rules are expected in the same order as the classpath, but
         // there could
         // be elements in the classpath not in the access rules or access rules
         // not in the classpath
-        for (int i = 0, max = pathElements.length; i < max; i++) {
-            if (i > 0)
-                result.append(File.pathSeparatorChar);
-            String pathElement = pathElements[i];
-            result.append(pathElement);
+        for (String pathElement : pathElements) {
+            result.add(pathElement);
             // the rules list is [path, rule, path, rule, ...]
             for (int j = nextRule; j < rulesLength; j += 2) {
                 String rule = rules[j];
                 if (pathElement.endsWith(rule)) {
-                    result.append(rules[j + 1]);
+                    result.add(rules[j + 1]);
                     nextRule = j + 2;
                     break;
                 }
@@ -497,7 +507,7 @@ public class JDTCompiler extends AbstractCompiler {
                     int ruleLength = rule.length();
                     if (pathElement.regionMatches(false, pathElement.length() - ruleLength + 1, rule, 0,
                             ruleLength - 1)) {
-                        result.append(rules[j + 1]);
+                        result.add(rules[j + 1]);
                         nextRule = j + 2;
                         break;
                     }
@@ -506,7 +516,7 @@ public class JDTCompiler extends AbstractCompiler {
                     // might
                     int ruleLength = rule.length();
                     if (pathElement.regionMatches(false, pathElement.length() - ruleLength - 1, rule, 0, ruleLength)) {
-                        result.append(rules[j + 1]);
+                        result.add(rules[j + 1]);
                         nextRule = j + 2;
                         break;
                     }
@@ -514,19 +524,20 @@ public class JDTCompiler extends AbstractCompiler {
             }
         }
 
-        String s = result.toString();
-        return s;
+        return result.toString();
     }
 
     /**
      * check the compiler arguments. Extract from files specified using @, lines marked with
-     * ADAPTER_PREFIX These lines specify information that needs to be interpreted by us.
-     * 
+     * {@link #ADAPTER_PREFIX}. These lines specify information that needs to be interpreted by us.
+     *
      * @param args
      *            compiler arguments to process
      */
-    private void checkCompilerArgs(Map<String, String> args, CustomCompilerConfiguration custom) {
-        for (String arg : args.keySet()) {
+    private void checkCompilerArgs(Collection<Map.Entry<String, String>> argEntries,
+            CustomCompilerConfiguration custom) {
+        for (Map.Entry<String, String> argEntry : argEntries) {
+            String arg = argEntry.getKey();
             if (arg.charAt(0) == '@') {
                 try {
                     char[] content = Util.getFileCharContent(new File(arg.substring(1)), null);
@@ -557,7 +568,7 @@ public class JDTCompiler extends AbstractCompiler {
                                     if (custom.fileEncodings == null)
                                         custom.fileEncodings = new HashMap<>();
                                     // use File to translate the string into a
-                                    // path with the correct File.seperator
+                                    // path with the correct File.separator
                                     custom.fileEncodings.put(str, enc);
                                 } else {
                                     if (custom.dirEncodings == null)
