@@ -18,13 +18,23 @@ package org.eclipse.tycho.compiler;
 
 import java.io.File;
 import java.io.IOException;
+import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.Collections;
+import java.util.Comparator;
 import java.util.HashSet;
+import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Map.Entry;
+import java.util.Objects;
+import java.util.Optional;
 import java.util.Set;
+import java.util.StringJoiner;
+import java.util.stream.Collectors;
 
 import org.apache.maven.artifact.Artifact;
 import org.apache.maven.artifact.repository.ArtifactRepository;
@@ -39,6 +49,7 @@ import org.apache.maven.plugins.annotations.Parameter;
 import org.apache.maven.project.MavenProject;
 import org.apache.maven.project.artifact.ProjectArtifact;
 import org.apache.maven.repository.RepositorySystem;
+import org.apache.maven.toolchain.ToolchainManager;
 import org.apache.maven.toolchain.ToolchainManagerPrivate;
 import org.apache.maven.toolchain.java.DefaultJavaToolChain;
 import org.codehaus.plexus.compiler.CompilerConfiguration;
@@ -46,16 +57,25 @@ import org.codehaus.plexus.compiler.util.scan.InclusionScanException;
 import org.codehaus.plexus.compiler.util.scan.SimpleSourceInclusionScanner;
 import org.codehaus.plexus.compiler.util.scan.SourceInclusionScanner;
 import org.codehaus.plexus.compiler.util.scan.StaleSourceScanner;
+import org.codehaus.plexus.logging.Logger;
 import org.codehaus.plexus.util.DirectoryScanner;
 import org.codehaus.plexus.util.FileUtils;
 import org.codehaus.plexus.util.xml.Xpp3Dom;
+import org.eclipse.jdt.internal.compiler.util.CtSym;
+import org.eclipse.jdt.internal.compiler.util.JRTUtil;
+import org.eclipse.osgi.util.ManifestElement;
+import org.eclipse.tycho.DefaultArtifactKey;
+import org.eclipse.tycho.ReactorProject;
 import org.eclipse.tycho.classpath.ClasspathEntry;
 import org.eclipse.tycho.classpath.ClasspathEntry.AccessRule;
 import org.eclipse.tycho.classpath.JavaCompilerConfiguration;
 import org.eclipse.tycho.classpath.SourcepathEntry;
 import org.eclipse.tycho.core.BundleProject;
-import org.eclipse.tycho.core.TychoConstants;
 import org.eclipse.tycho.core.TychoProject;
+import org.eclipse.tycho.core.dotClasspath.JREClasspathEntry;
+import org.eclipse.tycho.core.dotClasspath.M2ClasspathVariable;
+import org.eclipse.tycho.core.dotClasspath.ProjectClasspathEntry;
+import org.eclipse.tycho.core.ee.ExecutionEnvironmentUtils;
 import org.eclipse.tycho.core.ee.StandardExecutionEnvironment;
 import org.eclipse.tycho.core.ee.shared.ExecutionEnvironment;
 import org.eclipse.tycho.core.maven.ToolchainProvider;
@@ -65,15 +85,22 @@ import org.eclipse.tycho.core.osgitools.DefaultClasspathEntry;
 import org.eclipse.tycho.core.osgitools.DefaultClasspathEntry.DefaultAccessRule;
 import org.eclipse.tycho.core.osgitools.DefaultReactorProject;
 import org.eclipse.tycho.core.osgitools.OsgiBundleProject;
-import org.eclipse.tycho.core.osgitools.project.BuildOutputJar;
+import org.eclipse.tycho.core.osgitools.OsgiManifest;
 import org.eclipse.tycho.core.osgitools.project.EclipsePluginProject;
 import org.eclipse.tycho.core.utils.TychoProjectUtils;
 import org.eclipse.tycho.runtime.Adaptable;
+import org.osgi.framework.Constants;
+import org.osgi.framework.Filter;
+import org.osgi.framework.FrameworkUtil;
+import org.osgi.framework.InvalidSyntaxException;
+import org.osgi.framework.Version;
+import org.osgi.framework.namespace.ExecutionEnvironmentNamespace;
+import org.osgi.resource.Namespace;
 
 import copied.org.apache.maven.plugin.AbstractCompilerMojo;
 
-public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo implements JavaCompilerConfiguration,
-        Adaptable {
+public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo
+        implements JavaCompilerConfiguration, Adaptable {
 
     public static final String RULE_SEPARATOR = File.pathSeparator;
 
@@ -87,7 +114,7 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
     private static final String PREFS_FILE_PATH = ".settings" + File.separator + "org.eclipse.jdt.core.prefs";
 
     @Parameter(property = "project", readonly = true)
-    private MavenProject project;
+    protected MavenProject project;
 
     /**
      * Transitively add specified maven artifacts to compile classpath in addition to elements
@@ -110,8 +137,8 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
     /**
      * Which JDK to use for compilation. Default value is SYSTEM which means the currently running
      * JDK. If BREE is specified, MANIFEST header <code>Bundle-RequiredExecutionEnvironment</code>
-     * is used to define the JDK to compile against. In this case, you need to provide a <a
-     * href="http://maven.apache.org/guides/mini/guide-using-toolchains.html">toolchains.xml</a>
+     * is used to define the JDK to compile against. In this case, you need to provide a
+     * <a href="http://maven.apache.org/guides/mini/guide-using-toolchains.html">toolchains.xml</a>
      * configuration file. The value of BREE will be matched against the id of the toolchain
      * elements in toolchains.xml. Example:
      * 
@@ -179,8 +206,8 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
 
     /**
      * Whether a bundle is required to explicitly import non-java.* packages from the JDK. This is
-     * the design-time equivalent to the equinox runtime option <a
-     * href="http://wiki.eclipse.org/Equinox_Boot_Delegation#The_solution"
+     * the design-time equivalent to the equinox runtime option
+     * <a href="https://wiki.eclipse.org/Equinox_Boot_Delegation#The_solution"
      * >osgi.compatibility.bootdelegation</a>.
      */
     @Parameter(defaultValue = "false")
@@ -199,13 +226,20 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
      * ${project.basedir}/.settings/org.eclipse.jdt.core.prefs will be passed to the compiler. If
      * the file is not present, the build will not fail.
      */
-    @Parameter(defaultValue = "false")
+    @Parameter(defaultValue = "true")
     private boolean useProjectSettings;
 
     /**
-     * Current build output jar
+     * Whether the <code>-release</code> argument for the Java compiler should be derived from the
+     * target level. Enabled by default.
+     * <p>
+     * Disabling this can be useful in situations where compiling using <code>-release</code> cannot
+     * be used, e.g. when referencing internal JDK classes exported via an OSGI framework extension.
+     * In that case <code>&lt;release&gt;</code> should also be explicitly set to an empty value to
+     * prevent it from being inherited.
      */
-    private BuildOutputJar outputJar;
+    @Parameter(defaultValue = "true")
+    private boolean deriveReleaseCompilerArgumentFromTargetLevel = true;
 
     @Component(role = TychoProject.class)
     private Map<String, TychoProject> projectTypes;
@@ -224,10 +258,10 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
      * <code>${project.build.outputDirectory}</code>.
      * 
      * Set this to <code>false</code> in case you want to keep resources separate from java files in
-     * <code>src/main/resources</code> and handle them using <a
-     * href="http://maven.apache.org/plugins/maven-resources-plugin/"> maven-resources-plugin</a>
-     * (e.g. for <a
-     * href="http://maven.apache.org/plugins/maven-resources-plugin/examples/filter.html">resource
+     * <code>src/main/resources</code> and handle them using
+     * <a href="http://maven.apache.org/plugins/maven-resources-plugin/"> maven-resources-plugin</a>
+     * (e.g. for <a href=
+     * "http://maven.apache.org/plugins/maven-resources-plugin/examples/filter.html">resource
      * filtering<a/>.
      * 
      */
@@ -276,29 +310,104 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
     @Component
     ToolchainProvider toolchainProvider;
 
+    @Component
+    private ToolchainManager toolchainManager;
+
+    @Component
+    private Logger logger;
+
+    private StandardExecutionEnvironment[] manifestBREEs;
+
+    private File currentOutputDirectory;
+
+    private List<String> currentSourceRoots;
+
+    private List<String> currentExcludes;
+
     @Override
-    public void execute() throws MojoExecutionException, MojoFailureException {
-        StandardExecutionEnvironment[] manifestBREEs = bundleReader.loadManifest(project.getBasedir())
-                .getExecutionEnvironments();
-        getLog().debug("Manifest BREEs: " + Arrays.toString(manifestBREEs));
-        getLog().debug("Effective EE: " + getTargetExecutionEnvironment());
+    public final void execute() throws MojoExecutionException, MojoFailureException {
+        getLog().debug("Manifest BREEs: " + Arrays.toString(getBREE()));
+        getLog().debug("Target Platform EE: " + getTargetExecutionEnvironment());
         String effectiveTargetLevel = getTargetLevel();
         getLog().debug("Effective source/target: " + getSourceLevel() + "/" + effectiveTargetLevel);
 
         checkTargetLevelCompatibleWithManifestBREEs(effectiveTargetLevel, manifestBREEs);
 
-        for (BuildOutputJar jar : getEclipsePluginProject().getOutputJars()) {
-            this.outputJar = jar;
-            this.outputJar.getOutputDirectory().mkdirs();
+        doCompile();
+        doFinish();
+    }
+
+    private void doCompile() throws MojoExecutionException, MojoFailureException {
+        List<SourcepathEntry> sourcepath = getSourcepath();
+        if (sourcepath.isEmpty()) {
+            return;
+        }
+        Map<File, List<SourcepathEntry>> outputMap = sourcepath.stream().collect(
+                Collectors.groupingBy(SourcepathEntry::getOutputDirectory, LinkedHashMap::new, Collectors.toList()));
+
+        for (Entry<File, List<SourcepathEntry>> entry : outputMap.entrySet()) {
+            this.currentOutputDirectory = entry.getKey();
+            this.currentOutputDirectory.mkdirs();
+            this.currentSourceRoots = entry.getValue().stream().map(SourcepathEntry::getSourcesRoot)
+                    .map(root -> new File(root.toURI().normalize()).toString()).collect(Collectors.toList());
+            this.currentExcludes = entry.getValue().stream().map(SourcepathEntry::getExcludes).filter(Objects::nonNull)
+                    .flatMap(Collection::stream).distinct().collect(Collectors.toList());
             super.execute();
             doCopyResources();
         }
+        this.currentOutputDirectory = null;
+        this.currentSourceRoots = null;
+        this.currentExcludes = null;
+    }
 
-        // this does not include classes from nested jars
-        BuildOutputJar dotOutputJar = getEclipsePluginProject().getDotOutputJar();
-        if (dotOutputJar != null) {
-            project.getArtifact().setFile(dotOutputJar.getOutputDirectory());
+    /**
+     * Subclasses might override this method to perform final tasks and as a last opportunity to
+     * fail the compile
+     * 
+     * @throws MojoExecutionException
+     */
+    protected void doFinish() throws MojoExecutionException {
+        //empty
+    }
+
+    /**
+     * Only public for tests purpose!
+     */
+    public StandardExecutionEnvironment[] getBREE() {
+        if (manifestBREEs == null) {
+            OsgiManifest manifest = bundleReader.loadManifest(project.getBasedir());
+            manifestBREEs = Arrays.stream(manifest.getExecutionEnvironments())
+                    .map(ee -> ExecutionEnvironmentUtils.getExecutionEnvironment(ee, toolchainManager, session, logger))
+                    .toArray(StandardExecutionEnvironment[]::new);
+            if (manifestBREEs.length == 0) {
+                ManifestElement[] requireCapability = manifest.getManifestElements(Constants.REQUIRE_CAPABILITY);
+                if (requireCapability != null) {
+                    List<Filter> eeFilters = Arrays.stream(requireCapability)
+                            .filter(element -> ExecutionEnvironmentNamespace.EXECUTION_ENVIRONMENT_NAMESPACE
+                                    .equals(element.getValue())) //
+                            .map(element -> element.getDirective(Namespace.REQUIREMENT_FILTER_DIRECTIVE)) //
+                            .map(filterDirective -> {
+                                try {
+                                    return FrameworkUtil.createFilter(filterDirective);
+                                } catch (InvalidSyntaxException e) {
+                                    e.printStackTrace();
+                                    return null;
+                                }
+                            }).filter(Objects::nonNull).collect(Collectors.toList());
+                    manifestBREEs = ExecutionEnvironmentUtils.getProfileNames().stream() //
+                            .map(name -> name.split("-")) //
+                            .map(segments -> Map.of(ExecutionEnvironmentNamespace.EXECUTION_ENVIRONMENT_NAMESPACE,
+                                    segments[0], "version", segments[1]))
+                            .filter(eeCapability -> eeFilters.stream().anyMatch(filter -> filter.matches(eeCapability)))
+                            .map(ee -> ee.get(ExecutionEnvironmentNamespace.EXECUTION_ENVIRONMENT_NAMESPACE) + '-'
+                                    + ee.get("version"))
+                            .map(ee -> ExecutionEnvironmentUtils.getExecutionEnvironment(ee, toolchainManager, session,
+                                    logger))
+                            .toArray(StandardExecutionEnvironment[]::new);
+                }
+            }
         }
+        return manifestBREEs;
     }
 
     /*
@@ -309,7 +418,8 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
         if (!copyResources) {
             return;
         }
-        for (String sourceRoot : getCompileSourceRoots()) {
+        List<String> compileSourceRoots = getCompileSourceRoots();
+        for (String sourceRoot : compileSourceRoots) {
             // StaleSourceScanner.getIncludedSources throws IllegalStateException
             // if directory doesnt't exist
             File sourceRootFile = new File(sourceRoot);
@@ -321,21 +431,22 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
 
             Set<String> excludes = new HashSet<>();
             excludes.addAll(excludeResources);
+            excludes.addAll(getCompileSourceExcludePaths());
             excludes.addAll(getEclipsePluginProject().getBuildProperties().getBinExcludes());
             excludes.add("**/*.java");
             StaleSourceScanner scanner = new StaleSourceScanner(0L, MATCH_ALL, excludes);
             CopyMapping copyMapping = new CopyMapping();
             scanner.addSourceMapping(copyMapping);
             try {
-                scanner.getIncludedSources(sourceRootFile, this.outputJar.getOutputDirectory());
+                scanner.getIncludedSources(sourceRootFile, getOutputDirectory());
                 for (CopyMapping.SourceTargetPair sourceTargetPair : copyMapping.getSourceTargetPairs()) {
                     FileUtils.copyFile(new File(sourceRoot, sourceTargetPair.source), sourceTargetPair.target);
                 }
             } catch (InclusionScanException e) {
                 throw new MojoExecutionException("Exception while scanning for resource files in " + sourceRoot, e);
             } catch (IOException e) {
-                throw new MojoExecutionException("Exception copying resource files from " + sourceRoot + " to "
-                        + this.outputJar.getOutputDirectory(), e);
+                throw new MojoExecutionException(
+                        "Exception copying resource files from " + sourceRoot + " to " + getOutputDirectory(), e);
             }
         }
     }
@@ -347,7 +458,7 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
 
     @Override
     protected File getOutputDirectory() {
-        return outputJar.getOutputDirectory();
+        return this.currentOutputDirectory;
     }
 
     @Override
@@ -358,10 +469,23 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
                 classpath.add(location.getAbsolutePath() + toString(cpe.getAccessRules()));
             }
         }
+        if (session != null) {
+            ArtifactRepository repository = session.getLocalRepository();
+            if (repository != null) {
+                String basedir = repository.getBasedir();
+                Collection<ProjectClasspathEntry> classpathEntries = getEclipsePluginProject().getClasspathEntries();
+                for (ProjectClasspathEntry cpe : classpathEntries) {
+                    if (cpe instanceof M2ClasspathVariable) {
+                        M2ClasspathVariable cpv = (M2ClasspathVariable) cpe;
+                        classpath.add(new File(basedir, cpv.getRepositoryPath()).getAbsolutePath());
+                    }
+                }
+            }
+        }
         return classpath;
     }
 
-    private BundleProject getBundleProject() throws MojoExecutionException {
+    protected BundleProject getBundleProject() throws MojoExecutionException {
         TychoProject projectType = projectTypes.get(project.getPackaging());
         if (!(projectType instanceof BundleProject)) {
             throw new MojoExecutionException("Not a bundle project " + project.toString());
@@ -369,67 +493,29 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
         return (BundleProject) projectType;
     }
 
-    private String toString(List<AccessRule> rules) {
-        StringBuilder result = new StringBuilder(); // include all
+    private String toString(Collection<AccessRule> rules) {
+        StringJoiner result = new StringJoiner(RULE_SEPARATOR, "[", "]"); // include all
         if (rules != null) {
-            result.append("[");
             for (AccessRule rule : rules) {
-                if (result.length() > 1)
-                    result.append(RULE_SEPARATOR);
-                result.append(rule.isDiscouraged() ? "~" : "+");
-                result.append(rule.getPattern());
+                result.add((rule.isDiscouraged() ? "~" : "+") + rule.getPattern());
             }
-            if (result.length() > 1)
-                result.append(RULE_SEPARATOR);
-            result.append(RULE_EXCLUDE_ALL);
-            result.append("]");
+            result.add(RULE_EXCLUDE_ALL);
         } else {
             // include everything, not strictly necessary, but lets make this obvious
             //result.append("[+**/*]");
+            return "";
         }
         return result.toString();
     }
 
     @Override
     protected final List<String> getCompileSourceRoots() throws MojoExecutionException {
-        ArrayList<String> roots = new ArrayList<>();
-        for (File folder : outputJar.getSourceFolders()) {
-            roots.add(new File(folder.getAbsoluteFile().toURI().normalize()).toString());
-        }
-        return roots;
+        return currentSourceRoots;
     }
 
     @Override
-    public List<SourcepathEntry> getSourcepath() throws MojoExecutionException {
-        ArrayList<SourcepathEntry> entries = new ArrayList<>();
-        for (BuildOutputJar jar : getEclipsePluginProject().getOutputJars()) {
-            final File outputDirectory = jar.getOutputDirectory();
-            for (final File sourcesRoot : jar.getSourceFolders()) {
-                SourcepathEntry entry = new SourcepathEntry() {
-                    @Override
-                    public File getSourcesRoot() {
-                        return sourcesRoot;
-                    }
-
-                    @Override
-                    public File getOutputDirectory() {
-                        return outputDirectory;
-                    }
-
-                    @Override
-                    public List<String> getIncludes() {
-                        return null;
-                    }
-
-                    @Override
-                    public List<String> getExcludes() {
-                        return null;
-                    }
-                };
-                entries.add(entry);
-            }
-        }
-        return entries;
+    protected final List<String> getCompileSourceExcludePaths() throws MojoExecutionException {
+        return currentExcludes;
     }
 
     @Override
@@ -465,28 +551,46 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
     }
 
     @Override
-    protected CompilerConfiguration getCompilerConfiguration(List<String> compileSourceRoots)
-            throws MojoExecutionException, MojoFailureException {
-        CompilerConfiguration compilerConfiguration = super.getCompilerConfiguration(compileSourceRoots);
+    protected CompilerConfiguration getCompilerConfiguration(List<String> compileSourceRoots,
+            List<String> compileSourceExcludes) throws MojoExecutionException, MojoFailureException {
+        CompilerConfiguration compilerConfiguration = super.getCompilerConfiguration(compileSourceRoots,
+                compileSourceExcludes);
         if (useProjectSettings) {
             String prefsFilePath = project.getBasedir() + File.separator + PREFS_FILE_PATH;
             if (!new File(prefsFilePath).exists()) {
-                getLog().warn(
-                        "Parameter 'useProjectSettings' is set to true, but preferences file '" + prefsFilePath
-                                + "' could not be found!");
+                getLog().warn("Parameter 'useProjectSettings' is set to true, but preferences file '" + prefsFilePath
+                        + "' could not be found!");
             } else {
+                // make sure that "-properties" is the first custom argument, otherwise it's not possible to override
+                // any project setting on the command line because the last argument wins.
+                List<Entry<String, String>> copy = new ArrayList<>(
+                        compilerConfiguration.getCustomCompilerArgumentsEntries());
+                compilerConfiguration.getCustomCompilerArgumentsEntries().clear();
                 compilerConfiguration.addCompilerCustomArgument("-properties", prefsFilePath);
+                compilerConfiguration.getCustomCompilerArgumentsEntries().addAll(copy);
             }
         }
-        String encoding = getEclipsePluginProject().getBuildProperties().getJarToJavacDefaultEncodingMap()
-                .get(outputJar.getName());
-        if (encoding != null) {
-            compilerConfiguration.setSourceEncoding(encoding);
+        compilerConfiguration.setTargetVersion(getTargetLevel());
+        compilerConfiguration.setSourceVersion(getSourceLevel());
+        String releaseLevel = getReleaseLevel();
+        if (releaseLevel != null) {
+            compilerConfiguration.setReleaseVersion(releaseLevel);
         }
-        configureSourceAndTargetLevel(compilerConfiguration);
         configureJavaHome(compilerConfiguration);
         configureBootclasspathAccessRules(compilerConfiguration);
         configureCompilerLog(compilerConfiguration);
+        Collection<ProjectClasspathEntry> classpathEntries = getEclipsePluginProject().getClasspathEntries();
+        for (ProjectClasspathEntry cpe : classpathEntries) {
+            if (cpe instanceof JREClasspathEntry) {
+                JREClasspathEntry jreClasspathEntry = (JREClasspathEntry) cpe;
+                if (jreClasspathEntry.isModule()) {
+                    Collection<String> modules = jreClasspathEntry.getLimitModules();
+                    if (!modules.isEmpty()) {
+                        compilerConfiguration.addCompilerCustomArgument("--limit-modules", String.join(",", modules));
+                    }
+                }
+            }
+        }
         return compilerConfiguration;
     }
 
@@ -500,10 +604,22 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
         }
         logDirectory.mkdirs();
         String logFileName = null;
-        if (".".equals(outputJar.getName())) {
+        if (new File(project.getBuild().getOutputDirectory()).getAbsolutePath()
+                .equals(getOutputDirectory().getAbsolutePath())) {
             logFileName = "@dot";
         } else {
-            logFileName = outputJar.getName().replaceAll("/", "_");
+            String suffix = "-classes";
+            String basePath = new File(project.getBuild().getDirectory()).getAbsolutePath();
+            String subPath = getOutputDirectory().getAbsolutePath().substring(basePath.length()).replace('\\', '/');
+            if (subPath.startsWith("/")) {
+                subPath = subPath.substring(1);
+            }
+            String name = subPath.replaceAll("/", "_");
+            if (name.endsWith(suffix)) {
+                logFileName = name.substring(0, name.length() - suffix.length());
+            } else {
+                logFileName = name;
+            }
         }
         String logPath = logDirectory.getAbsolutePath();
         if (!logPath.endsWith(File.separator)) {
@@ -525,29 +641,41 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
             accessRules.addAll(getStrictBootClasspathAccessRules());
         } else {
             accessRules.add(new DefaultAccessRule("java/**", false));
-            for (String pkg : getTargetExecutionEnvironment().getSystemPackages()) {
-                accessRules.add(new DefaultAccessRule(pkg.trim().replace('.', '/') + "/*", false));
-            }
+            getTargetExecutionEnvironment().getSystemPackages().stream() //
+                    .map(systemPackage -> systemPackage.packageName) //
+                    .distinct() //
+                    .map(packageName -> packageName.trim().replace('.', '/') + "/*") //
+                    .map(accessRule -> new DefaultAccessRule(accessRule, false)) //
+                    .forEach(accessRules::add);
             // now add packages exported by framework extension bundles
-            accessRules.addAll(getBundleProject().getBootClasspathExtraAccessRules(project));
+            accessRules
+                    .addAll(getBundleProject().getBootClasspathExtraAccessRules(DefaultReactorProject.adapt(project)));
         }
-        if (accessRules.size() > 0) {
-            compilerConfiguration
-                    .addCompilerCustomArgument("org.osgi.framework.system.packages", toString(accessRules));
+        if (!accessRules.isEmpty()) {
+            compilerConfiguration.addCompilerCustomArgument("org.osgi.framework.system.packages",
+                    toString(accessRules));
         }
     }
 
-    @SuppressWarnings("unchecked")
     private List<AccessRule> getStrictBootClasspathAccessRules() throws MojoExecutionException {
-        return (List<AccessRule>) project
-                .getContextValue(TychoConstants.CTX_ECLIPSE_PLUGIN_STRICT_BOOTCLASSPATH_ACCESSRULES);
+        return ((OsgiBundleProject) getBundleProject()).getBundleClassPath(DefaultReactorProject.adapt(project))
+                .getStrictBootClasspathAccessRules();
     }
 
     private void configureJavaHome(CompilerConfiguration compilerConfiguration) throws MojoExecutionException {
         if (useJDK != JDKUsage.BREE) {
             return;
         }
-        String toolchainId = getTargetExecutionEnvironment().getProfileName();
+        StandardExecutionEnvironment[] brees = getBREE();
+        String toolchainId = null;
+        if (brees.length > 0) {
+            toolchainId = brees[0].getProfileName();
+        } else {
+            getLog().warn(
+                    "useJDK=BREE configured, but no BREE is set in bundle. Fail back to currently running execution environment ("
+                            + getTargetExecutionEnvironment().getProfileName() + ").");
+            toolchainId = getTargetExecutionEnvironment().getProfileName();
+        }
         DefaultJavaToolChain toolChain = toolchainProvider.findMatchingJavaToolChain(session, toolchainId);
         if (toolChain == null) {
             throw new MojoExecutionException("useJDK = BREE configured, but no toolchain of type 'jdk' with id '"
@@ -557,7 +685,8 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
         configureBootClassPath(compilerConfiguration, toolChain);
     }
 
-    private void configureBootClassPath(CompilerConfiguration compilerConfiguration, DefaultJavaToolChain javaToolChain) {
+    private void configureBootClassPath(CompilerConfiguration compilerConfiguration,
+            DefaultJavaToolChain javaToolChain) {
         Xpp3Dom config = (Xpp3Dom) javaToolChain.getModel().getConfiguration();
         if (config != null) {
             Xpp3Dom bootClassPath = config.getChild("bootClassPath");
@@ -566,10 +695,8 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
                 if (includeParent != null) {
                     Xpp3Dom[] includes = includeParent.getChildren("include");
                     if (includes.length > 0) {
-                        compilerConfiguration.addCompilerCustomArgument(
-                                "-bootclasspath",
-                                scanBootclasspath(javaToolChain.getJavaHome(), includes,
-                                        bootClassPath.getChild("excludes")));
+                        compilerConfiguration.addCompilerCustomArgument("-bootclasspath", scanBootclasspath(
+                                javaToolChain.getJavaHome(), includes, bootClassPath.getChild("excludes")));
                     }
                 }
             }
@@ -606,23 +733,17 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
         return values;
     }
 
-    private void configureSourceAndTargetLevel(CompilerConfiguration compilerConfiguration)
-            throws MojoExecutionException {
-        ExecutionEnvironment ee = getTargetExecutionEnvironment();
-        compilerConfiguration.setSourceVersion(getSourceLevel(ee));
-        compilerConfiguration.setTargetVersion(getTargetLevel(ee));
-    }
-
-    private ExecutionEnvironment getTargetExecutionEnvironment() throws MojoExecutionException {
+    private ExecutionEnvironment getTargetExecutionEnvironment() {
         // never null
-        return TychoProjectUtils.getExecutionEnvironmentConfiguration(project).getFullSpecification();
+        return TychoProjectUtils.getExecutionEnvironmentConfiguration(DefaultReactorProject.adapt(project))
+                .getFullSpecification();
     }
 
     @Override
     public List<ClasspathEntry> getClasspath() throws MojoExecutionException {
         TychoProject projectType = getBundleProject();
         ArrayList<ClasspathEntry> classpath = new ArrayList<>(
-                ((BundleProject) projectType).getClasspath(project));
+                ((BundleProject) projectType).getClasspath(DefaultReactorProject.adapt(project)));
 
         if (extraClasspathElements != null) {
             ArtifactRepository localRepository = session.getLocalRepository();
@@ -639,19 +760,22 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
                 ArtifactResolutionResult result = repositorySystem.resolve(request);
 
                 if (result.hasExceptions()) {
-                    throw new MojoExecutionException("Could not resolve extra classpath entry", result.getExceptions()
-                            .get(0));
+                    throw new MojoExecutionException("Could not resolve extra classpath entry",
+                            result.getExceptions().get(0));
                 }
 
                 for (Artifact b : result.getArtifacts()) {
-                    MavenProject bProject = null;
+                    ReactorProject bProject = null;
                     if (b instanceof ProjectArtifact) {
-                        bProject = ((ProjectArtifact) b).getProject();
+                        bProject = DefaultReactorProject.adapt(((ProjectArtifact) b).getProject());
                     }
                     ArrayList<File> bLocations = new ArrayList<>();
                     bLocations.add(b.getFile()); // TODO properly handle multiple project locations maybe
-                    classpath.add(new DefaultClasspathEntry(DefaultReactorProject.adapt(bProject), null, bLocations,
-                            null));
+                    classpath.add(new DefaultClasspathEntry(bProject,
+                            ((OsgiBundleProject) getBundleProject()).readOrCreateArtifactKey(b.getFile(), () -> {
+                                return new DefaultArtifactKey(b.getType(), b.getGroupId() + "." + b.getArtifactId(),
+                                        b.getVersion());
+                            }), bLocations, null));
                 }
             }
         }
@@ -665,10 +789,6 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
 
     @Override
     public String getSourceLevel() throws MojoExecutionException {
-        return getSourceLevel(getTargetExecutionEnvironment());
-    }
-
-    private String getSourceLevel(ExecutionEnvironment ee) throws MojoExecutionException {
         // first, explicit POM configuration
         if (source != null) {
             return source;
@@ -678,21 +798,21 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
         if (javacSource != null) {
             return javacSource;
         }
-        // then, execution environment
-        String eeSource = ee.getCompilerSourceLevelDefault();
-        if (eeSource != null) {
-            return eeSource;
+        String profileName = getEclipsePluginProject().getBuildProperties().getJreCompilationProfile();
+        if (profileName != null) {
+            return ExecutionEnvironmentUtils.getExecutionEnvironment(profileName, toolchainManager, session, logger)
+                    .getCompilerSourceLevelDefault();
         }
-
-        return DEFAULT_SOURCE_VERSION;
+        return Arrays.stream(getBREE()) //
+                .map(ExecutionEnvironment::getCompilerSourceLevelDefault) //
+                .filter(Objects::nonNull) //
+                .min(Comparator.comparing(Version::parseVersion)) //
+                .or(() -> Optional.ofNullable(getTargetExecutionEnvironment().getCompilerSourceLevelDefault())) //
+                .orElse(DEFAULT_SOURCE_VERSION);
     }
 
     @Override
     public String getTargetLevel() throws MojoExecutionException {
-        return getTargetLevel(getTargetExecutionEnvironment());
-    }
-
-    public String getTargetLevel(ExecutionEnvironment ee) throws MojoExecutionException {
         // first, explicit POM configuration
         if (target != null) {
             return target;
@@ -702,13 +822,67 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
         if (javacTarget != null) {
             return javacTarget;
         }
-        // then, execution environment
-        String eeTarget = ee.getCompilerTargetLevelDefault();
-        if (eeTarget != null) {
-            return eeTarget;
+        String profileName = getEclipsePluginProject().getBuildProperties().getJreCompilationProfile();
+        if (profileName != null) {
+            return ExecutionEnvironmentUtils.getExecutionEnvironment(profileName, toolchainManager, session, logger)
+                    .getCompilerTargetLevelDefault();
+        }
+        return Arrays.stream(getBREE()) //
+                .map(ExecutionEnvironment::getCompilerTargetLevelDefault) //
+                .filter(Objects::nonNull) //
+                .min(Comparator.comparing(Version::parseVersion)) //
+                .or(() -> Optional.ofNullable(getTargetExecutionEnvironment().getCompilerTargetLevelDefault())) //
+                .orElse(DEFAULT_TARGET_VERSION);
+    }
+
+    @Override
+    public String getReleaseLevel() throws MojoExecutionException {
+        // first, explicit POM configuration
+        if (release != null) {
+            return release;
         }
 
-        return DEFAULT_TARGET_VERSION;
+        // implicit determination disabled
+        if (!deriveReleaseCompilerArgumentFromTargetLevel) {
+            return null;
+        }
+
+        String targetLevel = getTargetLevel();
+        String[] targetLevelSplit = targetLevel.split("\\.");
+
+        String releaseLevel;
+        if (targetLevelSplit.length == 1 && targetLevelSplit[0].matches("\\d+")) {
+            releaseLevel = targetLevelSplit[0];
+        } else if (targetLevelSplit.length == 2 && "1".equals(targetLevelSplit[0])
+                && targetLevelSplit[1].matches("\\d+")) {
+            releaseLevel = targetLevelSplit[1];
+        } else {
+            logger.debug("Cannot determining 'maven.compiler.release' property automatically, because target level '"
+                    + targetLevel + "' has an unexpected format.");
+            return null;
+        }
+
+        CtSym ctSym;
+        try {
+            ctSym = JRTUtil.getCtSym(Paths.get(System.getProperty("java.home")));
+        } catch (IOException e) {
+            logger.warn("Unable to determine 'maven.compiler.release' property automatically", e);
+            return null;
+        }
+
+        // TODO: Replace this with CtSym#getReleaseCode(String) once eclipse.jdt.core/5ba272a2d4a7478e0eb3951208ab49b7c069f37d is available with newer ECJ release
+        int releaseLevelInt = Integer.parseInt(releaseLevel);
+        String releaseCode = releaseLevelInt < 10 ? releaseLevel
+                : String.valueOf((char) ('A' + (releaseLevelInt - 10)));
+
+        List<Path> releaseRoots = ctSym.releaseRoots(releaseCode);
+        if (!releaseRoots.isEmpty()) {
+            return releaseLevel;
+        } else {
+            logger.debug("Not determining 'maven.compiler.release' property automatically, because level '"
+                    + releaseLevel + "' is not supported by compiler.");
+            return null;
+        }
     }
 
     private void checkTargetLevelCompatibleWithManifestBREEs(String effectiveTargetLevel,
@@ -737,4 +911,5 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
         }
         return null;
     }
+
 }
