@@ -13,12 +13,16 @@ package org.eclipse.tycho.osgicompiler.test;
 import static java.util.Arrays.asList;
 import static org.hamcrest.CoreMatchers.containsString;
 import static org.hamcrest.CoreMatchers.not;
-import static org.junit.Assert.assertThat;
+import static org.hamcrest.MatcherAssert.assertThat;
 
 import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
 import java.util.List;
+import java.util.Set;
+import java.util.stream.Collectors;
 
 import org.apache.bcel.classfile.ClassFormatException;
 import org.apache.bcel.classfile.ClassParser;
@@ -30,14 +34,17 @@ import org.apache.maven.project.MavenProject;
 import org.codehaus.plexus.util.FileUtils;
 import org.eclipse.tycho.classpath.SourcepathEntry;
 import org.eclipse.tycho.compiler.AbstractOsgiCompilerMojo;
+import org.eclipse.tycho.core.ee.StandardExecutionEnvironment;
 import org.eclipse.tycho.testing.AbstractTychoMojoTestCase;
+import org.hamcrest.Matchers;
+import org.hamcrest.core.StringContains;
 
 import copied.org.apache.maven.plugin.CompilationFailureException;
 
 public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     private static final int TARGET_1_4 = 48;
-    private static final int TARGET_9 = 53;
+    private static final int TARGET_1_8 = 52;
 
     protected File storage;
 
@@ -49,9 +56,9 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
     }
 
     private AbstractOsgiCompilerMojo getMojo(List<MavenProject> projects, MavenProject project) throws Exception {
-        AbstractOsgiCompilerMojo mojo = (AbstractOsgiCompilerMojo) lookupMojo("compile", project.getFile());
-        setVariableValueToObject(mojo, "project", project);
-        setVariableValueToObject(mojo, "session", newMavenSession(project, projects));
+        AbstractOsgiCompilerMojo mojo = (AbstractOsgiCompilerMojo) lookupConfiguredMojo(project, "compile");
+//        setVariableValueToObject(mojo, "project", project);
+//        setVariableValueToObject(mojo, "session", newMavenSession(project, projects));
 
         // tycho-compiler-jdt does not support forked compilation
 //		        setVariableValueToObject(mojo, "fork", fork? Boolean.TRUE: Boolean.FALSE);
@@ -60,7 +67,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void testAccessRestrictionCompilationError() throws Exception {
         File basedir = getBasedir("projects/accessrules");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
 
         try {
             for (MavenProject project : projects) {
@@ -76,7 +83,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void testAccessRulesClasspath() throws Exception {
         File basedir = getBasedir("projects/accessrules");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
 
         getMojo(projects, projects.get(1)).execute();
         getMojo(projects, projects.get(2)).execute();
@@ -150,7 +157,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void test_multisourceP001_viaMojoConfiguration() throws Exception {
         File basedir = getBasedir("projects/multisource/p001");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
 
         MavenProject project = projects.get(0);
         getMojo(projects, project).execute();
@@ -161,7 +168,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void test_multisourceP002_viaBuildProperties() throws Exception {
         File basedir = getBasedir("projects/multisource/p002");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
 
         MavenProject project = projects.get(0);
         getMojo(projects, project).execute();
@@ -172,7 +179,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void test_multipleOutputJars() throws Exception {
         File basedir = getBasedir("projects/multijar");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
 
         MavenProject project = projects.get(0);
         getMojo(projects, project).execute();
@@ -186,7 +193,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void test_multipleOutputJars_getSourcepath() throws Exception {
         File basedir = getBasedir("projects/multijar");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
 
         MavenProject project = projects.get(0);
 
@@ -205,7 +212,16 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void testCopyResources() throws Exception {
         File basedir = getBasedir("projects/resources/p001");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
+        MavenProject project = projects.get(0);
+        getMojo(projects, project).execute();
+        assertTrue(new File(project.getBasedir(), "target/classes/testresources/Test.class").canRead());
+        assertTrue(new File(project.getBasedir(), "target/classes/testresources/test.properties").canRead());
+    }
+
+    public void testCopyResourcesWithNestedJar() throws Exception {
+        File basedir = getBasedir("projects/resources/p004");
+        List<MavenProject> projects = getSortedProjects(basedir);
         MavenProject project = projects.get(0);
         getMojo(projects, project).execute();
         assertTrue(new File(project.getBasedir(), "target/classes/testresources/Test.class").canRead());
@@ -214,7 +230,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void testExcludeCopyResources() throws Exception {
         File basedir = getBasedir("projects/resources/p002");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
         MavenProject project = projects.get(0);
         getMojo(projects, project).execute();
         assertTrue(new File(project.getBasedir(), "target/classes/testresources/Test.class").canRead());
@@ -223,28 +239,28 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void testCopyResourcesWithResourceCopyingSetToOff() throws Exception {
         File basedir = getBasedir("projects/resources/p003");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
         MavenProject project = projects.get(0);
         getMojo(projects, project).execute();
         assertTrue(new File(project.getBasedir(), "target/classes/testresources/Test.class").exists());
         assertFalse(new File(project.getBasedir(), "target/classes/testresources/test.properties").exists());
     }
 
-    public void testExecutionEnvironment() throws Exception {
+    public void testSourceCompileLevel() throws Exception {
         File basedir = getBasedir("projects/executionEnvironment");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
         MavenProject project;
         // project with neither POM nor MANIFEST configuration => must fallback to 
-        // source/target level == 9
+        // source/target level == 11
         project = projects.get(1);
         getMojo(projects, project).execute();
-        assertBytecodeMajorLevel(TARGET_9, new File(project.getBasedir(), "target/classes/Generic.class"));
+        assertBytecodeMajorLevel(55 /* Java 11 */, new File(project.getBasedir(), "target/classes/Generic.class"));
 
         // project with multiple execution envs.
         // Minimum source and target level must be taken
         project = projects.get(2);
         AbstractOsgiCompilerMojo mojo = getMojo(projects, project);
-        assertEquals("OSGi/Minimum-1.0", mojo.getExecutionEnvironment());
+        assertEquals("1.3", mojo.getSourceLevel());
         try {
             mojo.execute();
             fail("compilation failure due to assert keyword expected");
@@ -257,25 +273,61 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
         mojo = getMojo(projects, project);
         assertEquals("jsr14", mojo.getTargetLevel());
         assertEquals("1.5", mojo.getSourceLevel());
-        assertEquals("J2SE-1.5", mojo.getExecutionEnvironment());
         mojo.execute();
         assertBytecodeMajorLevel(TARGET_1_4, new File(project.getBasedir(), "target/classes/Generic.class"));
         // project with both explicit EE configuration in pom.xml and Bundle-RequiredExecutionEnvironment.
         // explicit configuration in the pom.xml win
         project = projects.get(4);
         mojo = getMojo(projects, project);
-        assertEquals("J2SE-1.5", mojo.getExecutionEnvironment());
+        mojo.execute();
+        assertEquals("1.3", mojo.getSourceLevel());
         // project with both explicit compiler configuration in build.properties and Bundle-RequiredExecutionEnvironment. 
         // build.properties should win. 
         project = projects.get(5);
         mojo = getMojo(projects, project);
+        mojo.execute();
         assertEquals("jsr14", mojo.getTargetLevel());
         assertEquals("1.5", mojo.getSourceLevel());
-        assertEquals("J2SE-1.5", mojo.getExecutionEnvironment());
-        mojo.execute();
         assertBytecodeMajorLevel(TARGET_1_4, new File(project.getBasedir(), "target/classes/Generic.class"));
     }
 
+    public void testNewerEEthanBREE() throws Exception {
+        File basedir = getBasedir("projects/executionEnvironment/p006-newerEEthanBREE");
+        List<MavenProject> projects = getSortedProjects(basedir);
+        MavenProject project = projects.get(0);
+        AbstractOsgiCompilerMojo mojo = getMojo(projects, project);
+        mojo.execute();
+        assertTrue(Integer.parseInt(mojo.getExecutionEnvironment().substring("JavaSE-".length())) >= 11);
+        assertEquals("1.8", mojo.getSourceLevel());
+        assertEquals("1.8", mojo.getTargetLevel());
+        assertEquals("8", mojo.getReleaseLevel());
+        assertBytecodeMajorLevel(TARGET_1_8, new File(project.getBasedir(), "target/classes/Noop.class"));
+    }
+
+    public void testNoBREEButEERequirement() throws Exception {
+        File basedir = getBasedir("projects/executionEnvironment/eeAsRequirement");
+        List<MavenProject> projects = getSortedProjects(basedir);
+        MavenProject project = projects.get(0);
+        AbstractOsgiCompilerMojo mojo = getMojo(projects, project);
+        StandardExecutionEnvironment[] ees = mojo.getBREE();
+        assertEquals(Set.of("JavaSE-1.8", "JavaSE/compact1-1.8"),
+                Arrays.stream(ees).map(StandardExecutionEnvironment::getProfileName).collect(Collectors.toSet()));
+    }
+
+    public void testAutomaticReleaseCompilerArgumentDeterminationDisabled() throws Exception {
+        File basedir = getBasedir(
+                "projects/executionEnvironment/p007-automaticReleaseCommpilerArgumentDeterminationDisabled");
+        List<MavenProject> projects = getSortedProjects(basedir);
+        MavenProject project = projects.get(0);
+        AbstractOsgiCompilerMojo mojo = getMojo(projects, project);
+        mojo.execute();
+        assertTrue(Integer.parseInt(mojo.getExecutionEnvironment().substring("JavaSE-".length())) >= 11);
+        assertEquals("1.8", mojo.getSourceLevel());
+        assertEquals("1.8", mojo.getTargetLevel());
+        assertNull(mojo.getReleaseLevel());
+        assertBytecodeMajorLevel(TARGET_1_8, new File(project.getBasedir(), "target/classes/Noop.class"));
+    }
+
     private void assertBytecodeMajorLevel(int majorLevel, File classFile) throws ClassFormatException, IOException {
         assertTrue(classFile.canRead());
         JavaClass javaClass = new ClassParser(classFile.getAbsolutePath()).parse();
@@ -284,7 +336,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void test_TYCHO0400indirectDependencies() throws Exception {
         File basedir = getBasedir("projects/indirectDependencies");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
 
         assertEquals("C", projects.get(1).getArtifactId());
         getMojo(projects, projects.get(1)).execute();
@@ -303,7 +355,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void test_embeddedNonClasspath() throws Exception {
         File basedir = getBasedir("projects/embedednonclasspath");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
 
         MavenProject project = projects.get(0);
         getMojo(projects, project).execute();
@@ -317,7 +369,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void test_bootclasspathAccessRules() throws Exception {
         File basedir = getBasedir("projects/bootclasspath-accessrules");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
 
         MavenProject project = projects.get(0);
         getMojo(projects, project).execute();
@@ -325,7 +377,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void testWarningAndErrorMessages() throws Exception {
         File basedir = getBasedir("projects/compilermessages");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
         MavenProject project = projects.get(0);
         AbstractOsgiCompilerMojo mojo = getMojo(projects, project);
         final List<CharSequence> warnings = new ArrayList<>();
@@ -360,9 +412,12 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void testCompilerArgs() throws Exception {
         File basedir = getBasedir("projects/compiler-args");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
         MavenProject project = projects.get(0);
         AbstractOsgiCompilerMojo mojo = getMojo(projects, project);
+
+        setVariableValueToObject(mojo, "showWarnings", Boolean.TRUE);
+
         final List<CharSequence> warnings = new ArrayList<>();
         mojo.setLog(new SystemStreamLog() {
 
@@ -389,10 +444,11 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
         // the code in the project does use boxing and the settings file 
         // turns on warning for auto boxing so we expect here a warning
         File basedir = getBasedir("projects/projectSettings/p001");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
         MavenProject project = projects.get(0);
         AbstractOsgiCompilerMojo mojo = getMojo(projects, project);
         setVariableValueToObject(mojo, "useProjectSettings", Boolean.TRUE);
+        setVariableValueToObject(mojo, "showWarnings", Boolean.TRUE);
         final List<CharSequence> warnings = new ArrayList<>();
         mojo.setLog(new SystemStreamLog() {
 
@@ -408,7 +464,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void testUseProjectSettingsSetToFalse() throws Exception {
         File basedir = getBasedir("projects/projectSettings/p001");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
         MavenProject project = projects.get(0);
         AbstractOsgiCompilerMojo mojo = getMojo(projects, project);
         setVariableValueToObject(mojo, "useProjectSettings", Boolean.FALSE);
@@ -427,7 +483,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void testUseProjectSettingsSetToTrueWithMissingPrefsFile() throws Exception {
         File basedir = getBasedir("projects/projectSettings/p002");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
         MavenProject project = projects.get(0);
         AbstractOsgiCompilerMojo mojo = getMojo(projects, project);
         setVariableValueToObject(mojo, "useProjectSettings", Boolean.TRUE);
@@ -446,6 +502,10 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
     }
 
     public void test367431_frameworkExtensionCompileAccessRules() throws Exception {
+        // This test assumes that the internal class com.sun.crypto.provider.SunJCE exists and its package is not exported.
+        // This is the case for all supported JDKs to date (1.8, 11, 14).
+        // Note: The bundle uses BREE 1.8 here, because apparently this kind of framework-extension does not
+        // correctly work with modular API (Java9+).
         File basedir = getBasedir("projects/367431_frameworkExtensionCompileAccessRules/bundle");
         List<MavenProject> projects = getSortedProjects(basedir,
                 new File("src/test/resources/projects/367431_frameworkExtensionCompileAccessRules/repository"));
@@ -456,7 +516,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void testBreeCompilerTargetCompatibilityIsChecked() throws Exception {
         File basedir = getBasedir("projects/bree-target-compatibility");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
 
         MavenProject project = projects.get(0);
         try {
@@ -475,7 +535,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void test386210_compilerConfigurationCrosstalk() throws Exception {
         File basedir = getBasedir("projects/crosstalk");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
 
         getMojo(projects, projects.get(1)).execute();
         getMojo(projects, projects.get(2)).execute();
@@ -483,7 +543,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void testCompilerLogWithMultiJarInSingleDirectory() throws Exception {
         File basedir = getBasedir("projects/logs/multiJarSingleDir");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
         MavenProject project = projects.get(0);
         lookupConfiguredMojo(project, "compile").execute();
         assertTrue(new File(basedir, "target/log-dir/@dot.log").canRead());
@@ -492,7 +552,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void testCompilerLogWithMultiJarInSubDirectory() throws Exception {
         File basedir = getBasedir("projects/logs/multiJarMultiDir");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
         MavenProject project = projects.get(0);
         lookupConfiguredMojo(project, "compile").execute();
         assertTrue(new File(basedir, "target/log-dir/@dot.log").canRead());
@@ -501,7 +561,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void testCompilerLogWithSingleJar() throws Exception {
         File basedir = getBasedir("projects/logs/singleJar");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
         MavenProject project = projects.get(0);
         lookupConfiguredMojo(project, "compile").execute();
         assertTrue(new File(basedir, "target/log-dir/@dot.xml").canRead());
@@ -509,7 +569,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void testCompilerLogWithCustomComilerArgs() throws Exception {
         File basedir = getBasedir("projects/logs/customCompilerArgs");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
         MavenProject project = projects.get(0);
         lookupConfiguredMojo(project, "compile").execute();
         assertTrue(new File(basedir, "target/@dot.xml").canRead());
@@ -517,7 +577,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     public void testCompilerLogWithCustomComilerArgsAndLog() throws Exception {
         File basedir = getBasedir("projects/logs/customCompilerArgsAndLog");
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+        List<MavenProject> projects = getSortedProjects(basedir);
         MavenProject project = projects.get(0);
         try {
             lookupConfiguredMojo(project, "compile").execute();
@@ -527,4 +587,29 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
                     + " plugin parameter and the custom compiler argument '-log'. Only either of them is allowed."));
         }
     }
+
+    public void testJreCompilationProfile() throws Exception {
+        File basedir = getBasedir("projects/jreCompilationProfile");
+        List<MavenProject> projects = getSortedProjects(basedir);
+        MavenProject project = projects.get(0);
+        AbstractOsgiCompilerMojo mojo = getMojo(Collections.singletonList(project), project);
+        mojo.execute();
+        assertEquals("1.8", mojo.getSourceLevel());
+        assertEquals("1.8", mojo.getTargetLevel());
+        assertBytecodeMajorLevel(TARGET_1_8, new File(project.getBasedir(), "target/classes/Test.class"));
+    }
+
+    public void testUseJDKBREE() throws Exception {
+        File basedir = getBasedir("projects/useJDKBREE");
+        List<MavenProject> projects = getSortedProjects(basedir);
+        MavenProject project = projects.get(0);
+        AbstractOsgiCompilerMojo mojo = getMojo(Collections.singletonList(project), project);
+        try {
+            mojo.execute();
+            fail("Mojo should fail because of missing toolchains");
+        } catch (MojoExecutionException ex) {
+            assertThat(ex.getMessage(), Matchers.allOf(StringContains.containsStringIgnoringCase("toolchain"),
+                    StringContains.containsString("JavaSE-1.8")));
+        }
+    }
 }
