@@ -4,7 +4,7 @@ import java.security.Security;
 
 public class TestClass {
     public TestClass() {
-        Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());
+        Security.addProvider(new com.sun.crypto.provider.SunJCE());
 
     }
 }
