@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2011 Sonatype Inc. and others.
+ * Copyright (c) 2008, 2020 Sonatype Inc. and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -14,8 +14,6 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.util.Properties;
 
-import org.codehaus.plexus.util.IOUtil;
-
 public class TychoVersion {
 
     private static final String TYCHO_VERSION = readVersion();
@@ -25,15 +23,12 @@ public class TychoVersion {
     }
 
     private static String readVersion() {
-        InputStream stream = TychoVersion.class.getResourceAsStream("version.properties");
-        try {
+        try (InputStream stream = TychoVersion.class.getResourceAsStream("version.properties")) {
             Properties p = new Properties();
             p.load(stream);
             return p.getProperty("version");
         } catch (IOException e) {
             throw new IllegalStateException(e);
-        } finally {
-            IOUtil.close(stream);
         }
     }
 
