@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2014 Sonatype Inc. and others.
+ * Copyright (c) 2008, 2020 Sonatype Inc. and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -13,9 +13,9 @@ package org.eclipse.tycho.core.test;
 import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.List;
-import java.util.Properties;
+import java.util.Set;
+import java.util.stream.Collectors;
 
 import org.apache.maven.execution.MavenSession;
 import org.apache.maven.model.Dependency;
@@ -27,12 +27,11 @@ import org.eclipse.tycho.artifacts.DependencyArtifacts;
 import org.eclipse.tycho.classpath.ClasspathEntry;
 import org.eclipse.tycho.core.TargetPlatformConfiguration;
 import org.eclipse.tycho.core.TychoProject;
+import org.eclipse.tycho.core.osgitools.DefaultReactorProject;
 import org.eclipse.tycho.core.osgitools.OsgiBundleProject;
 import org.eclipse.tycho.core.resolver.DefaultTargetPlatformConfigurationReader;
 import org.eclipse.tycho.core.resolver.TargetPlatformConfigurationException;
 import org.eclipse.tycho.core.shared.TargetEnvironment;
-import org.eclipse.tycho.core.utils.TychoProjectUtils;
-import org.eclipse.tycho.core.utils.TychoVersion;
 import org.eclipse.tycho.testing.AbstractTychoMojoTestCase;
 
 public class TychoTest extends AbstractTychoMojoTestCase {
@@ -57,10 +56,10 @@ public class TychoTest extends AbstractTychoMojoTestCase {
         List<MavenProject> projects = getSortedProjects(basedir);
         assertEquals(5, projects.size());
 
-        MavenProject p002 = (MavenProject) projects.get(1);
-        MavenProject p001 = (MavenProject) projects.get(2);
-        MavenProject p004 = (MavenProject) projects.get(3); // feature
-        MavenProject p003 = (MavenProject) projects.get(4); // site
+        MavenProject p002 = projects.get(1);
+        MavenProject p001 = projects.get(2);
+        MavenProject p004 = projects.get(3); // feature
+        MavenProject p003 = projects.get(4); // site
 
         assertEquals("moduleorder.p001", p001.getArtifactId());
         assertEquals("moduleorder.p002", p002.getArtifactId());
@@ -77,7 +76,7 @@ public class TychoTest extends AbstractTychoMojoTestCase {
         } catch (Exception e) {
 //	        List<Exception> exceptions = result.getExceptions();
 //	        assertEquals(1, exceptions.size());
-            assertTrue(e.getMessage().contains("Missing Constraint: Import-Package: moduleorder.p002"));
+            assertTrue(e.getMessage().contains("Unresolved requirement: Import-Package: moduleorder.p002"));
         }
     }
 
@@ -124,40 +123,36 @@ public class TychoTest extends AbstractTychoMojoTestCase {
         for (MavenProject project : projects) {
             artifactIds.add(project.getArtifactId());
         }
-        assertEquals(Arrays.asList("parent", "host", "dep", "fragment", "fragment2", "client"), artifactIds);
+        assertEquals(0, artifactIds.indexOf("parent"));
+        assertEquals(artifactIds.size() - 1, artifactIds.indexOf("client"));
+        assertTrue(artifactIds.indexOf("host") < artifactIds.indexOf("fragment2"));
+        assertTrue(artifactIds.indexOf("host") < artifactIds.indexOf("fragment")
+                && artifactIds.indexOf("dep") < artifactIds.indexOf("fragment"));
 
-        MavenProject host = projects.get(1);
-        MavenProject dep = projects.get(2);
-        MavenProject fragment = projects.get(3);
-        MavenProject fragment2 = projects.get(4);
-        MavenProject client = projects.get(5);
-
-        assertEquals("host", host.getArtifactId());
         // host does not know anything about fragments
+        MavenProject host = projects.stream().filter(p -> p.getArtifactId().equals("host")).findAny().get();
         List<Dependency> hostDependencies = host.getModel().getDependencies();
         assertEquals(0, hostDependencies.size());
 
-        assertEquals("fragment", fragment.getArtifactId());
+        MavenProject fragment = projects.stream().filter(p -> p.getArtifactId().equals("fragment")).findAny().get();
         List<Dependency> fragmentDependencies = fragment.getModel().getDependencies();
         // host first, then fragment dependency
         assertEquals(2, fragmentDependencies.size());
         assertEquals("host", fragmentDependencies.get(0).getArtifactId());
         assertEquals("dep", fragmentDependencies.get(1).getArtifactId());
 
-        assertEquals("fragment2", fragment2.getArtifactId());
         // host only
+        MavenProject fragment2 = projects.stream().filter(p -> p.getArtifactId().equals("fragment2")).findAny().get();
         List<Dependency> fragment2Dependencies = fragment2.getModel().getDependencies();
         assertEquals(1, fragment2Dependencies.size());
         assertEquals("host", fragment2Dependencies.get(0).getArtifactId());
 
-        assertEquals("client", client.getArtifactId());
         // depends on host and because host has ExtensibleAPI also depends fragment and fragent2
+        MavenProject client = projects.stream().filter(p -> p.getArtifactId().equals("client")).findAny().get();
         List<Dependency> clientDependencies = client.getModel().getDependencies();
         assertEquals(4, clientDependencies.size());
-        assertEquals("host", clientDependencies.get(0).getArtifactId());
-        assertEquals("fragment", clientDependencies.get(1).getArtifactId());
-        assertEquals("dep", clientDependencies.get(2).getArtifactId());
-        assertEquals("fragment2", clientDependencies.get(3).getArtifactId());
+        assertEquals(Set.of("host", "fragment", "dep", "fragment2"),
+                clientDependencies.stream().map(Dependency::getArtifactId).collect(Collectors.toSet()));
     }
 
     public void testMNGECLIPSE942() throws Exception {
@@ -166,7 +161,7 @@ public class TychoTest extends AbstractTychoMojoTestCase {
         File platformLocation = new File("src/test/resources/targetplatforms/MNGECLIPSE-942");
         MavenProject project = getSortedProjects(basedir, platformLocation).get(0);
         TychoProject projectType = lookup(TychoProject.class, project.getPackaging());
-        DependencyArtifacts platform = projectType.getDependencyArtifacts(project);
+        DependencyArtifacts platform = projectType.getDependencyArtifacts(DefaultReactorProject.adapt(project));
 
         assertEquals(2, platform.getArtifacts(ArtifactType.TYPE_ECLIPSE_PLUGIN).size());
         assertNotNull(platform.getArtifact(ArtifactType.TYPE_ECLIPSE_PLUGIN, "org.junit4.nl_ru", null));
@@ -180,7 +175,7 @@ public class TychoTest extends AbstractTychoMojoTestCase {
 
         OsgiBundleProject projectType = (OsgiBundleProject) lookup(TychoProject.class, project.getPackaging());
 
-        List<ClasspathEntry> classpath = projectType.getClasspath(project);
+        List<ClasspathEntry> classpath = projectType.getClasspath(DefaultReactorProject.adapt(project));
 
         assertEquals(3, classpath.size());
         assertEquals(1, classpath.get(0).getLocations().size());
@@ -202,38 +197,33 @@ public class TychoTest extends AbstractTychoMojoTestCase {
         List<MavenProject> projects = getSortedProjects(basedir, platformLocation);
         assertEquals(3, projects.size());
 
-        MavenProject b01 = projects.get(1);
         MavenProject b02 = projects.get(2);
 
         OsgiBundleProject projectType = (OsgiBundleProject) lookup(TychoProject.class, b02.getPackaging());
 
-        List<ClasspathEntry> classpath = projectType.getClasspath(b02);
+        List<ClasspathEntry> classpath = projectType.getClasspath(DefaultReactorProject.adapt(b02));
 
-        assertEquals(5, classpath.size());
+        assertClasspathContains(classpath, canonicalFile(
+                "target/local-repo/.cache/tycho/org.eclipse.equinox.launcher_1.0.101.R34x_v20081125.jar/launcher.properties"));
+        assertClasspathContains(classpath,
+                canonicalFile("target/projects/extraclasspath/b01/target/lib/nested.jar-classes"));
+        assertClasspathContains(classpath, canonicalFile(
+                "src/test/resources/targetplatforms/basic/plugins/org.eclipse.equinox.launcher_1.0.101.R34x_v20081125.jar"));
+        assertClasspathContains(classpath, new File(basedir, "b02/classes"));
+        assertClasspathContains(classpath, canonicalFile("target/projects/extraclasspath/b02/target/classes"));
+    }
 
-        // reference to external bundle entry not on classpath
-        assertEquals(1, classpath.get(0).getLocations().size());
-        assertEquals(canonicalFile(
-                "target/local-repo/.cache/tycho/org.eclipse.equinox.launcher_1.0.101.R34x_v20081125.jar/launcher.properties"),
-                classpath.get(0).getLocations().get(0).getCanonicalFile());
-        // reference to reactor project entry
-        assertEquals(1, classpath.get(1).getLocations().size());
-        assertEquals(canonicalFile("target/projects/extraclasspath/b01/target/lib/nested.jar-classes"),
-                classpath.get(1).getLocations().get(0).getCanonicalFile());
+    private void assertClasspathContains(List<ClasspathEntry> classpath, File file) throws IOException {
+        for (ClasspathEntry cpe : classpath) {
+            for (File cpfile : cpe.getLocations()) {
+                if (cpfile.getCanonicalFile().equals(file.getCanonicalFile())) {
+                    return;
+                }
+            }
+        }
+
+        fail("File " + file + " not found on the classpath");
 
-        // reference to external bundle
-        assertEquals(1, classpath.get(2).getLocations().size());
-        assertEquals(canonicalFile(
-                "src/test/resources/targetplatforms/basic/plugins/org.eclipse.equinox.launcher_1.0.101.R34x_v20081125.jar"),
-                classpath.get(2).getLocations().get(0).getCanonicalFile());
-        // reference to project local folder
-        assertEquals(1, classpath.get(3).getLocations().size());
-        assertEquals(new File(basedir, "b02/classes").getCanonicalFile(),
-                classpath.get(3).getLocations().get(0).getCanonicalFile());
-        // this bundle
-        assertEquals(1, classpath.get(4).getLocations().size());
-        assertEquals(canonicalFile("target/projects/extraclasspath/b02/target/classes"),
-                classpath.get(4).getLocations().get(0).getCanonicalFile());
     }
 
     public void testImplicitTargetEnvironment() throws Exception {
@@ -353,32 +343,6 @@ public class TychoTest extends AbstractTychoMojoTestCase {
         }
     }
 
-    public void testBundleRuntimeExecutionEnvironment() throws Exception {
-        File basedir = getBasedir("projects/bree");
-
-        Properties properties = new Properties();
-        properties.put("tycho-version", TychoVersion.getTychoVersion());
-
-        List<MavenProject> projects = getSortedProjects(basedir, properties, null);
-        assertEquals(6, projects.size());
-        int i = 0;
-
-        assertEquals("executionenvironment.manifest", projects.get(++i).getArtifactId());
-        assertEquals("CDC-1.1/Foundation-1.1", getActiveEEProfile(projects.get(i)));
-
-        assertEquals("executionenvironment.manifest-minimal", projects.get(++i).getArtifactId());
-        assertEquals("CDC-1.0/Foundation-1.0", getActiveEEProfile(projects.get(i)));
-
-        assertEquals("executionenvironment.pom-hard", projects.get(++i).getArtifactId());
-        assertEquals("OSGi/Minimum-1.2", getActiveEEProfile(projects.get(i)));
-
-        assertEquals("executionenvironment.buildproperties", projects.get(++i).getArtifactId());
-        assertEquals("OSGi/Minimum-1.1", getActiveEEProfile(projects.get(i)));
-
-        assertEquals("executionenvironment.pom-default", projects.get(++i).getArtifactId());
-        assertEquals("OSGi/Minimum-1.2", getActiveEEProfile(projects.get(i)));
-    }
-
     public void testWithProjectReferencesItself() throws Exception {
         File basedir = getBasedir("projects/referencesItself");
         try {
@@ -390,7 +354,4 @@ public class TychoTest extends AbstractTychoMojoTestCase {
 
     }
 
-    private static String getActiveEEProfile(MavenProject project) {
-        return TychoProjectUtils.getExecutionEnvironmentConfiguration(project).getProfileName();
-    }
 }
