@@ -1,9 +1,11 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2011 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2008, 2018 Sonatype Inc. and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -21,7 +23,8 @@ public class PomDependencyConsiderExtraClasspathTest extends AbstractTychoIntegr
     @Test
     public void test() throws Exception {
         Verifier verifier = getVerifier("/TYCHO0404pomDependencyConsiderExtraClasspath", false);
-        FileUtils.deleteDirectory(new File(verifier.localRepo, "TYCHO0404pomDependencyConsiderExtraClasspath"));
+        FileUtils.deleteDirectory(
+                new File(verifier.getLocalRepository(), "TYCHO0404pomDependencyConsiderExtraClasspath"));
         verifier.executeGoal("integration-test");
         verifier.verifyErrorFreeLog();
     }
