@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2013 SAP AG and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     SAP AG - initial API and implementation
@@ -31,4 +33,18 @@ public class MavenCompilerPluginTest extends AbstractTychoIntegrationTest {
         }
     }
 
+    @Test
+    public void testAdditionalBundles() throws Exception {
+        Verifier verifier = getVerifier("compiler.additional.bundles", true);
+        verifier.executeGoal("compile");
+        verifier.verifyErrorFreeLog();
+    }
+
+    @Test
+    public void testLimitModules() throws Exception {
+        Verifier verifier = getVerifier("compiler.limit.modules", true);
+        verifier.executeGoal("compile");
+        verifier.verifyErrorFreeLog();
+    }
+
 }
