@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2011 SAP AG and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     SAP AG - initial API and implementation
@@ -16,10 +18,12 @@ import static org.junit.Assert.fail;
 import org.apache.maven.it.VerificationException;
 import org.apache.maven.it.Verifier;
 import org.eclipse.tycho.test.AbstractTychoIntegrationTest;
+import org.junit.Ignore;
 import org.junit.Test;
 
 public class RequireJREPackagesImportTest extends AbstractTychoIntegrationTest {
 
+    @Ignore("TODO bug 514471 check if we can re-enable strict access rules for JDK packages")
     @Test
     public void testStrictImportJREPackages() throws Exception {
         Verifier verifier = getVerifier("compiler.requireJREPackageImports", false);
