@@ -170,7 +170,7 @@ public class BaselineValidateAndReplaceTest extends AbstractTychoIntegrationTest
     @Test
     public void testBaselineFailCommon_Changed_ignoredFiles() throws Exception {
         Verifier verifier = getVerifier("contentchanged", baselineRepo);
-        verifier.getCliOptions().addAll(Arrays.asList(new String[] { "--projects", "bundle01" }));
+        verifier.getCliOptions().addAll(Arrays.asList("--projects", "bundle01"));
         verifier.getCliOptions().add("-PignoreChanged");
         verifier.getCliOptions().add("-Dtycho.baseline=failCommon");
         verifier.executeGoals(Arrays.asList("clean", "package"));
