@@ -1,18 +1,20 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2011 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2008, 2018 Sonatype Inc. and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
  *******************************************************************************/
 package org.eclipse.tycho.test.pomGenerator;
 
-import java.io.File;
+import static org.junit.Assert.assertTrue;
 
-import junit.framework.Assert;
+import java.io.File;
 
 import org.apache.maven.it.Verifier;
 import org.eclipse.tycho.core.utils.TychoVersion;
@@ -25,12 +27,12 @@ public class Tycho122GeneratePomFileTest extends AbstractTychoIntegrationTest {
         Verifier verifier = getVerifier("/pomGenerator/tycho.demo", false);
 
         verifier.setAutoclean(false);
-        verifier.executeGoal("org.eclipse.tycho:tycho-pomgenerator-plugin:" + TychoVersion.getTychoVersion()
-                + ":generate-poms");
+        verifier.executeGoal(
+                "org.eclipse.tycho:tycho-pomgenerator-plugin:" + TychoVersion.getTychoVersion() + ":generate-poms");
         verifier.verifyErrorFreeLog();
 
         File pom = new File(verifier.getBasedir(), "pom.xml");
-        Assert.assertTrue("Must generate the pom.xml", pom.exists());
+        assertTrue("Must generate the pom.xml", pom.exists());
 
         // TODO the content of the pom.xml is broken (bug 363908)
     }
