@@ -1,9 +1,11 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2018 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2008, 2021 Sonatype Inc. and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -25,8 +27,7 @@ public class ExtraCompilerRequirementsTest extends AbstractTychoIntegrationTest
         // TODO this is an artificial test case - find a test closer to a real use case
         // TODO avoid remote repositories
         Verifier verifier = getVerifier("/resolver.extraRequirements/artificial", false);
-        verifier.getCliOptions().add("-De342-repo=" + P2Repositories.ECLIPSE_342.toString());
-        verifier.getCliOptions().add("-De352-repo=" + P2Repositories.ECLIPSE_352.toString());
+        verifier.getCliOptions().add("-De342-repo=" + P2Repositories.ECLIPSE_LATEST.toString());
         verifier.executeGoals(Arrays.asList("clean", "verify"));
         verifier.verifyErrorFreeLog();
     }
@@ -34,7 +35,7 @@ public class ExtraCompilerRequirementsTest extends AbstractTychoIntegrationTest
     @Test
     public void testDynamicImportPackage() throws Exception {
         Verifier verifier = getVerifier("/resolver.extraRequirements/dynamicimport-package", false);
-        verifier.getCliOptions().add("-De342-repo=" + P2Repositories.ECLIPSE_342.toString());
+        verifier.getCliOptions().add("-De342-repo=" + P2Repositories.ECLIPSE_LATEST.toString());
         verifier.executeGoals(Arrays.asList("clean", "verify"));
         verifier.verifyErrorFreeLog();
     }
@@ -42,7 +43,7 @@ public class ExtraCompilerRequirementsTest extends AbstractTychoIntegrationTest
     @Test
     public void testFragmentSplitPackage() throws Exception {
         Verifier verifier = getVerifier("/resolver.extraRequirements/fragment-split-package", false);
-        verifier.getCliOptions().add("-De342-repo=" + P2Repositories.ECLIPSE_342.toString());
+        verifier.getCliOptions().add("-De342-repo=" + P2Repositories.ECLIPSE_LATEST.toString());
         verifier.executeGoals(Arrays.asList("clean", "verify"));
         verifier.verifyErrorFreeLog();
     }
@@ -50,7 +51,7 @@ public class ExtraCompilerRequirementsTest extends AbstractTychoIntegrationTest
     @Test
     public void testFragmentImportPackage() throws Exception {
         Verifier verifier = getVerifier("/resolver.extraRequirements/implicit-fragment-import-package", false);
-        verifier.getCliOptions().add("-De342-repo=" + P2Repositories.ECLIPSE_342.toString());
+        verifier.getCliOptions().add("-De342-repo=" + P2Repositories.ECLIPSE_LATEST.toString());
         verifier.executeGoals(Arrays.asList("clean", "verify"));
         verifier.verifyErrorFreeLog();
     }
@@ -58,7 +59,7 @@ public class ExtraCompilerRequirementsTest extends AbstractTychoIntegrationTest
     @Test
     public void testImportPackageDirectives() throws Exception {
         Verifier verifier = getVerifier("/resolver.extraRequirements/import-package-directives", false);
-        verifier.getCliOptions().add("-De342-repo=" + P2Repositories.ECLIPSE_342.toString());
+        verifier.getCliOptions().add("-De342-repo=" + P2Repositories.ECLIPSE_LATEST.toString());
         verifier.executeGoals(Arrays.asList("clean", "verify"));
         verifier.verifyErrorFreeLog();
     }
