@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2011 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -27,7 +29,7 @@ public class AutoNoSourceBundleTest extends AbstractTychoIntegrationTest {
             verifier.executeGoal("verify");
             Assert.fail("Reference to a missing source bundle did not fail the build");
         } catch (VerificationException expected) {
-            verifier.verifyTextInLog("feature.feature.group 1.0.0.qualifier requires 'bundle.source 0.0.0' but it could not be found");
+            verifier.verifyTextInLog("feature.feature.group 1.0.0.qualifier requires 'org.eclipse.equinox.p2.iu; bundle.source 0.0.0' but it could not be found");
         }
     }
 
