@@ -1,9 +1,11 @@
 /*******************************************************************************
- * Copyright (c) 2016 SAP SE and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2016, 2021 SAP SE and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    SAP SE - initial API and implementation
@@ -23,7 +25,7 @@ public class CategoriesTest extends AbstractTychoIntegrationTest {
     public void testIncludeExcludeCategories() throws Exception {
         Verifier verifier = getVerifier("/surefire.junit47/categories", false);
         Properties props = verifier.getSystemProperties();
-        props.setProperty("kepler-repo", P2Repositories.ECLIPSE_KEPLER.toString());
+        props.setProperty("kepler-repo", P2Repositories.ECLIPSE_LATEST.toString());
         props.setProperty("groups", "tycho.demo.itp01.tests.FastTests");
         props.setProperty("excludedGroups", "tycho.demo.itp01.tests.SlowTests");
         verifier.executeGoal("verify");
