@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2013, 2018 Red Hat Inc. and others
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     Mickael Istria (Red Hat JBoss) - Test
@@ -26,7 +28,7 @@ public class P2InstalledTestRuntimeTest extends AbstractTychoIntegrationTest {
     public void testProvisionAppAndRunTest() throws Exception {
         Verifier verifier = getVerifier("surefire.p2InstalledRuntime", false);
         List<String> options = verifier.getCliOptions();
-        options.add("-Dp2.repo.url=" + ResourceUtil.P2Repositories.ECLIPSE_352.toString());
+        options.add("-Dp2.repo.url=" + ResourceUtil.P2Repositories.ECLIPSE_OXYGEN.toString());
         options.add("-PprovisionProduct");
         verifier.executeGoals(asList("clean", "integration-test"));
         verifier.verifyErrorFreeLog();
@@ -36,7 +38,7 @@ public class P2InstalledTestRuntimeTest extends AbstractTychoIntegrationTest {
     public void testRunTestOnProvisionedApp() throws Exception {
         Verifier verifier = getVerifier("surefire.p2InstalledRuntime", false);
         List<String> options = verifier.getCliOptions();
-        options.add("-Dp2.repo.url=" + ResourceUtil.P2Repositories.ECLIPSE_352.toString());
+        options.add("-Dp2.repo.url=" + ResourceUtil.P2Repositories.ECLIPSE_OXYGEN.toString());
         options.add("-PuseProvisionedProduct");
         options.add("-DproductClassifier=" + getProductClassifier());
         verifier.executeGoals(asList("clean", "integration-test"));
@@ -47,9 +49,9 @@ public class P2InstalledTestRuntimeTest extends AbstractTychoIntegrationTest {
     public void testDifferentHarnessVersions() throws Exception {
         Verifier verifier = getVerifier("surefire.p2InstalledRuntime", false);
         List<String> options = verifier.getCliOptions();
-        options.add("-Dp2.repo.url=" + ResourceUtil.P2Repositories.ECLIPSE_352.toString());
+        options.add("-Dp2.repo.url=" + ResourceUtil.P2Repositories.ECLIPSE_OXYGEN.toString());
         // Use different TP for test bundle and product under test
-        options.add("-Dother.p2.repo.url=" + ResourceUtil.P2Repositories.ECLIPSE_342.toString());
+        options.add("-Dother.p2.repo.url=" + ResourceUtil.P2Repositories.ECLIPSE_LATEST.toString());
         options.add("-PuseProvisionedProduct");
         options.add("-DproductClassifier=" + getProductClassifier());
         verifier.executeGoals(asList("clean", "integration-test"));
