@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2011 SAP AG and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    SAP AG - initial API and implementation
@@ -17,12 +19,12 @@ import org.junit.Test;
 
 public class TargetPlatformFilteringIntegrationTest extends AbstractTychoIntegrationTest {
 
-    @Test
-    public void test() throws Exception {
-        Verifier verifier = getVerifier("target.restriction.filter", false);
-        verifier.getSystemProperties().put("e342-repo", P2Repositories.ECLIPSE_342.toString());
-        verifier.getSystemProperties().put("e352-repo", P2Repositories.ECLIPSE_352.toString());
-        verifier.executeGoal("verify");
-        verifier.verifyErrorFreeLog();
-    }
+	@Test
+	public void test() throws Exception {
+		Verifier verifier = getVerifier("target.restriction.filter", false);
+		verifier.getSystemProperties().put("e342-repo", "https:////download.eclipse.org/releases/2021-06/");
+		verifier.getSystemProperties().put("e352-repo", P2Repositories.ECLIPSE_LATEST.toString());
+		verifier.executeGoal("verify");
+		verifier.verifyErrorFreeLog();
+	}
 }
