@@ -1,20 +1,23 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2014 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2008, 2020 Sonatype Inc. and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
  *    SAP SE - additional test cases
+ *    Christoph Läubrich - Adjust to new API
  *******************************************************************************/
 package org.eclipse.tycho.p2.resolver;
 
-import static org.hamcrest.CoreMatchers.isA;
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertTrue;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 import java.io.File;
 import java.io.IOException;
@@ -23,21 +26,17 @@ import java.util.List;
 
 import org.eclipse.tycho.core.resolver.shared.IncludeSourceMode;
 import org.eclipse.tycho.p2.target.facade.TargetDefinition.DirectoryLocation;
-import org.eclipse.tycho.p2.target.facade.TargetDefinition.FeaturePlatformLocation;
+import org.eclipse.tycho.p2.target.facade.TargetDefinition.FeaturesLocation;
 import org.eclipse.tycho.p2.target.facade.TargetDefinition.IncludeMode;
 import org.eclipse.tycho.p2.target.facade.TargetDefinition.InstallableUnitLocation;
 import org.eclipse.tycho.p2.target.facade.TargetDefinition.Location;
-import org.eclipse.tycho.p2.target.facade.TargetDefinition.ProfilePlatformLocation;
+import org.eclipse.tycho.p2.target.facade.TargetDefinition.ProfileLocation;
+import org.eclipse.tycho.p2.target.facade.TargetDefinitionFile;
 import org.eclipse.tycho.p2.target.facade.TargetDefinitionSyntaxException;
-import org.junit.Rule;
-import org.junit.Test;
-import org.junit.rules.ExpectedException;
+import org.junit.jupiter.api.Test;
 
 public class TargetDefinitionFileTest {
 
-    @Rule
-    public ExpectedException expectedException = ExpectedException.none();
-
     @Test
     public void testTarget() throws Exception {
         List<? extends Location> locations = readTarget("target.target").getLocations();
@@ -45,7 +44,7 @@ public class TargetDefinitionFileTest {
 
         InstallableUnitLocation location = (InstallableUnitLocation) locations.get(0);
         assertEquals(1, location.getRepositories().size());
-        assertEquals(URI.create("http://download.eclipse.org/eclipse/updates/3.5/"),
+        assertEquals(URI.create("https://download.eclipse.org/eclipse/updates/3.5/"),
                 location.getRepositories().get(0).getLocation());
         assertEquals(1, location.getUnits().size());
         assertEquals("org.eclipse.platform.sdk", location.getUnits().get(0).getId());
@@ -56,7 +55,7 @@ public class TargetDefinitionFileTest {
         assertEquals(2, l02.getRepositories().size());
         assertEquals(URI.create("http://subclipse.tigris.org/update_1.6.x/"),
                 l02.getRepositories().get(0).getLocation());
-        assertEquals(URI.create("http://download.eclipse.org/tools/mylyn/update/e3.4/"),
+        assertEquals(URI.create("https://download.eclipse.org/tools/mylyn/update/e3.4/"),
                 l02.getRepositories().get(1).getLocation());
     }
 
@@ -68,8 +67,8 @@ public class TargetDefinitionFileTest {
         assertEquals("Feature", locations.get(2).getTypeDescription());
         assertEquals("InstallableUnit", locations.get(3).getTypeDescription());
         assertTrue(locations.get(0) instanceof DirectoryLocation);
-        assertTrue(locations.get(1) instanceof ProfilePlatformLocation);
-        assertTrue(locations.get(2) instanceof FeaturePlatformLocation);
+        assertTrue(locations.get(1) instanceof ProfileLocation);
+        assertTrue(locations.get(2) instanceof FeaturesLocation);
         assertTrue(locations.get(3) instanceof InstallableUnitLocation);
     }
 
@@ -105,37 +104,22 @@ public class TargetDefinitionFileTest {
         assertEquals(false, locationWithoutIncludeSourceAttribute.includeSource());
     }
 
-    @Test
-    public void testIncludeSourceWhenIgnored() throws Exception {
-        List<? extends Location> locations = readTarget("includeSource.target", IncludeSourceMode.ignore)
-                .getLocations();
-        for (Location location : locations) {
-            assertEquals(false, ((InstallableUnitLocation) location).includeSource());
-        }
-    }
-
-    @Test
-    public void testIncludeSourceWhenForced() throws Exception {
-        List<? extends Location> locations = readTarget("includeSource.target", IncludeSourceMode.force).getLocations();
-        for (Location location : locations) {
-            assertEquals(true, ((InstallableUnitLocation) location).includeSource());
-        }
-    }
-
     @Test
     public void testInvalidXML() throws Exception {
-        expectedException.expectCause(isA(TargetDefinitionSyntaxException.class));
-        readTarget("invalidXML.target").getLocations();
+        try {
+            readTarget("invalidXML.target").getLocations();
+        } catch (RuntimeException e) {
+            assertEquals(TargetDefinitionSyntaxException.class, e.getCause().getClass());
+        }
     }
 
     public void testInvalidIncludeMode() throws Exception {
-        expectedException.expect(TargetDefinitionSyntaxException.class);
 
         List<? extends Location> locations = readTarget("invalidMode.target").getLocations();
 
         // allow exception to be thrown late
         InstallableUnitLocation invalidIncludeModeLocation = (InstallableUnitLocation) locations.get(0);
-        invalidIncludeModeLocation.getIncludeMode();
+        assertThrows(TargetDefinitionSyntaxException.class, () -> invalidIncludeModeLocation.getIncludeMode());
     }
 
     @Test
@@ -155,7 +139,7 @@ public class TargetDefinitionFileTest {
     }
 
     private TargetDefinitionFile readTarget(String fileName, IncludeSourceMode includeSourceMode) throws IOException {
-        return TargetDefinitionFile.read(new File("src/test/resources/modelio/" + fileName), includeSourceMode);
+        return TargetDefinitionFile.read(new File("src/test/resources/modelio/" + fileName));
     }
 
 }
