@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2012 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -21,9 +23,8 @@ import org.apache.maven.plugins.annotations.Parameter;
  * additional artifacts or post process standard artifacts using custom goals bound to package
  * phase.
  */
-@Mojo(name = "p2-metadata-default")
+@Mojo(name = "p2-metadata-default", threadSafe = true)
 public class P2MetadataDefaultMojo extends P2MetadataMojo {
-
     @Parameter(defaultValue = "true")
     private boolean defaultP2Metadata;
 
