@@ -1,25 +1,17 @@
 /*******************************************************************************
  * Copyright (c) 2008, 2013 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
  *******************************************************************************/
 package org.eclipse.tycho.plugins.p2;
 
-import static org.eclipse.tycho.p2.repository.RepositoryLayoutHelper.CLASSIFIER_P2_ARTIFACTS;
-import static org.eclipse.tycho.p2.repository.RepositoryLayoutHelper.CLASSIFIER_P2_METADATA;
-import static org.eclipse.tycho.p2.repository.RepositoryLayoutHelper.EXTENSION_P2_ARTIFACTS;
-import static org.eclipse.tycho.p2.repository.RepositoryLayoutHelper.EXTENSION_P2_METADATA;
-import static org.eclipse.tycho.p2.repository.RepositoryLayoutHelper.FILE_NAME_LOCAL_ARTIFACTS;
-import static org.eclipse.tycho.p2.repository.RepositoryLayoutHelper.FILE_NAME_P2_ARTIFACTS;
-import static org.eclipse.tycho.p2.repository.RepositoryLayoutHelper.FILE_NAME_P2_METADATA;
-import static org.eclipse.tycho.p2.repository.RepositoryLayoutHelper.KEY_ARTIFACT_ATTACHED;
-import static org.eclipse.tycho.p2.repository.RepositoryLayoutHelper.KEY_ARTIFACT_MAIN;
-
 import java.io.File;
 import java.io.FileOutputStream;
 import java.io.IOException;
@@ -46,7 +38,9 @@ import org.apache.maven.project.MavenProject;
 import org.apache.maven.project.MavenProjectHelper;
 import org.eclipse.sisu.equinox.EquinoxServiceFactory;
 import org.eclipse.tycho.ArtifactType;
+import org.eclipse.tycho.IDependencyMetadata.DependencyMetadataType;
 import org.eclipse.tycho.ReactorProject;
+import org.eclipse.tycho.TychoConstants;
 import org.eclipse.tycho.core.osgitools.DefaultReactorProject;
 import org.eclipse.tycho.p2.facade.internal.ArtifactFacade;
 import org.eclipse.tycho.p2.metadata.IArtifactFacade;
@@ -54,8 +48,9 @@ import org.eclipse.tycho.p2.metadata.IP2Artifact;
 import org.eclipse.tycho.p2.metadata.P2Generator;
 import org.eclipse.tycho.p2.metadata.PublisherOptions;
 
-@Mojo(name = "p2-metadata")
+@Mojo(name = "p2-metadata", threadSafe = true)
 public class P2MetadataMojo extends AbstractMojo {
+    private static final Object LOCK = new Object();
 
     @Parameter(property = "project")
     protected MavenProject project;
@@ -90,13 +85,32 @@ public class P2MetadataMojo extends AbstractMojo {
     private List<Repository> baselineRepositories;
 
     /**
-     * What happens when build artifact does not match baseline version.
+     * What happens when build artifact does not match baseline version:
+     * <ul>
+     * <li><code>disable</code>: Disable baseline validation.</li>
+     * <li><code>warn</code> (default): Warn about discrepancies between build and baseline
+     * artifacts but do not fail the build.</li>
+     * <li><code>failCommon</code>: Fail the build if there are discrepancies between artifacts
+     * present both in build and baseline. Attached artifacts only present in the build do not
+     * result in build failure.</li>
+     * <li><code>fail</code>: Fail the build if there are any discrepancy between build and baseline
+     * artifacts.</li>
+     * </ul>
      */
     @Parameter(property = "tycho.baseline", defaultValue = "warn")
     private BaselineMode baselineMode;
 
     /**
-     * Whether to replace build artifacts with baseline version or use reactor version.
+     * Whether to replace build artifacts with baseline version or use reactor version:
+     * <ul>
+     * <li><code>none</code>: Do not replace build artifacts with baseline version.</li>
+     * <li><code>common</code>: Replace build artifacts with baseline version. Attached artifacts
+     * only present in the build are not removed and will likely result in inconsistencies among
+     * artifacts of the same project! Use as last resort when baseline does not contain all build
+     * artifacts.</li>
+     * <li><code>all</code> (default): Replace build artifacts with baseline version. Attached
+     * artifacts only present in the build are removed.</li>
+     * </ul>
      */
     @Parameter(property = "tycho.baseline.replace", defaultValue = "all")
     private BaselineReplace baselineReplace;
@@ -113,7 +127,9 @@ public class P2MetadataMojo extends AbstractMojo {
 
     @Override
     public void execute() throws MojoExecutionException, MojoFailureException {
-        attachP2Metadata();
+        synchronized (LOCK) {
+            attachP2Metadata();
+        }
     }
 
     private <T> T getService(Class<T> type) {
@@ -162,11 +178,11 @@ public class P2MetadataMojo extends AbstractMojo {
                         baselineRepositories, baselineMode, baselineReplace);
             }
 
-            File contentsXml = new File(targetDir, FILE_NAME_P2_METADATA);
-            File artifactsXml = new File(targetDir, FILE_NAME_P2_ARTIFACTS);
+            File contentsXml = new File(targetDir, TychoConstants.FILE_NAME_P2_METADATA);
+            File artifactsXml = new File(targetDir, TychoConstants.FILE_NAME_P2_ARTIFACTS);
             p2generator.persistMetadata(generatedMetadata, contentsXml, artifactsXml);
-            projectHelper.attachArtifact(project, EXTENSION_P2_METADATA, CLASSIFIER_P2_METADATA, contentsXml);
-            projectHelper.attachArtifact(project, EXTENSION_P2_ARTIFACTS, CLASSIFIER_P2_ARTIFACTS, artifactsXml);
+            projectHelper.attachArtifact(project, TychoConstants.EXTENSION_P2_METADATA, TychoConstants.CLASSIFIER_P2_METADATA, contentsXml);
+            projectHelper.attachArtifact(project, TychoConstants.EXTENSION_P2_ARTIFACTS, TychoConstants.CLASSIFIER_P2_ARTIFACTS, artifactsXml);
 
             ReactorProject reactorProject = DefaultReactorProject.adapt(project);
 
@@ -185,13 +201,13 @@ public class P2MetadataMojo extends AbstractMojo {
             }
 
             // TODO 353889 distinguish between dependency resolution seed units ("primary") and other units of the project
-            reactorProject.setDependencyMetadata(true, installableUnits);
-            reactorProject.setDependencyMetadata(false, Collections.emptySet());
+            reactorProject.setDependencyMetadata(DependencyMetadataType.SEED, installableUnits);
+            reactorProject.setDependencyMetadata(DependencyMetadataType.RESOLVE, Collections.emptySet());
         } catch (IOException e) {
             throw new MojoExecutionException("Could not generate P2 metadata", e);
         }
 
-        File localArtifactsFile = new File(project.getBuild().getDirectory(), FILE_NAME_LOCAL_ARTIFACTS);
+        File localArtifactsFile = new File(project.getBuild().getDirectory(), TychoConstants.FILE_NAME_LOCAL_ARTIFACTS);
         writeArtifactLocations(localArtifactsFile, getAllProjectArtifacts(project));
     }
 
@@ -235,9 +251,9 @@ public class P2MetadataMojo extends AbstractMojo {
 
         for (Entry<String, File> entry : artifactLocations.entrySet()) {
             if (entry.getKey() == null) {
-                outputProperties.put(KEY_ARTIFACT_MAIN, entry.getValue().getAbsolutePath());
+                outputProperties.put(TychoConstants.KEY_ARTIFACT_MAIN, entry.getValue().getAbsolutePath());
             } else {
-                outputProperties.put(KEY_ARTIFACT_ATTACHED + entry.getKey(), entry.getValue().getAbsolutePath());
+                outputProperties.put(TychoConstants.KEY_ARTIFACT_ATTACHED + entry.getKey(), entry.getValue().getAbsolutePath());
             }
         }
 
