@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2010, 2011 SAP SE and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     SAP SE - initial API and implementation
@@ -29,8 +31,9 @@ import org.codehaus.plexus.archiver.util.DefaultFileSet;
  * Creates a zip archive with the aggregated p2 repository.
  * </p>
  */
-@Mojo(name = "archive-repository", defaultPhase = LifecyclePhase.PACKAGE)
+@Mojo(name = "archive-repository", defaultPhase = LifecyclePhase.PACKAGE, threadSafe = true)
 public final class ArchiveRepositoryMojo extends AbstractRepositoryMojo {
+    private static final Object LOCK = new Object();
 
     @Component(role = Archiver.class, hint = "zip")
     private Archiver inflater;
@@ -55,19 +58,21 @@ public final class ArchiveRepositoryMojo extends AbstractRepositoryMojo {
             return;
         }
 
-        File destFile = getBuildDirectory().getChild(finalName + ".zip");
+        synchronized (LOCK) {
+            File destFile = getBuildDirectory().getChild(finalName + ".zip");
 
-        try {
-            inflater.addFileSet(DefaultFileSet.fileSet(getAssemblyRepositoryLocation()).prefixed(""));
-            inflater.setDestFile(destFile);
-            inflater.createArchive();
-        } catch (ArchiverException e) {
-            throw new MojoExecutionException("Error packing p2 repository", e);
-        } catch (IOException e) {
-            throw new MojoExecutionException("Error packing p2 repository", e);
+            try {
+                inflater.addFileSet(DefaultFileSet.fileSet(getAssemblyRepositoryLocation()).prefixed(""));
+                inflater.setDestFile(destFile);
+                inflater.createArchive();
+            } catch (ArchiverException e) {
+                throw new MojoExecutionException("Error packing p2 repository", e);
+            } catch (IOException e) {
+                throw new MojoExecutionException("Error packing p2 repository", e);
+            }
+
+            getProject().getArtifact().setFile(destFile);
         }
-
-        getProject().getArtifact().setFile(destFile);
     }
 
 }
