@@ -1,13 +1,16 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2017 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2008, 2022 Sonatype Inc. and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
  *    Bachmann electronic GmbH. - #519941 Copy the shared license info
+ *    Christoph Läubrich - Issue #572 - Insert dynamic dependencies into the jar included pom 
  *******************************************************************************/
 package org.eclipse.tycho.packaging;
 
@@ -30,20 +33,22 @@ import org.apache.maven.plugins.annotations.LifecyclePhase;
 import org.apache.maven.plugins.annotations.Mojo;
 import org.apache.maven.plugins.annotations.Parameter;
 import org.apache.maven.plugins.annotations.ResolutionScope;
+import org.apache.maven.project.MavenProject;
 import org.codehaus.plexus.archiver.Archiver;
 import org.codehaus.plexus.archiver.FileSet;
 import org.codehaus.plexus.archiver.jar.JarArchiver;
 import org.codehaus.plexus.component.repository.exception.ComponentLookupException;
 import org.codehaus.plexus.util.IOUtil;
+import org.eclipse.tycho.BuildProperties;
+import org.eclipse.tycho.ReactorProject;
 import org.eclipse.tycho.artifacts.TargetPlatform;
 import org.eclipse.tycho.core.osgitools.DefaultReactorProject;
-import org.eclipse.tycho.core.shared.BuildProperties;
-import org.eclipse.tycho.core.shared.BuildPropertiesParser;
 import org.eclipse.tycho.core.utils.TychoProjectUtils;
 import org.eclipse.tycho.model.Feature;
 
-@Mojo(name = "package-feature", defaultPhase = LifecyclePhase.PACKAGE, requiresDependencyResolution = ResolutionScope.RUNTIME)
+@Mojo(name = "package-feature", defaultPhase = LifecyclePhase.PACKAGE, requiresDependencyResolution = ResolutionScope.RUNTIME, threadSafe = true)
 public class PackageFeatureMojo extends AbstractTychoPackagingMojo {
+    private static final Object LOCK = new Object();
 
     private static final String FEATURE_PROPERTIES = "feature.properties";
 
@@ -103,64 +108,64 @@ public class PackageFeatureMojo extends AbstractTychoPackagingMojo {
     @Component
     private LicenseFeatureHelper licenseFeatureHelper;
 
-    @Component
-    private BuildPropertiesParser buildPropertiesParser;
-
     @Override
     public void execute() throws MojoExecutionException, MojoFailureException {
-        outputDirectory.mkdirs();
+        synchronized (LOCK) {
+            outputDirectory.mkdirs();
 
-        Feature feature = Feature.loadFeature(basedir);
+            Feature feature = Feature.loadFeature(basedir);
 
-        File licenseFeature = licenseFeatureHelper.getLicenseFeature(feature, project);
+            File licenseFeature = licenseFeatureHelper.getLicenseFeature(feature, project);
 
-        updateLicenseProperties(feature, licenseFeature);
+            updateLicenseProperties(feature, licenseFeature);
 
-        File featureXml = new File(outputDirectory, Feature.FEATURE_XML);
-        try {
-            expandVersionQualifiers(feature);
-            Feature.write(feature, featureXml);
-        } catch (IOException e) {
-            throw new MojoExecutionException("Error updating feature.xml", e);
-        }
-
-        BuildProperties buildProperties = buildPropertiesParser.parse(project.getBasedir());
-        checkBinIncludesExist(buildProperties);
-
-        File featureProperties = getFeatureProperties(licenseFeature, buildProperties);
-
-        File outputJar = new File(outputDirectory, finalName + ".jar");
-        outputJar.getParentFile().mkdirs();
-
-        MavenArchiver archiver = new MavenArchiver();
-        JarArchiver jarArchiver = getJarArchiver();
-        archiver.setArchiver(jarArchiver);
-        archiver.setOutputFile(outputJar);
-        jarArchiver.setDestFile(outputJar);
-
-        try {
-            archiver.getArchiver().addFileSet(getManuallyIncludedFiles(buildProperties));
-            if (licenseFeature != null) {
-                archiver.getArchiver()
-                        .addArchivedFileSet(licenseFeatureHelper.getLicenseFeatureFileSet(licenseFeature));
+            File featureXml = new File(outputDirectory, Feature.FEATURE_XML);
+            try {
+                expandVersionQualifiers(feature);
+                Feature.write(feature, featureXml);
+            } catch (IOException e) {
+                throw new MojoExecutionException("Error updating feature.xml", e);
             }
-            archiver.getArchiver().addFile(featureXml, Feature.FEATURE_XML);
-            if (featureProperties != null) {
-                archiver.getArchiver().addFile(featureProperties, FEATURE_PROPERTIES);
-            }
-            if (archive == null) {
-                archive = new MavenArchiveConfiguration();
-                archive.setAddMavenDescriptor(false);
-            }
-            archiver.createArchive(session, project, archive);
-        } catch (Exception e) {
-            throw new MojoExecutionException("Error creating feature package", e);
-        }
 
-        project.getArtifact().setFile(outputJar);
+			BuildProperties buildProperties = DefaultReactorProject.adapt(project).getBuildProperties();
+            checkBinIncludesExist(buildProperties);
 
-        if (deployableFeature) {
-            assembleDeployableFeature();
+            File featureProperties = getFeatureProperties(licenseFeature, buildProperties);
+
+            File outputJar = new File(outputDirectory, finalName + ".jar");
+            outputJar.getParentFile().mkdirs();
+
+            MavenArchiver archiver = new MavenArchiver();
+            JarArchiver jarArchiver = getJarArchiver();
+            archiver.setArchiver(jarArchiver);
+            archiver.setOutputFile(outputJar);
+            jarArchiver.setDestFile(outputJar);
+
+            try {
+                archiver.getArchiver().addFileSet(getManuallyIncludedFiles(buildProperties));
+                if (licenseFeature != null) {
+                    archiver.getArchiver()
+                            .addArchivedFileSet(licenseFeatureHelper.getLicenseFeatureFileSet(licenseFeature));
+                }
+                archiver.getArchiver().addFile(featureXml, Feature.FEATURE_XML);
+                if (featureProperties != null) {
+                    archiver.getArchiver().addFile(featureProperties, FEATURE_PROPERTIES);
+                }
+                if (archive == null) {
+                    archive = new MavenArchiveConfiguration();
+                    archive.setAddMavenDescriptor(false);
+                }
+				MavenProject mavenProject = project;
+				archiver.createArchive(session, mavenProject, archive);
+            } catch (Exception e) {
+                throw new MojoExecutionException("Error creating feature package", e);
+            }
+
+            project.getArtifact().setFile(outputJar);
+
+            if (deployableFeature) {
+                assembleDeployableFeature();
+            }
         }
     }
 
@@ -207,23 +212,19 @@ public class PackageFeatureMojo extends AbstractTychoPackagingMojo {
     }
 
     private void appendToOrAddFeatureProperties(File targetFeatureProperties, File licenseFeature) throws IOException {
-        InputStream inputStream = null;
-        FileWriter writer = null;
         try (ZipFile zip = new ZipFile(licenseFeature)) {
             ZipEntry entry = zip.getEntry(FEATURE_PROPERTIES);
             if (entry != null) {
-                inputStream = zip.getInputStream(entry);
-                writer = new FileWriter(targetFeatureProperties.getAbsolutePath(), true);
-                // if we append, first add a new line to be sure that we start 
-                // in a new line of the existing file
-                if (targetFeatureProperties.exists()) {
-                    IOUtil.copy("\n", writer);
+                try (InputStream inputStream = zip.getInputStream(entry);
+                        FileWriter writer = new FileWriter(targetFeatureProperties.getAbsolutePath(), true)) {
+                    // if we append, first add a new line to be sure that we start 
+                    // in a new line of the existing file
+                    if (targetFeatureProperties.exists()) {
+                        IOUtil.copy("\n", writer);
+                    }
+                    IOUtil.copy(inputStream, writer);
                 }
-                IOUtil.copy(inputStream, writer);
             }
-        } finally {
-            IOUtil.close(writer);
-            IOUtil.close(inputStream);
         }
     }
 
@@ -240,14 +241,15 @@ public class PackageFeatureMojo extends AbstractTychoPackagingMojo {
     }
 
     private void assembleDeployableFeature() throws MojoExecutionException {
-        UpdateSiteAssembler assembler = new UpdateSiteAssembler(session, target);
+        UpdateSiteAssembler assembler = new UpdateSiteAssembler(plexus, target);
         getDependencyWalker().walk(assembler);
     }
 
     private void expandVersionQualifiers(Feature feature) throws MojoFailureException {
-        feature.setVersion(DefaultReactorProject.adapt(project).getExpandedVersion());
+        ReactorProject reactorProject = DefaultReactorProject.adapt(project);
+        feature.setVersion(reactorProject.getExpandedVersion());
 
-        TargetPlatform targetPlatform = TychoProjectUtils.getTargetPlatformIfAvailable(project);
+        TargetPlatform targetPlatform = TychoProjectUtils.getTargetPlatformIfAvailable(reactorProject);
         if (targetPlatform == null) {
             getLog().warn(
                     "Skipping version reference expansion in eclipse-feature project using the deprecated -Dtycho.targetPlatform configuration");
