@@ -1,25 +1,23 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2017 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2008, 2021 Sonatype Inc. and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
  *******************************************************************************/
 package org.eclipse.tycho.packaging;
 
-import java.io.BufferedInputStream;
-import java.io.BufferedOutputStream;
 import java.io.File;
 import java.io.FileInputStream;
-import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
 import java.io.IOException;
-import java.io.InputStream;
-import java.io.OutputStream;
-import java.io.StringWriter;
+import java.nio.charset.StandardCharsets;
+import java.nio.file.Files;
 import java.util.Arrays;
 import java.util.Enumeration;
 import java.util.HashMap;
@@ -43,10 +41,8 @@ import org.codehaus.plexus.archiver.ArchiverException;
 import org.codehaus.plexus.archiver.util.ArchiveEntryUtils;
 import org.codehaus.plexus.archiver.zip.ZipArchiver;
 import org.codehaus.plexus.component.repository.exception.ComponentLookupException;
-import org.codehaus.plexus.logging.Logger;
 import org.codehaus.plexus.util.DirectoryScanner;
 import org.codehaus.plexus.util.FileUtils;
-import org.codehaus.plexus.util.IOUtil;
 import org.codehaus.plexus.util.SelectorUtils;
 import org.codehaus.plexus.util.io.RawInputStreamFacade;
 import org.eclipse.pde.internal.swt.tools.IconExe;
@@ -57,6 +53,7 @@ import org.eclipse.tycho.core.ArtifactDependencyVisitor;
 import org.eclipse.tycho.core.ArtifactDependencyWalker;
 import org.eclipse.tycho.core.PluginDescription;
 import org.eclipse.tycho.core.osgitools.BundleReader;
+import org.eclipse.tycho.core.osgitools.DefaultReactorProject;
 import org.eclipse.tycho.core.resolver.shared.PlatformPropertiesUtils;
 import org.eclipse.tycho.core.shared.TargetEnvironment;
 import org.eclipse.tycho.core.utils.TychoProjectUtils;
@@ -65,8 +62,9 @@ import org.eclipse.tycho.locking.facade.FileLocker;
 import org.eclipse.tycho.model.BundleConfiguration;
 import org.eclipse.tycho.model.ProductConfiguration;
 
-@Mojo(name = "product-export")
+@Mojo(name = "product-export", threadSafe = true)
 public class ProductExportMojo extends AbstractTychoPackagingMojo {
+    private static final Object LOCK = new Object();
     /**
      * The product configuration, a .product file. This file manages all aspects of a product
      * definition from its constituent plug-ins to configuration files to branding.
@@ -91,7 +89,8 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
     /**
      * @deprecated use target-platform-configuration <environments/> element
      */
-    @Parameter
+	@Deprecated
+	@Parameter
     private TargetEnvironment[] environments;
 
     @Parameter(property = "tycho.product.createArchive", defaultValue = "true")
@@ -117,110 +116,111 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
     @Component
     private BundleReader manifestReader;
 
-    @Component
-    private Logger logger;
-
     @Component
     private FileLockService fileLockService;
 
     @Override
     public void execute() throws MojoExecutionException, MojoFailureException {
-        getLog().warn(
-                "The packaging type 'eclipse-application' is deprecated; use 'eclipse-repository' instead. "
-                        + "See http://wiki.eclipse.org/Tycho_Messages_Explained#Eclipse_Application");
-        if (!productConfigurationFile.exists()) {
-            throw new MojoExecutionException("Product configuration file not found "
-                    + productConfigurationFile.getAbsolutePath());
-        }
+        synchronized (LOCK) {
+            getLog().warn("The packaging type 'eclipse-application' is deprecated; use 'eclipse-repository' instead. "
+                    + "See https://wiki.eclipse.org/Tycho_Messages_Explained#Eclipse_Application");
+            if (!productConfigurationFile.exists()) {
+                throw new MojoExecutionException(
+                        "Product configuration file not found " + productConfigurationFile.getAbsolutePath());
+            }
 
-        try {
-            getLog().debug("Parsing productConfiguration");
-            productConfiguration = ProductConfiguration.read(productConfigurationFile);
-        } catch (IOException e) {
-            throw new MojoExecutionException("Error reading product configuration file", e);
-        }
+            try {
+                getLog().debug("Parsing productConfiguration");
+                productConfiguration = ProductConfiguration.read(productConfigurationFile);
+            } catch (IOException e) {
+                throw new MojoExecutionException("Error reading product configuration file", e);
+            }
 
-        // build results will vary from system to system without explicit target environment configuration
-        boolean implicitTargetEnvironment = TychoProjectUtils.getTargetPlatformConfiguration(project)
-                .isImplicitTargetEnvironment();
-        if (productConfiguration.includeLaunchers() && implicitTargetEnvironment && environments == null) {
-            throw new MojoFailureException("Product includes native launcher but no target environment was specified");
-        }
+            // build results will vary from system to system without explicit target environment configuration
+            boolean implicitTargetEnvironment = TychoProjectUtils
+                    .getTargetPlatformConfiguration(DefaultReactorProject.adapt(project)).isImplicitTargetEnvironment();
+            if (productConfiguration.includeLaunchers() && implicitTargetEnvironment && environments == null) {
+                throw new MojoFailureException(
+                        "Product includes native launcher but no target environment was specified");
+            }
 
-        if (separateEnvironments) {
-            for (TargetEnvironment environment : getEnvironments()) {
-                File target = getTarget(environment);
+            if (separateEnvironments) {
+                for (TargetEnvironment environment : getEnvironments()) {
+                    File target = getTarget(environment);
+                    File targetEclipse = new File(target, "eclipse");
+                    targetEclipse.mkdirs();
+
+                    generateDotEclipseProduct(targetEclipse);
+                    generateConfigIni(environment, targetEclipse);
+                    includeRootFiles(environment, targetEclipse);
+
+                    ProductAssembler assembler = new ProductAssembler(plexus, manifestReader, targetEclipse,
+                            environment);
+                    assembler.setIncludeSources(includeSources);
+                    getDependencyWalker(environment).walk(assembler);
+
+                    if (productConfiguration.includeLaunchers()) {
+                        copyExecutable(environment, targetEclipse);
+                    }
+
+                    if (createProductArchive) {
+                        createProductArchive(target, toString(environment));
+                    }
+                }
+            } else {
+                File target = getTarget(null);
                 File targetEclipse = new File(target, "eclipse");
                 targetEclipse.mkdirs();
 
                 generateDotEclipseProduct(targetEclipse);
-                generateConfigIni(environment, targetEclipse);
-                includeRootFiles(environment, targetEclipse);
+                generateConfigIni(null, targetEclipse);
 
-                ProductAssembler assembler = new ProductAssembler(session, manifestReader, targetEclipse, environment);
+                for (TargetEnvironment environment : getEnvironments()) {
+                    includeRootFiles(environment, targetEclipse);
+                }
+
+                ProductAssembler assembler = new ProductAssembler(plexus, manifestReader, targetEclipse, null);
                 assembler.setIncludeSources(includeSources);
-                getDependencyWalker(environment).walk(assembler);
+                if (forcePackedDependencies) {
+                    assembler.setUnpackFeatures(false);
+                    assembler.setUnpackPlugins(false);
+                }
+                getDependencyWalker().walk(assembler);
 
                 if (productConfiguration.includeLaunchers()) {
-                    copyExecutable(environment, targetEclipse);
+                    for (TargetEnvironment environment : getEnvironments()) {
+                        copyExecutable(environment, targetEclipse);
+                    }
                 }
 
                 if (createProductArchive) {
-                    createProductArchive(target, toString(environment));
-                }
-            }
-        } else {
-            File target = getTarget(null);
-            File targetEclipse = new File(target, "eclipse");
-            targetEclipse.mkdirs();
-
-            generateDotEclipseProduct(targetEclipse);
-            generateConfigIni(null, targetEclipse);
-
-            for (TargetEnvironment environment : getEnvironments()) {
-                includeRootFiles(environment, targetEclipse);
-            }
-
-            ProductAssembler assembler = new ProductAssembler(session, manifestReader, targetEclipse, null);
-            assembler.setIncludeSources(includeSources);
-            if (forcePackedDependencies) {
-                assembler.setUnpackFeatures(false);
-                assembler.setUnpackPlugins(false);
-            }
-            getDependencyWalker().walk(assembler);
-
-            if (productConfiguration.includeLaunchers()) {
-                for (TargetEnvironment environment : getEnvironments()) {
-                    copyExecutable(environment, targetEclipse);
+                    createProductArchive(target, null);
                 }
             }
 
-            if (createProductArchive) {
-                createProductArchive(target, null);
+            // String version = getTychoProjectFacet().getArtifactKey( project ).getVersion();
+            // String productVersion = VersioningHelper.getExpandedVersion( project, version );
+            // productConfiguration.setVersion( productVersion.toString() );
+
+            try {
+                ProductConfiguration.write(productConfiguration, expandedProductFile);
+
+                if (p2inf.canRead()) {
+                    FileUtils.copyFile(p2inf, new File(expandedProductFile.getParentFile(), p2inf.getName()));
+                }
+            } catch (IOException e) {
+                throw new MojoExecutionException("Error writing expanded product configuration file", e);
             }
-        }
 
-        // String version = getTychoProjectFacet().getArtifactKey( project ).getVersion();
-        // String productVersion = VersioningHelper.getExpandedVersion( project, version );
-        // productConfiguration.setVersion( productVersion.toString() );
-
-        try {
-            ProductConfiguration.write(productConfiguration, expandedProductFile);
-
-            if (p2inf.canRead()) {
-                FileUtils.copyFile(p2inf, new File(expandedProductFile.getParentFile(), p2inf.getName()));
+            if (!createProductArchive || environments != null) {
+                project.getArtifact().setFile(expandedProductFile);
             }
-        } catch (IOException e) {
-            throw new MojoExecutionException("Error writing expanded product configuration file", e);
-        }
-
-        if (!createProductArchive || environments != null) {
-            project.getArtifact().setFile(expandedProductFile);
         }
     }
 
     private ArtifactDependencyWalker getDependencyWalker(TargetEnvironment environment) {
-        return getTychoProjectFacet(PackagingType.TYPE_ECLIPSE_APPLICATION).getDependencyWalker(project, environment);
+        return getTychoProjectFacet(PackagingType.TYPE_ECLIPSE_APPLICATION)
+                .getDependencyWalker(DefaultReactorProject.adapt(project), environment);
     }
 
     private List<TargetEnvironment> getEnvironments() {
@@ -230,7 +230,7 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
             return Arrays.asList(environments);
         }
 
-        return TychoProjectUtils.getTargetPlatformConfiguration(project).getEnvironments();
+        return TychoProjectUtils.getTargetPlatformConfiguration(DefaultReactorProject.adapt(project)).getEnvironments();
     }
 
     private File getTarget(TargetEnvironment environment) {
@@ -260,17 +260,17 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
     /**
      * Root files are files that must be packaged with an Eclipse install but are not features or
      * plug-ins. These files are added to the root or to a specified sub folder of the build.
-     * 
+     *
      * <pre>
      * root=
      * root.<confi>=
      * root.folder.<subfolder>=
      * root.<config>.folder.<subfolder>=
      * </pre>
-     * 
+     *
      * Not supported are the properties root.permissions and root.link.
-     * 
-     * @see http 
+     *
+     * @see http
      *      ://help.eclipse.org/ganymede/index.jsp?topic=/org.eclipse.pde.doc.user/tasks/pde_rootfiles
      *      .htm
      * @throws MojoExecutionException
@@ -279,58 +279,52 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
         Properties properties = project.getProperties();
         String generatedBuildProperties = properties.getProperty("generatedBuildProperties");
         getLog().debug("includeRootFiles from " + generatedBuildProperties);
-        if (generatedBuildProperties != null) {
-            Properties rootProperties = new Properties();
-            try {
-                FileInputStream stream = new FileInputStream(new File(project.getBasedir(), generatedBuildProperties));
-                try {
-                    rootProperties.load(stream);
-                } finally {
-                    stream.close();
-                }
-                if (!rootProperties.isEmpty()) {
-                    String config = getConfig(environment);
-                    String root = "root";
-                    String rootConfig = "root." + config;
-                    String rootFolder = "root.folder.";
-                    String rootConfigFolder = "root." + config + ".folder.";
-                    Set<Entry<Object, Object>> entrySet = rootProperties.entrySet();
-                    for (Iterator iterator = entrySet.iterator(); iterator.hasNext();) {
-                        Entry<String, String> entry = (Entry<String, String>) iterator.next();
-                        String key = entry.getKey().trim();
-                        // root=
-                        if (root.equals(key)) {
-                            handleRootEntry(target, entry.getValue(), null);
-                        }
-                        // root.xxx=
-                        else if (rootConfig.equals(key)) {
-                            handleRootEntry(target, entry.getValue(), null);
-                        }
-                        // root.folder.yyy=
-                        else if (key.startsWith(rootFolder)) {
-                            String subFolder = entry.getKey().substring((rootFolder.length()));
-                            handleRootEntry(target, entry.getValue(), subFolder);
-                        }
-                        // root.xxx.folder.yyy=
-                        else if (key.startsWith(rootConfigFolder)) {
-                            String subFolder = entry.getKey().substring((rootConfigFolder.length()));
-                            handleRootEntry(target, entry.getValue(), subFolder);
-                        } else {
-                            getLog().debug("ignoring property " + entry.getKey() + "=" + entry.getValue());
-                        }
-                    }
-                }
-            } catch (FileNotFoundException e) {
-                throw new MojoExecutionException("Error including root files for product", e);
-            } catch (IOException e) {
-                throw new MojoExecutionException("Error including root files for product", e);
+        if (generatedBuildProperties == null) {
+            return;
+        }
+
+        Properties rootProperties = new Properties();
+        try (FileInputStream stream = new FileInputStream(new File(project.getBasedir(), generatedBuildProperties))) {
+            rootProperties.load(stream);
+        } catch (IOException e) {
+            throw new MojoExecutionException("Error including root files for product", e);
+        }
+
+        String config = getConfig(environment);
+        String root = "root";
+        String rootConfig = "root." + config;
+        String rootFolder = "root.folder.";
+        String rootConfigFolder = "root." + config + ".folder.";
+        Set<Entry<Object, Object>> entrySet = rootProperties.entrySet();
+        for (Iterator iterator = entrySet.iterator(); iterator.hasNext();) {
+            Entry<String, String> entry = (Entry<String, String>) iterator.next();
+            String key = entry.getKey().trim();
+            // root=
+            if (root.equals(key)) {
+                handleRootEntry(target, entry.getValue(), null);
+            }
+            // root.xxx=
+            else if (rootConfig.equals(key)) {
+                handleRootEntry(target, entry.getValue(), null);
+            }
+            // root.folder.yyy=
+            else if (key.startsWith(rootFolder)) {
+                String subFolder = entry.getKey().substring((rootFolder.length()));
+                handleRootEntry(target, entry.getValue(), subFolder);
+            }
+            // root.xxx.folder.yyy=
+            else if (key.startsWith(rootConfigFolder)) {
+                String subFolder = entry.getKey().substring((rootConfigFolder.length()));
+                handleRootEntry(target, entry.getValue(), subFolder);
+            } else {
+                getLog().debug("ignoring property " + entry.getKey() + "=" + entry.getValue());
             }
         }
     }
 
     /**
      * @param rootFileEntry
-     *            files and directories seperated by semicolons, the syntax is:
+     *            files and directories separated by semicolons, the syntax is:
      *            <ul>
      *            <li>for a relative file: file:license.html,...</li>
      *            <li>for a absolute file: absolute:file:/eclipse/about.html,...</li>
@@ -427,17 +421,15 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
         setPropertyIfNotNull(props, "id", productConfiguration.getId());
 
         File eclipseproduct = new File(target, ".eclipseproduct");
-        try {
-            FileOutputStream fos = new FileOutputStream(eclipseproduct);
+        try (FileOutputStream fos = new FileOutputStream(eclipseproduct)) {
             props.store(fos, "Eclipse Product File");
-            fos.close();
         } catch (IOException e) {
             throw new MojoExecutionException("Error creating .eclipseproduct file.", e);
         }
     }
 
-    private void generateConfigIni(TargetEnvironment environment, File target) throws MojoExecutionException,
-            MojoFailureException {
+    private void generateConfigIni(TargetEnvironment environment, File target)
+			throws MojoExecutionException {
         getLog().debug("Generating config.ini");
         Properties props = new Properties();
         String id = productConfiguration.getProduct();
@@ -455,27 +447,23 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
         configsFolder.mkdirs();
 
         File configIni = new File(configsFolder, "config.ini");
-        FileOutputStream fos = null;
-        try {
-            fos = new FileOutputStream(configIni);
+        try (FileOutputStream fos = new FileOutputStream(configIni)) {
             props.store(fos, "Product Runtime Configuration File");
         } catch (IOException e) {
             throw new MojoExecutionException("Error creating .eclipseproduct file.", e);
-        } finally {
-            IOUtil.close(fos);
         }
     }
 
-    private void generateOSGiBundles(Properties props, TargetEnvironment environment) throws MojoFailureException {
+	private void generateOSGiBundles(Properties props, TargetEnvironment environment) {
         Map<String, BundleConfiguration> bundlesToStart = productConfiguration.getPluginConfiguration();
 
         if (bundlesToStart == null) {
             bundlesToStart = new HashMap<>();
 
             // This is the wellknown set of bundles for Eclipse based application for 3.3 and 3.4 without p2
-            bundlesToStart.put("org.eclipse.equinox.common", // 
+            bundlesToStart.put("org.eclipse.equinox.common", //
                     new BundleConfiguration("org.eclipse.equinox.common", 2, true));
-            bundlesToStart.put("org.eclipse.core.runtime", // 
+            bundlesToStart.put("org.eclipse.core.runtime", //
                     new BundleConfiguration("org.eclipse.core.runtime", -1, true));
         }
 
@@ -529,8 +517,8 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
         return bundles;
     }
 
-    private void copyExecutable(TargetEnvironment environment, File target) throws MojoExecutionException,
-            MojoFailureException {
+    private void copyExecutable(TargetEnvironment environment, File target)
+			throws MojoExecutionException {
         getLog().debug("Creating launcher");
 
         ArtifactDescriptor artifact = getDependencyArtifacts().getArtifact(ArtifactType.TYPE_ECLIPSE_FEATURE,
@@ -540,7 +528,7 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
             throw new MojoExecutionException("Native launcher is not found for " + environment.toString());
         }
 
-        File location = artifact.getLocation();
+        File location = artifact.getLocation(true);
 
         String os = environment.getOs();
         String ws = environment.getWs();
@@ -564,7 +552,7 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
         // make launcher executable
         try {
             getLog().debug("running chmod");
-            ArchiveEntryUtils.chmod(launcher, 0755, logger);
+            ArchiveEntryUtils.chmod(launcher, 0755);
         } catch (ArchiverException e) {
             throw new MojoExecutionException("Unable to make launcher being executable", e);
         }
@@ -637,6 +625,17 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
                 }
             } else if (PlatformPropertiesUtils.OS_LINUX.equals(os)) {
                 String icon = productConfiguration.getLinuxIcon();
+                if (icon != null) {
+                    try {
+                        File sourceXPM = new File(project.getBasedir(), removeFirstSegment(icon));
+                        File targetXPM = new File(launcher.getParentFile(), "icon.xpm");
+						Files.copy(sourceXPM.toPath(), targetXPM.toPath());
+                    } catch (IOException e) {
+                        throw new MojoExecutionException("Unable to create ico.xpm", e);
+                    }
+                }
+            } else if (PlatformPropertiesUtils.OS_FREEBSD.equals(os)) {
+                String icon = productConfiguration.getFreeBSDIcon();
                 if (icon != null) {
                     try {
                         File sourceXPM = new File(project.getBasedir(), removeFirstSegment(icon));
@@ -661,10 +660,11 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
                         // Modify eclipse.ini
                         File iniFile = new File(osxEclipseApp + "/Contents/MacOS/eclipse.ini");
                         if (iniFile.exists() && iniFile.canWrite()) {
-                            StringBuffer buf = readFileToString(iniFile);
+							StringBuilder buf = new StringBuilder(
+									Files.readString(iniFile.toPath(), StandardCharsets.UTF_8));
                             int pos = buf.indexOf("Eclipse.icns");
                             buf.replace(pos, pos + 12, source.getName());
-                            writeStringToFile(iniFile, buf.toString());
+							Files.writeString(iniFile.toPath(), buf.toString());
                         }
                     } catch (Exception e) {
                         throw new MojoExecutionException("Unable to create macosx icon", e);
@@ -674,53 +674,26 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
         }
     }
 
-    private void writeStringToFile(File iniFile, String string) throws IOException {
-        OutputStream os = new BufferedOutputStream(new FileOutputStream(iniFile));
-        try {
-            IOUtil.copy(string, os);
-        } finally {
-            IOUtil.close(os);
-        }
-    }
-
-    private StringBuffer readFileToString(File iniFile) throws IOException {
-        InputStream is = new BufferedInputStream(new FileInputStream(iniFile));
-        try {
-            StringWriter buffer = new StringWriter();
-
-            IOUtil.copy(is, buffer, "UTF-8");
-
-            return buffer.getBuffer();
-        } finally {
-            IOUtil.close(is);
-        }
-    }
-
     private void unzipDirectory(File source, String sourceRelPath, File target, String excludes) throws IOException {
         FileLocker locker = fileLockService.getFileLocker(source);
         locker.lock();
-        try {
-            ZipFile zip = new ZipFile(source);
-            try {
-                Enumeration<? extends ZipEntry> entries = zip.entries();
+        try (ZipFile zip = new ZipFile(source)) {
+            Enumeration<? extends ZipEntry> entries = zip.entries();
 
-                while (entries.hasMoreElements()) {
-                    ZipEntry entry = entries.nextElement();
+            while (entries.hasMoreElements()) {
+                ZipEntry entry = entries.nextElement();
 
-                    if (entry.isDirectory()) {
-                        continue;
-                    }
+                if (entry.isDirectory()) {
+                    continue;
+                }
 
-                    String name = entry.getName();
+                String name = entry.getName();
 
-                    if (name.startsWith(sourceRelPath) && !SelectorUtils.matchPath(excludes, name)) {
-                        File targetFile = new File(target, name.substring(sourceRelPath.length()));
-                        targetFile.getParentFile().mkdirs();
-                        FileUtils.copyStreamToFile(new RawInputStreamFacade(zip.getInputStream(entry)), targetFile);
-                    }
+                if (name.startsWith(sourceRelPath) && !SelectorUtils.matchPath(excludes, name)) {
+                    File targetFile = new File(target, name.substring(sourceRelPath.length()));
+                    targetFile.getParentFile().mkdirs();
+                    FileUtils.copyStreamToFile(new RawInputStreamFacade(zip.getInputStream(entry)), targetFile);
                 }
-            } finally {
-                zip.close();
             }
         } finally {
             locker.release();
@@ -766,7 +739,8 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
         }
 
         if (PlatformPropertiesUtils.OS_LINUX.equals(os) || PlatformPropertiesUtils.OS_SOLARIS.equals(os)
-                || PlatformPropertiesUtils.OS_HPUX.equals(os) || PlatformPropertiesUtils.OS_AIX.equals(os)) {
+                || PlatformPropertiesUtils.OS_HPUX.equals(os) || PlatformPropertiesUtils.OS_AIX.equals(os)
+                || PlatformPropertiesUtils.OS_FREEBSD.equals(os)) {
             return new File(target, "launcher");
         }
 
