@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2008, 2015 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -11,7 +13,7 @@
 package org.eclipse.tycho.buildversion;
 
 import static org.hamcrest.CoreMatchers.containsString;
-import static org.junit.Assert.assertThat;
+import static org.hamcrest.MatcherAssert.assertThat;
 
 import java.io.File;
 import java.io.IOException;
@@ -148,7 +150,7 @@ public class BuildQualifierTest extends AbstractTychoMojoTestCase {
     public void testStableQualifier() throws Exception {
         File basedir = getBasedir("projects/stablebuildqualifier/basic");
 
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+		List<MavenProject> projects = getSortedProjects(basedir);
         MavenSession session = newMavenSession(projects.get(0), projects);
 
         executeMojo(session, getProject(projects, "bundle01"));
@@ -172,7 +174,7 @@ public class BuildQualifierTest extends AbstractTychoMojoTestCase {
     public void testUnparsableIncludedArtifactQualifier() throws Exception {
         File basedir = getBasedir("projects/stablebuildqualifier/unpasablequalifier");
 
-        List<MavenProject> projects = getSortedProjects(basedir, null);
+		List<MavenProject> projects = getSortedProjects(basedir);
         MavenSession session = newMavenSession(projects.get(0), projects);
 
         executeMojo(session, getProject(projects, "bundle"));
@@ -189,7 +191,7 @@ public class BuildQualifierTest extends AbstractTychoMojoTestCase {
         ReactorProject reactorProject = DefaultReactorProject.adapt(project);
 
         DefaultDependencyArtifacts dependencyArtifacts = (DefaultDependencyArtifacts) TychoProjectUtils
-                .getDependencyArtifacts(project);
+                .getDependencyArtifacts(reactorProject);
 
         // replace target platform dependencies with fake attached feature and bundle atrifacts
         ArtifactDescriptor attachedFeature = dependencyArtifacts.getArtifact(ArtifactType.TYPE_ECLIPSE_FEATURE,
