@@ -1,9 +1,11 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2015 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2008, 2019 Sonatype Inc. and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -17,8 +19,10 @@ import java.util.Arrays;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.stream.Collectors;
 
 import org.apache.maven.model.Model;
+import org.apache.maven.model.Repository;
 import org.apache.maven.model.io.xpp3.MavenXpp3Reader;
 import org.apache.maven.plugin.Mojo;
 import org.codehaus.plexus.util.ReaderFactory;
@@ -45,15 +49,12 @@ public class GeneratePomsMojoTest extends AbstractTychoMojoTestCase {
                 "generate-poms", null);
         setVariableValueToObject(generateMojo, "baseDir", baseDir);
         if (extraDirs != null) {
-            StringBuilder sb = new StringBuilder();
-            for (File dir : extraDirs) {
-                if (sb.length() > 0)
-                    sb.append(',');
-                sb.append(dir.getAbsolutePath());
-            }
-            setVariableValueToObject(generateMojo, "extraDirs", sb.toString());
+            String dirs = Arrays.stream(extraDirs).map(File::getAbsolutePath).collect(Collectors.joining(","));
+            setVariableValueToObject(generateMojo, "extraDirs", dirs);
         }
         setVariableValueToObject(generateMojo, "executionEnvironment", "J2SE-1.5"); // the default value
+        setVariableValueToObject(generateMojo, "repoURL", "https://download.eclipse.org/releases/latest/");
+        setVariableValueToObject(generateMojo, "repoID", "eclipse-latest");
         if (params != null) {
             for (Map.Entry<String, Object> param : params.entrySet()) {
                 setVariableValueToObject(generateMojo, param.getKey(), param.getValue());
@@ -168,6 +169,13 @@ public class GeneratePomsMojoTest extends AbstractTychoMojoTestCase {
         assertEquals("1.0.0", model.getVersion());
         assertEquals("pom", model.getPackaging());
 
+        List<Repository> repositories = model.getRepositories();
+        assertEquals(1, repositories.size());
+        Repository repo = repositories.get(0);
+        assertEquals("p2", repo.getLayout());
+        assertEquals("https://download.eclipse.org/releases/latest/", repo.getUrl());
+        assertEquals("eclipse-latest", repo.getId());
+
         List<String> modules = model.getModules();
         assertEquals(6, modules.size());
 
@@ -207,8 +215,7 @@ public class GeneratePomsMojoTest extends AbstractTychoMojoTestCase {
         Model aggmodel = readModel(baseDir, "p003/poma.xml");
         List<String> aggrmodules = aggmodel.getModules();
         assertEquals(5, aggrmodules.size());
-        assertEquals(Arrays.asList(new String[] { "../p001", "../p001.tests", "../p002", "../p004", "." }),
-                aggrmodules);
+        assertEquals(Arrays.asList("../p001", "../p001.tests", "../p002", "../p004", "."), aggrmodules);
 
         assertEquals("eclipse-test-plugin", readModel(baseDir, "p001.tests/pom.xml").getPackaging());
         assertEquals("eclipse-test-plugin", readModel(baseDir, "p004/pom.xml").getPackaging());
