@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2011, 2017 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Guillaume Dufour - Support for release-process like Maven
@@ -19,6 +21,7 @@ import org.eclipse.tycho.versions.pom.PomFile;
 @Component(role = EclipseVersionUpdater.class, instantiationStrategy = "per-lookup")
 public class EclipseVersionUpdater extends VersionUpdater {
 
+    @Override
     protected void addVersionChange(VersionsEngine engine, PomFile pom, String osgiVersion) {
         engine.addVersionChange(new PomVersionChange(pom, osgiVersion, pom.getVersion()));
     }
