@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2008, 2016 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -30,6 +32,7 @@ import org.eclipse.tycho.versions.engine.VersionChangesDescriptor;
 import org.eclipse.tycho.versions.pom.PomFile;
 
 @Component(role = MetadataManipulator.class, hint = "eclipse-update-site")
+@Deprecated
 public class SiteXmlManipulator extends AbstractMetadataManipulator {
 
     @Override
