@@ -1,9 +1,11 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2016 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2008, 2020 Sonatype Inc. and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -12,16 +14,15 @@
 package org.eclipse.tycho.versions.pom.tests;
 
 import static org.hamcrest.CoreMatchers.is;
-import static org.junit.Assert.assertThat;
+import static org.hamcrest.MatcherAssert.assertThat;
 
 import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.UnsupportedEncodingException;
 import java.net.URL;
+import java.nio.charset.StandardCharsets;
 
-import org.codehaus.plexus.util.IOUtil;
 import org.eclipse.tycho.versions.pom.PomFile;
 import org.junit.Assert;
 import org.junit.Test;
@@ -150,11 +151,8 @@ public class MutablePomFileTest {
     }
 
     private PomFile getPom(String path) throws IOException {
-        InputStream is = getClass().getResourceAsStream(path);
-        try {
+        try (InputStream is = getClass().getResourceAsStream(path)) {
             return PomFile.read(is, true);
-        } finally {
-            IOUtil.close(is);
         }
     }
 
@@ -169,17 +167,14 @@ public class MutablePomFileTest {
 
     private static byte[] toByteArray(String path) throws IOException {
         byte expected[];
-        InputStream is = MutablePomFileTest.class.getResourceAsStream(path);
-        try {
-            expected = IOUtil.toByteArray(is);
-        } finally {
-            IOUtil.close(is);
+        try (InputStream is = MutablePomFileTest.class.getResourceAsStream(path)) {
+            expected = is.readAllBytes();
         }
         return expected;
     }
 
-    private static String toAsciiStringWithoutLineFeeds(byte[] bytes) throws UnsupportedEncodingException {
-        return new String(bytes, "ascii").replace("\n", "").replace("\r", "");
+    private static String toAsciiStringWithoutLineFeeds(byte[] bytes) {
+        return new String(bytes, StandardCharsets.US_ASCII).replace("\n", "").replace("\r", "");
     }
 
 }
