@@ -46,28 +46,34 @@ import org.codehaus.plexus.util.FileUtils;
 
 /**
  * Base class for bundling sources into a jar archive.
- * 
+ *
  * @version $Id: AbstractSourceJarMojo.java 763422 2009-04-08 21:59:54Z pgier $
  * @since 2.0.3
  */
 public abstract class AbstractSourceJarMojo extends AbstractMojo {
+
+    /**
+     * Lock object to ensure thread-safety
+     */
+    private static final Object LOCK = new Object();
+
     private static final String[] DEFAULT_INCLUDES = new String[] { "**/*" };
 
     private static final String[] DEFAULT_EXCLUDES = new String[] {};
 
     /**
-     * List of files to include. Specified as fileset patterns which are relative to the input directory
-     * whose contents is being packaged into the JAR.
-     * 
+     * List of files to include. Specified as fileset patterns which are relative to the input
+     * directory whose contents is being packaged into the JAR.
+     *
      * @since 2.1
      */
     @Parameter
     private String[] includes;
 
     /**
-     * List of files to exclude. Specified as fileset patterns which are relative to the input directory
-     * whose contents is being packaged into the JAR.
-     * 
+     * List of files to exclude. Specified as fileset patterns which are relative to the input
+     * directory whose contents is being packaged into the JAR.
+     *
      * @since 2.1
      */
     @Parameter
@@ -76,7 +82,7 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
     /**
      * Exclude commonly excluded files such as SCM configuration. These are defined in the plexus
      * FileUtils.getDefaultExcludes()
-     * 
+     *
      * @since 2.1
      */
     @Parameter(defaultValue = "true")
@@ -102,25 +108,27 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
 
     /**
      * The archive configuration to use. See
-     * <a href="http://maven.apache.org/shared/maven-archiver/index.html">Maven Archiver Reference</a>.
-     * 
+     * <a href="http://maven.apache.org/shared/maven-archiver/index.html">Maven Archiver
+     * Reference</a>.
+     *
      * @since 2.1
      */
     @Parameter
     private MavenArchiveConfiguration archive = new MavenArchiveConfiguration();
 
     /**
-     * Path to the default MANIFEST file to use. It will be used if <code>useDefaultManifestFile</code>
-     * is set to <code>true</code>.
-     * 
+     * Path to the default MANIFEST file to use. It will be used if
+     * <code>useDefaultManifestFile</code> is set to <code>true</code>.
+     *
      * @since 2.1
      */
     @Parameter(defaultValue = "${project.build.outputDirectory}/META-INF/MANIFEST.MF", required = true, readonly = true)
     private File defaultManifestFile;
 
     /**
-     * Set this to <code>true</code> to enable the use of the <code>defaultManifestFile</code>. <br/>
-     * 
+     * Set this to <code>true</code> to enable the use of the <code>defaultManifestFile</code>.
+     * <br/>
+     *
      * @since 2.1
      */
     @Parameter(defaultValue = "false")
@@ -128,16 +136,16 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
 
     /**
      * Specifies whether or not to attach the artifact to the project
-     * 
+     *
      */
     @Parameter(property = "attach", defaultValue = "true")
     private boolean attach;
 
     /**
      * Specifies whether or not to exclude resources from the sources-jar. This can be convenient if
-     * your project includes large resources, such as images, and you don't want to include them in the
-     * sources-jar.
-     * 
+     * your project includes large resources, such as images, and you don't want to include them in
+     * the sources-jar.
+     *
      * @since 2.0.4
      */
     @Parameter(property = "source.excludeResources", defaultValue = "false")
@@ -145,7 +153,7 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
 
     /**
      * Specifies whether or not to include the POM file in the sources-jar.
-     * 
+     *
      * @since 2.1
      */
     @Parameter(property = "source.includePom", defaultValue = "false")
@@ -164,8 +172,8 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
     protected File outputDirectory;
 
     /**
-     * The filename to be used for the generated archive file. For the source:jar goal, "-sources" is
-     * appended to this filename. For the source:test-jar goal, "-test-sources" is appended.
+     * The filename to be used for the generated archive file. For the source:jar goal, "-sources"
+     * is appended to this filename. For the source:test-jar goal, "-test-sources" is appended.
      */
     @Parameter(property = "project.build.finalName")
     protected String finalName;
@@ -177,10 +185,10 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
     protected List reactorProjects;
 
     /**
-     * NOT SUPPORTED. Whether creating the archive should be forced. If set to true, the jar will always
-     * be created. If set to false, the jar will only be created when the sources are newer than the
-     * jar.
-     * 
+     * NOT SUPPORTED. Whether creating the archive should be forced. If set to true, the jar will
+     * always be created. If set to false, the jar will only be created when the sources are newer
+     * than the jar.
+     *
      * @since 2.1
      */
     @Parameter(property = "source.forceCreation", defaultValue = "false")
@@ -193,7 +201,9 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
     /** {@inheritDoc} */
     @Override
     public void execute() throws MojoExecutionException {
-        packageSources(project);
+        synchronized (LOCK) {
+            packageSources(project);
+        }
     }
 
     // ----------------------------------------------------------------------
@@ -201,7 +211,7 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
     // ----------------------------------------------------------------------
 
     /**
-     * @return the wanted classifier, ie <code>sources</code> or <code>test-sources</code>
+     * @return the wanted classifier, i.e. <code>sources</code> or <code>test-sources</code>
      */
     protected abstract String getClassifier();
 
@@ -217,7 +227,7 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
      *            not null
      * @return the compile or test resources
      */
-    protected abstract List getResources(MavenProject p) throws MojoExecutionException;
+    protected abstract List<Resource> getResources(MavenProject p) throws MojoExecutionException;
 
     protected void packageSources(MavenProject p) throws MojoExecutionException {
         if (isRelevantProject(p)) {
@@ -267,13 +277,7 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
                 archive.setForced(true);
             }
             archiver.createArchive(session, project, archive);
-        } catch (IOException e) {
-            throw new MojoExecutionException("Error creating source archive: " + e.getMessage(), e);
-        } catch (ArchiverException e) {
-            throw new MojoExecutionException("Error creating source archive: " + e.getMessage(), e);
-        } catch (DependencyResolutionRequiredException e) {
-            throw new MojoExecutionException("Error creating source archive: " + e.getMessage(), e);
-        } catch (ManifestException e) {
+        } catch (IOException | ArchiverException | DependencyResolutionRequiredException | ManifestException e) {
             throw new MojoExecutionException("Error creating source archive: " + e.getMessage(), e);
         }
 
@@ -297,9 +301,7 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
             }
         }
 
-        for (Iterator i = getSources(p).iterator(); i.hasNext();) {
-            Resource resource = (Resource) i.next();
-
+        for (Resource resource : getSources(p)) {
             File sourceDirectory = new File(resource.getDirectory());
             if (sourceDirectory.exists()) {
                 String path = resource.getTargetPath();
@@ -315,8 +317,7 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
         }
 
         //MAPI: this should be taken from the resources plugin
-        for (Iterator i = getResources(p).iterator(); i.hasNext();) {
-            Resource resource = (Resource) i.next();
+        for (Resource resource : getResources(p)) {
 
             File sourceDirectory = new File(resource.getDirectory());
 
@@ -324,11 +325,11 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
                 continue;
             }
 
-            List resourceIncludes = resource.getIncludes();
+            List<String> resourceIncludes = resource.getIncludes();
 
             String[] combinedIncludes = getCombinedIncludes(resourceIncludes);
 
-            List resourceExcludes = resource.getExcludes();
+            List<String> resourceExcludes = resource.getExcludes();
 
             String[] combinedExcludes = getCombinedExcludes(resourceExcludes);
 
@@ -349,11 +350,9 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
         archiver.setArchiver(jarArchiver);
 
         if (project.getBuild() != null) {
-            List resources = project.getBuild().getResources();
-
-            for (Iterator i = resources.iterator(); i.hasNext();) {
-                Resource r = (Resource) i.next();
+            List<Resource> resources = project.getBuild().getResources();
 
+            for (Resource r : resources) {
                 if (r.getDirectory().endsWith("maven-shared-archive-resources")) {
                     addDirectory(archiver.getArchiver(), new File(r.getDirectory()), getCombinedIncludes(null),
                             getCombinedExcludes(null));
@@ -401,14 +400,15 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
     }
 
     /**
-     * Combines the includes parameter and additional includes. Defaults to {@link #DEFAULT_INCLUDES} If
-     * the additionalIncludes parameter is null, it is not added to the combined includes.
-     * 
+     * Combines the includes parameter and additional includes. Defaults to
+     * {@link #DEFAULT_INCLUDES} If the additionalIncludes parameter is null, it is not added to the
+     * combined includes.
+     *
      * @param additionalIncludes
      *            The includes specified in the pom resources section
      * @return The combined array of includes.
      */
-    private String[] getCombinedIncludes(List additionalIncludes) {
+    private String[] getCombinedIncludes(List<String> additionalIncludes) {
         ArrayList<String> combinedIncludes = new ArrayList<>();
 
         if (includes != null && includes.length > 0) {
@@ -420,7 +420,7 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
         }
 
         // If there are no other includes, use the default.
-        if (combinedIncludes.size() == 0) {
+        if (combinedIncludes.isEmpty()) {
             combinedIncludes.addAll(Arrays.asList(DEFAULT_INCLUDES));
         }
 
@@ -428,15 +428,15 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
     }
 
     /**
-     * Combines the user parameter {@link #excludes}, the default excludes from plexus FileUtils, and
-     * the contents of the parameter addionalExcludes.
-     * 
+     * Combines the user parameter {@link #excludes}, the default excludes from plexus FileUtils,
+     * and the contents of the parameter addionalExcludes.
+     *
      * @param additionalExcludes
      *            Additional excludes to add to the array
      * @return The combined list of excludes.
      */
 
-    private String[] getCombinedExcludes(List additionalExcludes) {
+    private String[] getCombinedExcludes(List<String> additionalExcludes) {
         ArrayList<String> combinedExcludes = new ArrayList<>();
 
         if (useDefaultExcludes) {
@@ -451,7 +451,7 @@ public abstract class AbstractSourceJarMojo extends AbstractMojo {
             combinedExcludes.addAll(additionalExcludes);
         }
 
-        if (combinedExcludes.size() == 0) {
+        if (combinedExcludes.isEmpty()) {
             combinedExcludes.addAll(Arrays.asList(DEFAULT_EXCLUDES));
         }
 
