@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2008, 2018 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -17,8 +19,6 @@ import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
 import java.net.URL;
-import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collections;
 import java.util.Enumeration;
 import java.util.HashMap;
@@ -27,8 +27,10 @@ import java.util.Map;
 import java.util.Properties;
 import java.util.Set;
 
+import org.apache.maven.plugin.failsafe.util.FailsafeSummaryXmlUtils;
 import org.apache.maven.plugin.surefire.StartupReportConfiguration;
 import org.apache.maven.plugin.surefire.log.api.PrintStreamLogger;
+import org.apache.maven.plugin.surefire.report.ConsoleReporter;
 import org.apache.maven.plugin.surefire.report.DefaultReporterFactory;
 import org.apache.maven.surefire.booter.BooterConstants;
 import org.apache.maven.surefire.booter.ClassLoaderConfiguration;
@@ -53,6 +55,7 @@ import org.eclipse.core.runtime.Status;
 import org.eclipse.osgi.service.resolver.ResolverError;
 import org.osgi.framework.Bundle;
 import org.osgi.framework.BundleException;
+import org.osgi.framework.Constants;
 
 public class OsgiSurefireBooter {
     private static final String XSD = "https://maven.apache.org/surefire/maven-surefire-plugin/xsd/surefire-test-report.xsd";
@@ -74,9 +77,24 @@ public class OsgiSurefireBooter {
         for (String key : testProps.stringPropertyNames()) {
             propertiesMap.put(key, testProps.getProperty(key));
         }
+        if (Boolean.parseBoolean(testProps.getProperty("printBundles"))) {
+            System.out.println("====== Installed Bundles ========");
+            Bundle fwbundle = getBundle(Constants.SYSTEM_BUNDLE_SYMBOLICNAME);
+            Bundle[] bundles = fwbundle.getBundleContext().getBundles();
+            for (Bundle bundle : bundles) {
+                System.out.println("[" + bundle.getBundleId() + "][" + bundle.getState() + "] "
+                        + bundle.getSymbolicName() + " (" + bundle.getVersion() + ")");
+            }
+            System.out.println("=================================");
+        }
         PropertiesWrapper wrapper = new PropertiesWrapper(propertiesMap);
         List<String> suiteXmlFiles = wrapper.getStringList(BooterConstants.TEST_SUITE_XML_FILES);
 
+        String timeoutParameter = getArgumentValue(args, "-timeout");
+        if (timeoutParameter != null) {
+            DumpStackTracesTimer.startStackDumpTimeoutTimer(timeoutParameter);
+        }
+
         boolean forkRequested = true;
         boolean inForkedVM = true;
         boolean useSystemClassloader = false;
@@ -88,35 +106,43 @@ public class OsgiSurefireBooter {
         ClasspathConfiguration classPathConfig = new ClasspathConfiguration(false, false);
         StartupConfiguration startupConfiguration = new StartupConfiguration(provider, classPathConfig,
                 new ClassLoaderConfiguration(useSystemClassloader, useManifestOnlyJar), forkRequested, inForkedVM);
-        // TODO dir scanning with no includes done here (done in TestMojo already) 
+        // TODO dir scanning with no includes done here (done in TestMojo already)
         // but without dirScannerParams we get an NPE accessing runOrder
         DirectoryScannerParameters dirScannerParams = new DirectoryScannerParameters(testClassesDir,
                 Collections.<String> emptyList(), Collections.<String> emptyList(), Collections.<String> emptyList(),
                 failIfNoTests, runOrder);
         ReporterConfiguration reporterConfig = new ReporterConfiguration(reportsDir, trimStackTrace);
         TestRequest testRequest = new TestRequest(suiteXmlFiles, testClassesDir,
-                TestListResolver.getEmptyTestListResolver());
+                TestListResolver.getEmptyTestListResolver(), rerunFailingTestsCount);
         ProviderConfiguration providerConfiguration = new ProviderConfiguration(dirScannerParams,
                 new RunOrderParameters(runOrder, null), failIfNoTests, reporterConfig, null, testRequest,
                 extractProviderProperties(testProps), null, false, Collections.<CommandLineOption> emptyList(),
                 skipAfterFailureCount, Shutdown.DEFAULT, 30);
         StartupReportConfiguration startupReportConfig = new StartupReportConfiguration(useFile, printSummary,
-                StartupReportConfiguration.PLAIN_REPORT_FORMAT, redirectTestOutputToFile, disableXmlReport, reportsDir,
-                trimStackTrace, null, new File(reportsDir, "TESTHASH"), false, rerunFailingTestsCount, XSD, null);
+                ConsoleReporter.PLAIN, redirectTestOutputToFile, disableXmlReport, reportsDir, trimStackTrace, null,
+                new File(reportsDir, "TESTHASH"), false, rerunFailingTestsCount, XSD, null, false);
         ReporterFactory reporterFactory = new DefaultReporterFactory(startupReportConfig,
                 new PrintStreamLogger(startupReportConfig.getOriginalSystemOut()));
-        // API indicates we should use testClassLoader below but surefire also tries 
+        // API indicates we should use testClassLoader below but surefire also tries
         // to load surefire classes using this classloader
         RunResult result = ProviderFactory.invokeProvider(null, createCombinedClassLoader(testPlugin), reporterFactory,
                 providerConfiguration, false, startupConfiguration, true);
+        String failsafe = testProps.getProperty("failsafe");
+        if (failsafe != null && !failsafe.isBlank()) {
+            FailsafeSummaryXmlUtils.writeSummary(result, new File(failsafe), false);
+        }
         // counter-intuitive, but null indicates OK here
         return result.getFailsafeCode() == null ? 0 : result.getFailsafeCode();
     }
 
     private static ClassLoader createCombinedClassLoader(String testPlugin) throws BundleException {
         ClassLoader testClassLoader = getBundleClassLoader(testPlugin);
+        ClassLoader contextClassLoader = Thread.currentThread().getContextClassLoader();
         ClassLoader surefireClassLoader = ForkedBooter.class.getClassLoader();
-        return new CombinedClassLoader(testClassLoader, surefireClassLoader);
+        return new CombinedClassLoader(testClassLoader, surefireClassLoader,
+                // Not used contextClassLoader directly because it's a ContextFinder
+                // which not work with tycho sufire osgibooster bundle
+                new ContextFinderWithoutTychoBundle(contextClassLoader.getParent()));
     }
 
     /*
@@ -133,13 +159,7 @@ public class OsgiSurefireBooter {
     }
 
     private static File getTestProperties(String[] args) throws CoreException {
-        String arg = null;
-        for (int i = 0; i < args.length; i++) {
-            if ("-testproperties".equals(args[i].toLowerCase())) {
-                arg = args[i + 1];
-                break;
-            }
-        }
+        String arg = getArgumentValue(args, "-testproperties");
         if (arg != null) {
             File file = new File(arg);
             if (file.canRead()) {
@@ -151,12 +171,15 @@ public class OsgiSurefireBooter {
                 null));
     }
 
-    private static List<String> getIncludesExcludes(String string) {
-        ArrayList<String> list = new ArrayList<String>();
-        if (string != null) {
-            list.addAll(Arrays.asList(string.split(",")));
+    private static String getArgumentValue(String[] args, String argumentName) {
+        String arg = null;
+        for (int i = 0; i < args.length; i++) {
+            if (argumentName.equalsIgnoreCase(args[i]) && args.length >= i + 1) {
+                arg = args[i + 1];
+                break;
+            }
         }
-        return list;
+        return arg;
     }
 
     private static Properties loadProperties(File file) throws IOException {
@@ -171,10 +194,7 @@ public class OsgiSurefireBooter {
     }
 
     private static ClassLoader getBundleClassLoader(String symbolicName) throws BundleException {
-        Bundle bundle = Activator.getBundle(symbolicName);
-        if (bundle == null) {
-            throw new RuntimeException("Bundle " + symbolicName + " is not found");
-        }
+        Bundle bundle = getBundle(symbolicName);
         try {
             bundle.start();
         } catch (BundleException ex) {
@@ -192,10 +212,17 @@ public class OsgiSurefireBooter {
             }
             throw ex;
         }
-
         return new BundleClassLoader(bundle);
     }
 
+    protected static Bundle getBundle(String symbolicName) {
+        Bundle bundle = Activator.getBundle(symbolicName);
+        if (bundle == null) {
+            throw new RuntimeException("Bundle " + symbolicName + " is not found");
+        }
+        return bundle;
+    }
+
     private static class BundleClassLoader extends ClassLoader {
         private Bundle bundle;
 
@@ -203,16 +230,25 @@ public class OsgiSurefireBooter {
             this.bundle = target;
         }
 
+        @Override
         protected Class<?> findClass(String name) throws ClassNotFoundException {
             return bundle.loadClass(name);
         }
 
+        @Override
         protected URL findResource(String name) {
             return bundle.getResource(name);
         }
 
+        @Override
         protected Enumeration<URL> findResources(String name) throws IOException {
             return bundle.getResources(name);
         }
+
+        @Override
+        public String toString() {
+            return bundle.getSymbolicName() + " [" + bundle.getVersion() + "]";
+        }
     }
+
 }
