@@ -1,21 +1,21 @@
 /*******************************************************************************
  * Copyright (c) 2008, 2011 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
  *******************************************************************************/
 package org.eclipse.tycho.surefire.osgibooter;
 
-import org.eclipse.core.runtime.IPlatformRunnable;
 import org.eclipse.core.runtime.Platform;
 import org.eclipse.equinox.app.IApplication;
 import org.eclipse.equinox.app.IApplicationContext;
 
-@SuppressWarnings("deprecation")
 public class UITestApplication extends AbstractUITestApplication implements IApplication {
 
     private IApplicationContext fContext;
@@ -25,9 +25,7 @@ public class UITestApplication extends AbstractUITestApplication implements IApp
 
     @Override
     protected void runApplication(Object application, String[] args) throws Exception {
-        if (application instanceof IPlatformRunnable) {
-            ((IPlatformRunnable) application).run(args);
-        } else if (application instanceof IApplication) {
+        if (application instanceof IApplication) {
             ((IApplication) application).start(fContext);
         }
     }
