@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2010, 2017 SAP SE and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     SAP SE - initial API and implementation
@@ -12,10 +14,10 @@
 package org.eclipse.tycho.surefire;
 
 import static org.hamcrest.CoreMatchers.containsString;
+import static org.hamcrest.MatcherAssert.assertThat;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertThat;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
@@ -39,7 +41,7 @@ public class TestMojoTest {
 
     @Test
     public void testSplitArgLineNull() throws Exception {
-        TestMojo testMojo = new TestMojo();
+        AbstractTestMojo testMojo = new TestPluginMojo();
         String[] parts = testMojo.splitArgLine(null);
         assertNotNull(parts);
         assertEquals(0, parts.length);
@@ -47,7 +49,7 @@ public class TestMojoTest {
 
     @Test
     public void testSplitArgLineMultipleArgs() throws Exception {
-        TestMojo testMojo = new TestMojo();
+        AbstractTestMojo testMojo = new TestPluginMojo();
         String[] parts = testMojo.splitArgLine(" -Dfoo=bar -Dkey2=value2 \"-Dkey3=spacy value\"");
         assertEquals(3, parts.length);
         assertEquals("-Dfoo=bar", parts[0]);
@@ -57,7 +59,7 @@ public class TestMojoTest {
 
     @Test
     public void testSplitArgLineUnbalancedQuotes() throws Exception {
-        TestMojo testMojo = new TestMojo();
+        AbstractTestMojo testMojo = new TestPluginMojo();
         try {
             testMojo.splitArgLine("\"'missing closing double-quote'");
             fail("unreachable code");
@@ -69,7 +71,7 @@ public class TestMojoTest {
     @Test
     public void testAddProgramArgsWithSpaces() throws Exception {
         EquinoxLaunchConfiguration cli = createEquinoxConfiguration();
-        TestMojo testMojo = new TestMojo();
+        AbstractTestMojo testMojo = new TestPluginMojo();
         testMojo.addProgramArgs(cli, "-data", "/path with spaces ");
         assertEquals(2, cli.getProgramArguments().length);
         assertEquals("-data", cli.getProgramArguments()[0]);
@@ -79,7 +81,7 @@ public class TestMojoTest {
     @Test
     public void testAddProgramArgsNullArg() throws Exception {
         EquinoxLaunchConfiguration cli = createEquinoxConfiguration();
-        TestMojo testMojo = new TestMojo();
+        AbstractTestMojo testMojo = new TestPluginMojo();
         // null arg must be ignored
         testMojo.addProgramArgs(cli, "-data", null);
         assertEquals(1, cli.getProgramArguments().length);
@@ -87,27 +89,27 @@ public class TestMojoTest {
 
     @Test
     public void testShouldSkipWithNoValueSet() {
-        TestMojo testMojo = new TestMojo();
+        AbstractTestMojo testMojo = new TestPluginMojo();
         assertFalse(testMojo.shouldSkip());
     }
 
     @Test
     public void testShouldSkipWithSkipTestsSetToTrue() throws Exception {
-        TestMojo testMojo = new TestMojo();
+        AbstractTestMojo testMojo = new TestPluginMojo();
         setParameter(testMojo, "skipTests", Boolean.TRUE);
         assertTrue(testMojo.shouldSkip());
     }
 
     @Test
     public void testShouldSkipWithMavenTestSkipSetToTrue() throws Exception {
-        TestMojo testMojo = new TestMojo();
+        AbstractTestMojo testMojo = new TestPluginMojo();
         setParameter(testMojo, "skip", Boolean.TRUE);
         assertTrue(testMojo.shouldSkip());
     }
 
     @Test
     public void testShouldSkipThatSkipTestsWillBePrefered() throws Exception {
-        TestMojo testMojo = new TestMojo();
+        AbstractTestMojo testMojo = new TestPluginMojo();
         setParameter(testMojo, "skip", Boolean.FALSE);
         setParameter(testMojo, "skipTests", Boolean.TRUE);
         assertTrue(testMojo.shouldSkip());
@@ -115,7 +117,7 @@ public class TestMojoTest {
 
     @Test
     public void testShouldSkipWithSkipExeSetToTrue() throws Exception {
-        TestMojo testMojo = new TestMojo();
+        AbstractTestMojo testMojo = new TestPluginMojo();
         setParameter(testMojo, "skipExec", Boolean.TRUE);
         assertTrue(testMojo.shouldSkip());
     }
@@ -143,7 +145,7 @@ public class TestMojoTest {
 
     @Test
     public void testParallelModeMissingThreadCountParameter() throws Exception {
-        TestMojo testMojo = new TestMojo();
+        AbstractTestMojo testMojo = new TestPluginMojo();
         setParameter(testMojo, "parallel", ParallelMode.both);
         try {
             testMojo.getMergedProviderProperties();
@@ -155,7 +157,7 @@ public class TestMojoTest {
 
     @Test
     public void testParallelModeThreadCountSetTo1() throws Exception {
-        TestMojo testMojo = new TestMojo();
+        AbstractTestMojo testMojo = new TestPluginMojo();
         setParameter(testMojo, "parallel", ParallelMode.both);
         setParameter(testMojo, "threadCount", 1);
         try {
@@ -168,7 +170,7 @@ public class TestMojoTest {
 
     @Test
     public void testParallelModeWithThreadCountSet() throws Exception {
-        TestMojo testMojo = new TestMojo();
+        AbstractTestMojo testMojo = new TestPluginMojo();
         setParameter(testMojo, "parallel", ParallelMode.both);
         setParameter(testMojo, "threadCount", 2);
         Properties providerProperties = testMojo.getMergedProviderProperties();
@@ -178,7 +180,7 @@ public class TestMojoTest {
 
     @Test
     public void testParallelModeWithUseUnlimitedThreads() throws Exception {
-        TestMojo testMojo = new TestMojo();
+        AbstractTestMojo testMojo = new TestPluginMojo();
         setParameter(testMojo, "parallel", ParallelMode.both);
         setParameter(testMojo, "useUnlimitedThreads", Boolean.TRUE);
         Properties providerProperties = testMojo.getMergedProviderProperties();
@@ -188,7 +190,7 @@ public class TestMojoTest {
 
     @Test(expected = MojoExecutionException.class)
     public void testParallelModeWithPerCoreThreadCountMissingCount() throws Exception {
-        TestMojo testMojo = new TestMojo();
+        AbstractTestMojo testMojo = new TestPluginMojo();
         setParameter(testMojo, "parallel", ParallelMode.both);
         setParameter(testMojo, "perCoreThreadCount", true);
         testMojo.getMergedProviderProperties();
@@ -196,7 +198,7 @@ public class TestMojoTest {
 
     @Test
     public void testParallelModeWithPerCoreThreadCount() throws Exception {
-        TestMojo testMojo = new TestMojo();
+        AbstractTestMojo testMojo = new TestPluginMojo();
         setParameter(testMojo, "parallel", ParallelMode.both);
         setParameter(testMojo, "perCoreThreadCount", true);
         setParameter(testMojo, "threadCount", 1);
@@ -209,7 +211,7 @@ public class TestMojoTest {
     public ScanResult createDirectoryAndScanForTests(List<String> includes, List<String> excludes) throws Exception {
         File directory = null;
         try {
-            TestMojo testMojo = new TestMojo();
+            AbstractTestMojo testMojo = new TestPluginMojo();
             directory = Files.createTempDirectory(this.getClass().getName()).toFile();
             File aTestFile = new File(directory, "ATest.class");
             aTestFile.createNewFile();
